/*
  clear_counters.c
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "m3_WebClient.h"
#include "m3_TestClient.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static void usage ()
{
  printf ("usage: clear_counters [switchnames...]\n");
  exit (1);
}

int main (int argc, char**argv)
{
  m3_Component::initialize (printLine);
  m3_RollcallClient rollcall;
  m3_TestClient client;
  m3_WebClient web;
  char*s;
  int*switches = 0;
  int numSwitches = 0;


  if (argc == 1)
  {
    rollcall.open ();
    numSwitches = rollcall.findAddresses (&switches);
    rollcall.close ();
    printf ("%d m3 switches\n", numSwitches);
  }
  else
  {
    numSwitches = argc - 1;
    switches = (int*) malloc (sizeof (int) * numSwitches);
    if (!switches)
    {
      printf ("malloc failed\n");
      return 1;
    }
    
    for (int i = 0; i < numSwitches; i++)
    {   
      if (!(switches [i] = web.getServer (argv [i + 1])))
      {
	printf ("couldn't get IP address for %s\n", argv [i + 1]);
	usage ();
      }
    }
  }

  for (int i = 0; i < numSwitches; i++)
  {
    printf ("%d.%d.%d.%d\n", ((char*)&switches [i]) [0] & 0xff, ((char*)&switches [i]) [1]  & 0xff, ((char*)&switches [i]) [2] & 0xff, ((char*)&switches [i]) [3] & 0xff);

    if (!(s = web.post (switches [i], "/change_uc1", 2, "reboot", "on", "change", "all")))
    {
      printf ("POST failed\n");
      return 1;
    }
    while (s = web.token ())
    {
      if (!strcmp (s, "no"))
      {
	if ((s = web.token ()) && !strcmp (s, "problem"))
	  break;
      }
    }
    
    if (!s)
    {
      printf ("POST failed\n");
      return 1;
    }

    if (!client.open (switches [i]))
    {
      printf ("couldn't open test port\n");
      return 1;
    }
    if (!client.reboot ())
    {
      printf ("couldn't reboot\n");
      return 1;
    }
  }
  return 0;
}

