/*
  describe.c
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "m3_Description.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"
#include "mt_Graph.h"
#include "mt_MapFile.h"
#include "m3_XbarTable.h"
#include "m3_SwitchFile.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static void usage ()
{
  printf ("usage: describe [-c | -f | -x] [switchnames | filenames] | [.map .xbars .switches]\n");
  exit (1);
}

static void sleep (int seconds)
{
  fd_set mask;
  struct timeval timeout;
  timeout.tv_sec = seconds;
  timeout.tv_usec = 0;
  
  FD_ZERO (&mask);
  
  select (0, &mask, 0, 0, &timeout);
}

int main (int argc, char**argv)
{
  mt_Component::initialize (printLine);
  m3_Component::initialize (printLine);
  m3_WebClient web;
  m3_Description describe;
  m3_RollcallClient rollcall;
  int continuous;
  int offline;
  int extended;
  char**switchnames, *s;
  int*switches = 0;
  int numSwitches = 0;
  mt_Graph g;
  m3_XbarTable*xbars = 0;
  m3_SwitchFile sf;
  
  continuous = (argc > 1 && !strcmp (argv[1], "-c"));
  offline = (argc > 1 && !strcmp (argv[1], "-f"));
  extended = (argc > 1 && !strcmp (argv[1], "-x"));


  if (extended)
  {
    mt_MapFile mf (argv [2], mt_File::_READ);

    if (!mf.read (&g))
    {
      m3_Component::printFormat ("couldn't read %s", argv [2]);
      usage ();
    }

    if (!(xbars = new m3_XbarTable (g.getNumSwitches ())) || 
	!xbars->fromFile (g.getHost (0), &g, argv [3]))
    {
      m3_Component::printFormat ("couldn't make xbars table");
      usage ();
    }

    if (!(numSwitches = sf.fromFile (argv [4], &switches)))
    {
      m3_Component::printFormat ("couldn't read switches file");
      usage ();
    }
  }
  else if (!offline && (continuous && argc == 2 || !continuous  && argc == 1))
  {
    rollcall.open ();
    numSwitches = rollcall.findAddresses (&switches);
    if (!numSwitches || !switches)
    {
      printf ("didn't find any switches\n");
      return 1;
    }
  }
  else
  {
    int c = (offline || continuous) ? 2 : 1;
    if (argc < c)
      usage ();
    numSwitches = argc - c;
    switchnames = argv + c;
  }

  m3_Description*descriptions [numSwitches];
  
  for (int i = 0; i < numSwitches; i++)
  {
    descriptions [i] = new m3_Description;
    if (!descriptions [i])
    {
      printf ("new failed\n");
      return 1;
    }
  }

  if (extended)
  {
    for (int i = 0; i < numSwitches; i++)
    {
      if (!descriptions [i]->examine (&web, switches [i]))
      {
	printf ("failed\n");
	usage ();
      }
    }

    if (!descriptions [0]->connect (&g, xbars, descriptions, numSwitches))
    {
      printf ("failed\n");
      usage ();
    }

    for (int i = 0; i < numSwitches; i++)
    {
      if (!descriptions [i]->print  (stdout))
      {
	printf ("failed\n");
	usage ();
      }
    }
  }
  else if (!continuous)
  {
    for (int i = 0; i < numSwitches; i++)
    {
      if (offline && !descriptions [i]->examineFile (&web, switchnames [i]) ||
	  !offline && switchnames && !descriptions [i]->examine (&web, switchnames [i]) ||
	  !offline && switches && !descriptions [i]->examine (&web, switches [i]))
      {
	printf ("failed\n");
	usage ();
      }

      if ((extended && !descriptions [0]->connect (&g, xbars, descriptions, numSwitches)) || !descriptions [i]->print (stdout))
      {
	printf ("failed\n");
	usage ();
      }
      
      printf ("\n");
    }
  }
  else
  {
    int silent = 1;
    for (;;)
    {
      for (int i = 0; i < numSwitches; i++)
      {
	if (offline && !descriptions [i]->examineFile (&web, switchnames [i]) ||
	    !offline && switchnames && !descriptions [i]->examine (&web, switchnames [i]) ||
	    !offline && switches && !descriptions [i]->examine (&web, switches [i]) ||
	    !descriptions [i]->printErrors (stdout, silent))
	{
	  printf ("failed\n");
	  usage ();
	}
      }
      silent = 0;
      sleep (1);
    }
  }
  
  return 0;
}

