/*
  disable.c
  disable invalid route ports.
  program to listen to invalidRoute traps and to disable the ports
  causing them.
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#include "m3_defines.h"
#include "m3_TestClient.h"
#include "m3_SwitchFile.h"
#include "m3_WebClient.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static void usage (char*s)
{  
  printf ("usage:\n");
  printf ("%s switches_file\n", s);
  exit (1);
}
static void fail (char*s)
{
  printf ("%s failed\n", s);
  exit (1);
}

int main (int argc, char**argv)
{
  m3_Component::initialize (printLine);
  m3_TestClient test;
  m3_SwitchFile switchFile;
  m3_WebClient web;
  char*filename = argv [1];
  int*switches = 0;
  int numSwitches = 0;
  int types [2] = {1<<15,0};
  char hostname [256];
  struct hostent*hp;
  m3_notify_message_t m;
  int length;
  struct sockaddr_in a;
  int fd;

  if (argc != 2 || !(numSwitches = switchFile.fromFile (argv [1], &switches)))
    usage (argv[0]);
	
  fd = socket (AF_INET, SOCK_DGRAM, 0);
  if (fd < 0)
  {
    perror ("failed opening datagram socket");
    usage (argv[0]);
  }
  memset ((char*)&a, 0, sizeof (a));

  a.sin_family = AF_INET;
  a.sin_addr.s_addr = INADDR_ANY;
  a.sin_port = 0;
      
  if (bind (fd, (struct sockaddr*) &a, sizeof (a)) < 0)
  {
    perror ("binding datagram socket failed");
    usage (argv[0]);
  }
      
  length = sizeof (a);
  if (getsockname (fd, (struct sockaddr*) &a, (socklen_t*) &length) < 0)
  {
    perror ("getsockname failed");
    usage (argv[0]);
  }
      
  if (gethostname (hostname, 255) < 0 || !(hp = gethostbyname (hostname)))
  {
    printf ("couldn't get our ip address\n");
    usage (argv[0]);
  }
      
  printf ("using udp port %d\n", ntohs (a.sin_port));
  for (int i = 0; i < numSwitches; i++)
  {
    if (!test.open (switches [i]) || !test.setNotify ((char*) hp->h_addr, ntohs (a.sin_port), types [0], types [1]))
    {
      printf ("Couldn't open %d.%d.%d.%d\n",
	      (unsigned) (((char*)(&switches [i])) [0] & 0xff),
	      (unsigned) (((char*)(&switches [i])) [1] & 0xff),
	      (unsigned) (((char*)(&switches [i])) [2] & 0xff),
	      (unsigned) (((char*)(&switches [i])) [3] & 0xff));
      usage (argv [0]);
    }

    for (;;)
    {
      length = sizeof (a);      
      while (recvfrom (fd, (char*) &m, sizeof (m), 0, (struct sockaddr*) &a, (socklen_t*) &length) == sizeof (m))
      {
	length = sizeof (a);
	printf ("%d.%d.%d.%d ",
		(unsigned) (((char*)(&a.sin_addr.s_addr)) [0] & 0xff),
		(unsigned) (((char*)(&a.sin_addr.s_addr)) [1] & 0xff),
		(unsigned) (((char*)(&a.sin_addr.s_addr)) [2] & 0xff),
		(unsigned) (((char*)(&a.sin_addr.s_addr)) [3] & 0xff));
	
	printf ("%s (%d) low: %d high: %d %s %d slot %d\n",
		htonl (m.type) > 0 && htonl (m.type) <= M3_NUM_TRAPS ? 
		m3_trap_names [htonl (m.type) - 1] : "?", 
		htonl (m.type),
		htonl (m.low), htonl (m.high), 
		htonl (m.partType) > 0 && htonl (m.partType) <= M3_NUM_TABLES ? 
		m3_table_names [htonl (m.partType) - 1] : "?",
		htonl (m.index),
		htonl (m.slot));
	
	char url [M3_DISPLAY_STRING_LENGTH];
	sprintf (url, "/change_xbarPort%d", htonl (m.index));
	web.post ((int) a.sin_addr.s_addr, url, 1, "port_off", "on");
      }
    }
  }    
}
