/*
  find_ethernet.c
  program to find ethernet addresses.
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#include "m3_TestClient.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static void usage (char*s)
{
  printf ("usage: %s [switchname ...]\n", s);
  exit (1);
}

int main (int argc, char**argv)
{
  m3_Component::initialize (printLine);
  int fat [6];
  char ethernet [6];

  int*addresses = 0;
  int numAddresses;
  m3_RollcallClient rollcall;
  m3_TestClient test;
  char e [6];

  if (argc < 1)
    usage (argv [0]);
  
  if (argc == 1)
  {
    if (!rollcall.open ())
    {
      printf ("open failed\n");
      return 1;
    }  
    
    numAddresses = rollcall.findAddresses (&addresses);
    rollcall.close ();
  }
  else
    numAddresses = argc - 1;
  
  for (int i = 0; i < numAddresses; i++)
  {
    int a;

    if (argc != 1)
    {
      struct hostent*hp;
      if (!(hp = gethostbyname (argv [i + 1])))
      {	
	printf ("couldn't get ip address of %s\n", argv [i + 1]);
	usage (argv [0]);
      }
      memcpy (&a, hp->h_addr, 4);
    }
    else
      a = addresses [i];
    
    if (!test.open (a) || !test.getEthernetAddress (e))
      return 1;

    printf ("%d.%d.%d.%d %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n",
	    (unsigned) (((char*)(&a)) [0] & 0xff),
	    (unsigned) (((char*)(&a)) [1] & 0xff),
	    (unsigned) (((char*)(&a)) [2] & 0xff),
	    (unsigned) (((char*)(&a)) [3] & 0xff),
	    (unsigned) e [0] & 0xff,
	    (unsigned) e [1] & 0xff,
	    (unsigned) e [2] & 0xff,
	    (unsigned) e [3] & 0xff,
	    (unsigned) e [4] & 0xff,
	    (unsigned) e [5] & 0xff);
  }
  return 0;
  exception: return 1;
}

