/*
  find_ip.c
  program to find ip addresses.
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#include "m3_TestClient.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static m3_RollcallClient*client = 0;

static void usage (char*s)
{
  printf ("usage: %s [ethernet address]\n", s);
  exit (1);
}

int get_ip_address (char ethernet [6])
{
  int*addresses = 0;
  m3_RollcallClient rollcall;
  m3_TestClient test;
  char e [6];
  
  if (!rollcall.open ())
  {
    printf ("open failed\n");
    return 1;
  }  
  
  int numAddresses = rollcall.findAddresses (&addresses);

  rollcall.close ();

  for (int i = 0; addresses && i < numAddresses; i++)
    if (test.open (addresses [i]) &&
	test.getEthernetAddress (e) &&
	!memcmp (e, ethernet, 6))
      return addresses [i];

  return 0;
}

int print_addresses ()
{
  int*addresses = 0;
  m3_RollcallClient rollcall;
  m3_TestClient test;
  char e [6];
  
  if (!rollcall.open ())
  {
    printf ("open failed\n");
    return 1;
  }  
  
  int numAddresses = rollcall.findAddresses (&addresses);

  rollcall.close ();

  printf ("found %d switches\n", numAddresses);
  
  for (int i = 0; addresses && i < numAddresses; i++)
    if (test.open (addresses [i]) &&
	test.getEthernetAddress (e))
    {
      printf ("%d.%d.%d.%d %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n",
	      (unsigned) (((char*)(&addresses [i])) [0] & 0xff),
	      (unsigned) (((char*)(&addresses [i])) [1] & 0xff),
	      (unsigned) (((char*)(&addresses [i])) [2] & 0xff),
	      (unsigned) (((char*)(&addresses [i])) [3] & 0xff),
	      (unsigned) e [0] & 0xff,
	      (unsigned) e [1] & 0xff,
	      (unsigned) e [2] & 0xff,
	      (unsigned) e [3] & 0xff,
	      (unsigned) e [4] & 0xff,
	      (unsigned) e [5] & 0xff);
    }
	
  return 0;
}

int main (int argc, char**argv)
{
  m3_Component::initialize (printLine);
  int fat [6];
  char ethernet [6];
  
  if (argc > 2)
    usage (argv[0]); 
  
  if (argc == 1)
    return print_addresses ();
  
  if (sscanf (argv [1], "%x:%x:%x:%x:%x:%x",
	      &fat [0],
	      &fat [1],
	      &fat [2],
	      &fat [3],
	      &fat [4],
	      &fat [5]) != 6 &&
      sscanf (argv [1], "%x.%x.%x.%x.%x.%x",
	      &fat [0],
	      &fat [1],
	      &fat [2],
	      &fat [3],
	      &fat [4],
	      &fat [5]) != 6)
  {
    printf ("%s is not a good ethernet address\n", argv [1]);
    usage (argv [0]);
  }
  for (int i = 0; i < 6; i++)
    ethernet [i] = fat [i];
  
  int ip = get_ip_address (ethernet);

  printf ("%d.%d.%d.%d\n",
	  (unsigned) (((char*)&ip) [0]) & 0xff,
	  (unsigned) (((char*)&ip) [1]) & 0xff,
	  (unsigned) (((char*)&ip) [2]) & 0xff,
	  (unsigned) (((char*)&ip) [3]) & 0xff);

  return 1;
}

