#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "m3_RollcallClient.h"
#include "m3_XbarFinder.h"

#include "mt_Graph.h"
#include "mt_MapFile.h"
#include "mt_Network.h"

#define die() {printf ("died on %s:%d\n", __FILE__, __LINE__); exit (1);}
#define insist(e) do { if (!(e)) {printf ("assertion failed. %s:%d (%s)", __FILE__,__LINE__, #e); goto exception;}} while (0)

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static void usage ()
{
  printf ("usage: find_xbars <unit> <mapfile> <hostname> [ [-proxy] | [switchname ...]]\n");
  exit (1);
}

class m3_FindXbar : public mt_Job
{
  public:
  m3_FindXbar (mt_Node*node, mt_Network*network);
  virtual void dump (FILE*fp);
  virtual int willWait ();
  virtual int start ();
  virtual void receive (int event, char*p, int length);
};

m3_FindXbar::m3_FindXbar (mt_Node*node, mt_Network*network) : mt_Job (node, network)
{
}
void m3_FindXbar::dump (FILE*fp)
{
}
int m3_FindXbar::willWait ()
{
  return 1;
}
int m3_FindXbar::start ()
{
  return 1;
}
void m3_FindXbar::receive (int event, char*p, int length)
{
}

int main (int argc, char*argv [])
{
  mt_Component::initialize (printLine);
  m3_Component::initialize (printLine);

  char*mapfile = argv [2];
  char*hostname = argv [3];

  m3_XbarFinder finder;
  mt_Graph graph;
  mt_Node*myself;
  mt_Network*network;
  mt_Address address;
  char networkHostname [mt_Network::HOSTNAME_LENGTH + 1];
  int hostType;
  int proxy = 0;
  
  if (argc < 4)
    usage ();

  int unit = atoi (argv [1]);
  
  mt_MapFile mf (mapfile, mt_File::_READ);

  if (!mf.read (&graph))
  {
    m3_Component::printFormat ("couldn't read %s\n", mapfile);
    return 0;
  }
  
  if (!(myself = graph.getNode (hostname)))
  {
    m3_Component::printFormat ("node \"%s\" is not in %s", hostname, mapfile);
    return 0;
  }
  
  if (!graph.computeRoutes (myself))
  {  
    m3_Component::printFormat ("couldn't compute routes");
    return 0;
  }
  
  network = mt_getNetwork ();
  insist (network);
  
  m3_FindXbar*fb;
  fb = new m3_FindXbar (myself, network);
  insist (fb);
  
  if (!network->open (fb, &address, networkHostname, &hostType, unit, 3))
    return 0;
  
  if (argc == 5 && !strcmp (argv [4], "-proxy"))
    proxy = 1;
  else if (!(argc < 5 ? finder.setNotify () : finder.setNotify (argv + 4, argc - 4)))
    return 0;
  
  if (proxy)
  {
    printf ("start proxy to port %d and hit return to continue", finder.getPort ());
    getchar ();
  }
  
  for (int i = 0; i < graph.getNumSwitches (); i++)
  {
    char a [4];
    char e [6];
    
    int xbar;
    
    printf ("%s", graph.getSwitch (i)->getName ());
    
    for (int j = 1; j < 4; j++)
    {
      mt_Route*r = graph.getSwitch (i)->getRoute (0);
      int in = myself->getIn (r);
      insist (in >= 0 && in < 16);

      if (finder.find (fb, r, a, e, &xbar, 1000 * 1000, j * 1000 * 1000))
      {
	int inPort = (xbar - 1) % 16;
	int correction = inPort - in;
	insist (correction >= 0 && correction < 16);

	printf (" %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%d:%d",
		(unsigned) (e [0] & 0xff),
		(unsigned) (e [1] & 0xff),
		(unsigned) (e [2] & 0xff),
		(unsigned) (e [3] & 0xff), 
		(unsigned) (e [4] & 0xff), 
		(unsigned) (e [5] & 0xff), 
		1 + (xbar - 1) / 16, correction);
	break;
      }
    }
    
    printf ("\n");
  }

  return 0;
  exception: return 1;
}
