/*
   hex2flash.c
   host program to spew a hex file into a vxWorks
   board over the ethernet.

   (c), 2000, Myricom, Inc.
   finucane@myri.com (David Finucane)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#include <signal.h>

#include "m3_defines.h"
#include "m3_Hex2Flash.h"

#define PERMANENT_ARGC 6
#define VARIABLE_ARGC 3

static char *usage = "hex2flash <host> <filename> [ethernet-address ip-address netmask [hex words...]]";

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static int extended [M3_FLASH_MAX_EXTENDED];
static int get_extended (int argc, char*argv [], int extended [])
{
  int n;
  int i;
  
  if (argc <= PERMANENT_ARGC)
    return 0;
  
  n = (argc - PERMANENT_ARGC);
  if (n > M3_FLASH_MAX_EXTENDED)
  {
    printf ("too many extended values\n");
    return -1;
  }
  
  for (i = 0; i < n; i++)
  {
    if (1 != sscanf (argv [PERMANENT_ARGC + i], "%i", &extended[i]))
    {
      printf ("couldn't convert %s into a hex number\n", argv [PERMANENT_ARGC + i]);
      return -1;
    }
  }
  return n;
}

int main (int argc, char *argv[])
{
  char ethernet[6];
  int fatness[6];
  int variable = 1;
  int num_extended = 0;
  char ip_address [4];
  char netmask [4];
  int i;
  
#ifdef USE_SIG_HOLD
  sighold(SIGPIPE);
#else
  sigblock(sigmask(SIGPIPE));
#endif

  m3_Component::initialize (printLine);
  
  if (argc < VARIABLE_ARGC)
  {
    printf("usage: %s \n", usage);
    return 1;
  }

  if (argc >= PERMANENT_ARGC) 
  {
    variable = 0;
    
    if ((sscanf (argv[3], "%x.%x.%x.%x.%x.%x",&fatness[0], &fatness[1], &fatness[2], &fatness[3], &fatness[4], &fatness[5]) != 6 &&
	 sscanf (argv[3], "%x:%x:%x:%x:%x:%x",&fatness[0], &fatness[1], &fatness[2],&fatness[3], &fatness[4], &fatness[5]) != 6) ||
	fatness [0] == 0 &&
	fatness [0] == fatness [1] &&
	fatness [1] && fatness [2] && 
	fatness [2] && fatness [3] && 
	fatness [3] && fatness [4] && 
	fatness [4] && fatness [5])
    {
      printf("%s bad ethernet address\n", argv[3]);
      printf("usage: %s \n", usage);
      return 1;
    }
  
    for (i = 0; i < 6; i++)
      ethernet[i] = fatness[i] & 0xFF;
  
    if (4 != sscanf (argv [4], "%d.%d.%d.%d", &fatness [0], &fatness [1], &fatness [2], &fatness [3]) ||
	fatness [0] < 0 || fatness [0] > 255 ||
	fatness [1] < 0 || fatness [1] > 255 ||
	fatness [2] < 0 || fatness [2] > 255 ||
	fatness [3] < 0 || fatness [3] > 255)
    {
      printf ("%s is not a good ip address\n", argv [4]);
      printf("usage: %s \n", usage);
      return 1;
    }
  
    for (i = 0; i < 4; i++)
      ip_address [i] = fatness [i] & 0xff;
  
    if (4 != sscanf (argv [5], "%d.%d.%d.%d", &fatness [0], &fatness [1], &fatness [2], &fatness [3]) ||
	fatness [0] < 0 || fatness [0] > 255 ||
	fatness [1] < 0 || fatness [1] > 255 ||
	fatness [2] < 0 || fatness [2] > 255 ||
	fatness [3] < 0 || fatness [3] > 255)
    {
      printf ("%s is not a good netmask\n", argv [5]);
      printf("usage: %s \n", usage);
      return 1;
    }

    for (i = 0; i < 4; i++)
      netmask [i] = fatness [i] & 0xff;

    if ((num_extended = get_extended (argc, argv, extended)) < 0)
    {
      printf ("usage: %s\n", usage);
      return 1;
    }
  }
  
  m3_Hex2Flash*flash = new m3_Hex2Flash ();
  
  return flash && flash->program (argv [1], argv[2], variable, ethernet, ip_address, netmask, num_extended, extended);
}
