/*
   hexes2ucs.c
   host program to program all ucs in a switch
   over the ethernet.

   finucane@myri.com (David Finucane)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <signal.h>

#include "m3_SwitchTest.h"
#include "m3_TestClient.h"
#include "m3_Hex2Uc.h"

#define die(s) {printf s; exit (1);}
static char *usage = "hexes2ucs <host> <hex directory>";

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static m3_SwitchTestSlot slot;

static char filename [255];
static char fileVersion [255];
static char slotType [255];
static char fileType [255];
static char slotDate [255];
static char fileDate [255];

int main (int argc, char *argv[])
{
  m3_Component::initialize (printLine);
  m3_Hex2Uc hex2uc (0);
  int port;
  char*hostarg = argv[1];
  char*dirarg = argv [2];

#ifdef USE_SIG_HOLD
  sighold (SIGPIPE);
#else
  sigblock (sigmask (SIGPIPE));
#endif

  if (argc < 3)
  {
    printf ("usage: %s \n", usage);
    return 1;
  }

  m3_SwitchTest switchTest (1, argv [1]);

  for (int i = 0; i < M3_MAX_SLOT; i++)
  {    
    if (!switchTest.getSlot (0, i + 1, &slot))
      continue;

    printf ("\n");
    
    if (!m3_TestClient::firmwareGetDate (slot.firmwareVersion, slotDate))
      die (("couldn't get firmware date date from slot %d", i));
    
    if (!m3_TestClient::firmwareGetType (slot.firmwareVersion, slotType))
      die (("couldn't get type from slot %d", i));
    
    if (!m3_TestClient::firmwareGetFilename (slot.firmwareVersion, "/", dirarg, filename))
      die (("couldn't get filename from firmware string in slot %d", i));
    
    if (!hex2uc.getFirmwareVersion (filename, fileVersion))
      die (("couldn't get firmware version from %s\n", filename));
    
    if (!m3_TestClient::firmwareGetDate (fileVersion, fileDate))
      die (("couldn't get date from firmware string in %s\n", filename));

    if (!m3_TestClient::firmwareGetType (fileVersion, fileType))
      die (("couldn't get type from firmware string in %s\n", filename));
    
    printf ("filename for slot %d is %s\n", i, filename);
    printf ("slot date is %s\n", slotDate);
    printf ("file date is %s\n", fileDate);
    
    if (strcmp (fileType, slotType))
      die (("type mismatch %s != %s\n", fileType, slotType));

    if (m3_TestClient::firmwareDateCompare (slotDate, fileDate) >= 0)
    {
      printf ("up to date\n");
      continue;
    }
    
    printf ("reprogramming slot.\n");   
 
    if (!hex2uc.program (hostarg, filename, i))
      die (("couldn't program %s into slot %d\n", filename, i));
  }
}
