/*
  invalid_proxy.c
  very evil thing to fake invalidRoute traps from m2 switches
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#include <sys/time.h>
#include <sys/types.h>


#include "m3_TestClient.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static m3_TestClient*client = 0;
static void signal_handler (int)
{
  if (client)
    client->signaled = 1;
}

static void usage (char*s)
{
  printf ("usage: %s <host> <port> [-v] [m2 switch names] [m2 ethernet addresses]\n", s);
  exit (1);
}

static int poll (m3_TestClient*client, int*invalid, int numSwitches, char**switchNames, int*s, int*port)
{
  for (int i = 0; i < numSwitches; i++)
  {
    if (!s)
      printf ("opening connection to %s\n", switchNames [i]);
    
    if (!client->open (switchNames [i]))
    {
      printf ("open failed for %s\n", switchNames [i]);
      return 0;
    }
    if (!client->m2_getInvalidRoutes (invalid, port))
    {
      printf ("m2_getInvalidRoutes failed for %s\n", switchNames [i]);
      return 0;
    }
    
    if (s && port)
    {
      *s = i;
      return 1;
    }
  }
  return port == 0;
}

int main (int argc, char**argv)
{
  m3_Component::initialize (printLine);
  m3_TestClient client;
  int verbose = 0;
  int numSwitches = 0;
  char**switchNames = 0;
  char**ethernetAddresses = 0;
  int host;
  int port;
  
  if (argc < 3)
    usage (argv [0]);  

  if (!strcmp (argv [3], "-v"))
    verbose = 1;
    
  numSwitches = (argc - 3 - verbose) / 2;
  switchNames = argv + 3 + verbose;
  ethernetAddresses = switchNames + numSwitches;

  printf ("%d m3 switches\n", numSwitches);

  char hostname [256];
  struct hostent*hp;
  struct sockaddr_in sa;
  sa.sin_family = AF_INET;
  sa.sin_port = htons (atoi (argv[2]));

  if (!(hp = gethostbyname (argv [1])))
  {
    printf ("couldn't get ip address for %s\n", argv [1]);
    return 1;
  }
  memcpy ((char*)&sa.sin_addr.s_addr, (char*)hp->h_addr, 4);

  int fd;
  int option;
  option = 1;

  fd = socket (AF_INET, SOCK_DGRAM, 0);
  if (fd <= 0)
  {
    perror ("socket failed for fd");
    return 1;
  }
  setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, (char*) &option, sizeof (option));

  int invalid [16 * 16];

  printf ("polling the first time");

  if (!poll (&client, invalid, numSwitches, switchNames, 0, 0))
  {
    printf ("couldn't poll the first time");
    return 1;
  }

  printf ("forwarding faked invalidRoute traps to %s:%s\n", argv [1], argv [2]);
  
  int*ports;
  ports = (int*) calloc (numSwitches * 16 * 16, sizeof (int));

  if (!ports)
  {
    printf ("calloc failed\n");
    return 1;
  }
  
  for (;;)
  {     
    int s, p;

    if (!poll (&client, invalid, numSwitches, switchNames, &s, &p))
      continue;
    
    if (s < 0 || s > numSwitches * 16)
    {
      printf ("bad switch %d\n", s);
      continue;
    }
    
    if (p < 0 || p > 16 * 16)
    {
      printf ("bad port %d\n", p);
      continue;
    }
    
    if (ports [s * 16 * 16 + p])
    {
      printf ("duplicate port %d\n", p);
      continue;
    }
    
    ports [s * 16 * 16 + p] = 1;

    m3_notify_message_t m;
    m.type = htonl (M3_invalidRouteTrap);
    memcpy (m.ethernet_address, ethernetAddresses [s], 6);
    m.index = htonl (p);

    sendto (fd, (char*) &m, sizeof (m), MSG_DONTWAIT, (struct sockaddr*) &sa, sizeof (sa));
    
    if (verbose)
      printf ("%s %d %d\n", switchNames [s], 1 + p / 16, 1 + p % 16);
  }    
}
