/*
  m3_Description.c
  finucane@myri.com
*/

#include "m3_defines.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "m3_Description.h"
#include "insist.h"

m3_DescriptionPart::m3_DescriptionPart ()
{
  parent = 0;
  type = index = 0;
}

m3_DescriptionLinecardPort::m3_DescriptionLinecardPort ()
{
  node = 0;
}

m3_DescriptionPort::m3_DescriptionPort ()
{
  port = 0;
  portNumber = 0;
  portDown = 0;
  control = 0;
  
  for (int i = 0; i < 2; i++)
    serdesFaultCount [i] = badCrcs [i] = goodCrcs [i] = 0;
}

m3_DescriptionXbar::m3_DescriptionXbar ()
{
  slot = uc = 0;
}

m3_DescriptionLinecard::m3_DescriptionLinecard ()
{
  slot = uc = 0;
  for (int i = 0; i < M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES; i++)
    temperatures [i] = 0;
  
}

char*m3_DescriptionPort::string64 (char*s, unsigned long long c64)
{
  insist (this && s);
  
  unsigned low, high;
  
  low = c64 & 0xffffffff;
  high = c64 >> 32;
  
  if (high)
    sprintf (s, "0x%X%08X", high, low);
  else
    sprintf (s, "%u", low);
  return s;
  exception: return 0;
}

int m3_DescriptionPart::print (FILE*fp)
{
  insist (this && fp);

  if (type)
  {
    fprintf (fp, "%s %d", m3_table_names [type - 1], index);
  }
  return 1;
  exception: return 0;
}

m3_DescriptionFan::m3_DescriptionFan ()
{
  fanSpeed = minimumFanSpeed = 0;
}

int m3_DescriptionFan::print (FILE*fp)
{
  insist (this && fp);
  
  if (type)
  {
    m3_DescriptionPart::print (fp);
    fprintf (fp, "\n");
    fprintf (fp, "fanSpeed %d\n", fanSpeed);
    fprintf (fp, "minimumFanSpeed %d\n", minimumFanSpeed);
  }
  return 1;
  exception: return 0;
}

  
int m3_DescriptionPort::printPort (FILE*fp)
{
  insist (this && fp);
  char sb [61];
  
  if (type)
  {
    if (parent)
    {
      parent->m3_DescriptionPart::print (fp);
      fprintf (fp, " Port %d ", portNumber);
    }
    fprintf (fp, "(");
    m3_DescriptionPart::print (fp);
    fprintf (fp, ")");

    if (badCrcs [1])
      fprintf (fp, " badCrcs %s", string64 (sb, badCrcs [1]));
    if (serdesFaultCount [1])
      fprintf (fp, " serdesFaultCount %d", serdesFaultCount [1]);
  }
  
  return 1;
  exception: return 0;  
}

int m3_DescriptionPort::printErrors (char*server, FILE*fp)
{
  insist (this && server && fp);
  char sb [61];
  
  if (type &&  (badCrcs [0] != badCrcs [1] || serdesFaultCount [0] != serdesFaultCount [1]))
  {
    fprintf (fp, "%s ", server);
    
    print (fp);
    
#if 0
    if (parent)
    {
      parent->m3_DescriptionPart::print (fp);
      fprintf (fp, " Port %d ", portNumber);
    }
    fprintf (fp, "(");
    m3_DescriptionPart::print (fp);
    fprintf (fp, ")");

    if (badCrcs [0] != badCrcs [1])
      fprintf (fp, " badCrcs %d", (int) (badCrcs [1] - badCrcs [0]));
    if (serdesFaultCount [0] != serdesFaultCount [1])
      fprintf (fp, " serdesFaultCount %d", (serdesFaultCount [1] - serdesFaultCount [0]));
    fprintf (fp, "\n");

#endif
  }
  
  return 1;
  exception: return 0;  
}

int m3_DescriptionPort::print (FILE*fp)
{
  insist (this && fp);
  if (type)
  {
    printPort (fp); 
    if (port)
    {
      fprintf (fp, " <--> ");
      port->printPort (fp);
    }
    
    fprintf (fp, "\n");
  }
  return 1;
  exception: return 0;
}

int m3_DescriptionLinecardPort::print (FILE*fp)
{
  insist (this && fp);

  if (node || port)
  {
    fprintf (fp, " <==> ");    

    if (node)
      fprintf (fp, "%s\n", node->getName ());
    else
    {
      m3_Description*s1;
      m3_DescriptionLinecard*c1 = (m3_DescriptionLinecard*) port->parent;
      insist (c1 && c1->type == M3_LINECARD_TABLE);
      insist (port->portNumber > 0 && port->portNumber <= M3_MAX_LINE_CARD_PORT);
      
      s1 = (m3_Description*) c1->parent;
      insist (s1 && !s1->parent);
      
      fprintf (fp, "%d.%d.%d.%d ",
	       (unsigned) (((char*)(&s1->ipAddress)) [0] & 0xff),
	       (unsigned) (((char*)(&s1->ipAddress)) [1] & 0xff),
	       (unsigned) (((char*)(&s1->ipAddress)) [2] & 0xff),
	       (unsigned) (((char*)(&s1->ipAddress)) [3] & 0xff));

      c1->ports [port->portNumber - 1].print (fp);
    }
  }
  return 1;
  exception: return 0;
}

int m3_DescriptionXbar::print (FILE*fp)
{
  if (type)
  {
    m3_DescriptionPart::print (fp);

    fprintf (fp, "\n");
    
    for (int i = 0; i < 16; i++)
      ports [i].print (fp);
  }
  return 1;
}

int m3_DescriptionLinecard::print (FILE*fp)
{
  insist (this && fp);
  
  if (type)
  {
    m3_DescriptionPart::print (fp);
    fprintf (fp, "\n");
    
    for (int i = 0; i < M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES && temperatures [i]; i++)
      fprintf (fp, "Temperature %d\n", temperatures [i]);
    
    xbar.print (fp);
  
    for (int i = 0; i < 8; i++)
    {
      ports [i].print (fp);
      linecardPorts [i].print (fp);
    }
    
  }
  
  return 1;
  exception: return 0;
}


m3_Description::m3_Description ()
{
  server = 0;
  *serverString = 0;
  *filename = 0;
  index = type = 0;
  parent = 0;
  numBackplaneTemperatures = 0;
  numFans = 0;
  fanCardTemperature = 0;
  
  for (int i = 0; i < M3_DESCRIPTION_NUM_BACKPLANE_TEMPERATURES; i++)
    backplaneTemperatures [i] = 0;
  
  exception:;
}

m3_Description::~m3_Description ()
{
}

int m3_Description::clear ()
{
  backplaneType = 0;
  numBackplaneXbars = 0;
  
  for (int i = 0; i < M3_MAX_LINE_CARD; i++)
    linecards [i].type = 0;

  for (int i = 0; i < M3_MAX_BACKPLANE_XBAR; i++)
    backplaneXbars [i].type = 0;
  return 1;
}


char*m3_Description::get (m3_WebClient*web, const char*fmt, ...)
{
  va_list args;
  insist (this && web && fmt);

  *buffer = 0;  
  va_start (args, fmt);
  vsprintf (buffer, fmt, args);
  va_end (args);


  if (*buffer == '/')
    return *filename ? web->get (filename) : (server ? web->get (server, buffer) : web->get (serverString, buffer));

  char*s;
  while ((s = web->token ()) && strcmp (s, buffer));
  
  return s ? web->token () : 0;

  exception: return 0;
}

char*m3_Description::getBefore (m3_WebClient*web, char*token, const char*fmt, ...)
{
  va_list args;
  insist (this && web && fmt);

  *buffer = 0;  
  va_start (args, fmt);
  vsprintf (buffer, fmt, args);
  va_end (args);

  char*s, *mark;
  mark = web->getMark ();
  
  insist (strcmp (token, buffer));
  
  while ((s = web->token ()) && strcmp (s, token) && strcmp (s, buffer));

  if (!s) return 0;
  
  if (!strcmp (s, token))
  {
    web->setMark (mark);
    return 0;
  }
  return web->token ();
  exception: return 0;
}

void m3_Description::fake4 ()
{
  for (int i = 0; i < backplaneType; i++)
  {
    if (linecards [i].fake4)
    {
      for (int j = 4; j < 8; j++)
      {
	linecards [i].ports [j].type = linecards [i].ports [0].type;
	linecards [i].ports [j].parent = &linecards [i];
	linecards [i].ports [j].index = 100000;
	insist (linecards [i].ports [j].index > 0);
	linecards [i].ports [j].portNumber = i + 1;
	linecards [i].ports [j].port = 0;
	linecards [i].ports [j].serdesFaultCount [1] = 0;
	linecards [i].ports [j].goodCrcs [1] = 0;
	linecards [i].ports [j].badCrcs [1] = 0;
      }
    }
  }
  exception:;
}

int m3_Description::examine (m3_WebClient*web)
{
  char*s;
  int n;
  
  insist (this && web);
  
  clear ();
  
  if (!get (web, "/all"))
  {
    printFormat ("couldn't get all");
    return 0;
  }

  if (!get (web, "BigUc"))
  {
    printFormat ("couldn't get bigUc slot");
    return 0;
  }
  
  if (!(s = get (web, "backplaneType")) || ((backplaneType = atoi (s)) <= 0))
  {
    printFormat ("couldn't get backplaneType.");
    return 0;
  }

  if (!(s = get (web, "ethernetAddress")))
  {
    printFormat ("couldn't get ethernet address");
    return 0;
  }
  
  int fat [6];        
  if (6 != sscanf (s, "%x:%x:%x:%x:%x:%x",
		   &fat [0], &fat [1], &fat [2], &fat [3], &fat [4], &fat [5]))
  {
    printFormat ("couldn't parse ethernet address");
    return 0;
  }

  for (int i = 0; i < 6; i++)
    ethernetAddress [i] = fat [i];

  if (!(s = get (web, "ipAddress")))
  {
    printFormat ("couldn't get ipAddress address");
    return 0;
  }

  if (4 != sscanf (s, "%d.%d.%d.%d", &fat [0], &fat [1], &fat [2], &fat [3]))
  {
    printFormat ("couldn't parse ip address");
    return 0;
  }

  for (int i = 0; i < 4; i++)
    ((char*)&ipAddress) [i] = fat [i];

  for (int i = 0; i < M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES && (s = get (web, "temperature")); i++)
    bigUcTemperatures [i] = atoi (s);

  web->setMark (0);  

  for (int i = 1 ; i <= backplaneType; i++)
  {
    if (!getLinecard (web, &i))
    {
      printFormat ("couldn't get line card %d", i);
      return 0;
    }
  }
  int slot;

  numTemperatures = 0;

  while (getBackplaneXbars (web));
  
  getFans (web);

  fake4 ();
  
  if (!connect ())
    return 0;
  
  return 1;
  exception: return 0;
}

int m3_Description::getFans (m3_WebClient*web)
{
  insist (this && web);
  char*s;  
  int slot, uc;

  do
  {
    if (!(s = get (web, "Slot")))
      return 0;
    
    slot = atoi (s);
  }
  while (slot != 50);

  insist (slot == 50);
  
  while (numFans < M3_MAX_FAN && (s = getBefore (web, "Temperature", "Fan")))
  {
    fans [numFans].type = M3_FAN_TABLE;
    fans [numFans].parent = this;
    fans [numFans].index = atoi (s);
    
    if (!(s = get (web, "fanSpeed")))
    {
      printFormat ("couldn't get fanSpeed for fan %d", numFans + 1);
      return 0;
    }
    fans [numFans].fanSpeed = atoi (s);

    if (!(s = get (web, "minimumFanSpeed")))
    {
      printFormat ("couldn't get minimumFanSpeed for fan %d", numFans + 1);
      return 0;
    }
    fans [numFans].minimumFanSpeed = atoi (s);
    numFans++;
  }
  
  if (!(s = get (web, "temperature")))
  {
    printFormat ("couldn't get temperature for fan card");
    return 0;
  }
  
  fanCardTemperature = atoi (s);
  return 1;
  exception: return 0;  
}

int m3_Description::getBackplaneXbars (m3_WebClient*web)
{
  insist (this && web);
  char*s, *mark;  
  int slot, uc;

  mark = web->getMark ();
  
  if (!(s = get (web, "Slot")))
    return 0;
  
  slot = atoi (s);
  
  if (slot < 33 || slot >= 50)
  {
    web->setMark (mark);
    return 0;
  }
  
  if (!(s = get (web, "Uc")))
    return 0;
  
  uc = atoi (s);
  insist (uc > 0 && uc < M3_MAX_UC);
  
  if (s = getBefore (web, "Slot", "Xbar16"))
  {
    insist (numBackplaneXbars < M3_MAX_BACKPLANE_XBAR);
    
    backplaneXbars [numBackplaneXbars].type = M3_XBAR16_TABLE;
    backplaneXbars [numBackplaneXbars].parent = this;
    backplaneXbars [numBackplaneXbars].index = atoi (s);
    backplaneXbars [numBackplaneXbars].uc = uc;
    backplaneXbars [numBackplaneXbars].slot = slot;

    insist (backplaneXbars [numBackplaneXbars].index > 0);
    
    for (int i = 0; i < 16; i++)
    {
      if (!(s = get (web, "XbarPort")))
      {
	printFormat ("couldn't get XbarPort %d, backplaneXbar %d", i, numBackplaneXbars + 1);
	return 0;
      }

      backplaneXbars [numBackplaneXbars].ports [i].type = M3_XBAR_PORT_TABLE;
      backplaneXbars [numBackplaneXbars].ports [i].parent = &backplaneXbars [numBackplaneXbars];
      backplaneXbars [numBackplaneXbars].ports [i].index = atoi (s);
      insist (backplaneXbars [numBackplaneXbars].ports [i].index > 0);

      backplaneXbars [numBackplaneXbars].ports [i].portNumber = i + 1;
      backplaneXbars [numBackplaneXbars].ports [i].port = 0;
      backplaneXbars [numBackplaneXbars].ports [i].serdesFaultCount [1] = 0;

      if (!(s = get (web, "goodCrcs")) || !get64 (s, &backplaneXbars [numBackplaneXbars].ports [i].goodCrcs [1]))
      {
	printFormat ("couldn't get goodCrcs for XbarPort %d, backplaneXbar %d", i, numBackplaneXbars + 1);
	return 0;
      }
      if (!(s = get (web, "badCrcs")) || !get64 (s, &backplaneXbars [numBackplaneXbars].ports [i].badCrcs [1]))
      {
	printFormat ("couldn't get badCrcs for XbarPort %d, backplaneXbar %d", i, numBackplaneXbars + 1);
	return 0;
      }
      if (!(s = get (web, "portDown")))
      {
	printFormat ("couldn't get portDown for XbarPort %d, backplaneXbar %d", i, numBackplaneXbars + 1);
	return 0;
      }
      backplaneXbars [numBackplaneXbars].ports [i].portDown = atoi (s);
      
      if (!(s = get (web, "controlForXbarPort")))
      {
	printFormat ("couldn't get control for XbarPort %d, backplaneXbar %d", i, numBackplaneXbars + 1);
	return 0;
      }
      backplaneXbars [numBackplaneXbars].ports [i].control = atoi (s);
    }
    numBackplaneXbars++;
    char*t;

    while (numBackplaneTemperatures < M3_DESCRIPTION_NUM_BACKPLANE_TEMPERATURES && (t = getBefore (web, "Slot", "temperature")))
      backplaneTemperatures [numBackplaneTemperatures++] = atoi (t);
  }
  
  return s != 0;
  exception: return 0;
}

int m3_Description::getLinecard (m3_WebClient*web, int*slot)
{
  insist (this && web && slot && *slot);
  char*s,*mark;
  int portType;
  int found, uc;
  
  mark = web->getMark ();
  
  if (!(s = get (web, "Slot")))
    return 0;

  found = atoi (s);
  insist (found >= *slot);

  while (*slot < found && *slot <= M3_MAX_LINE_CARD)
  {
    linecards [*slot - 1].type = 0;
    linecards [*slot - 1].parent = 0;
    linecards [*slot - 1].xbar.type = 0;
    linecards [*slot - 1].xbar.index = 0;
    linecards [*slot - 1].xbar.parent = 0;
    
    (*slot)++;
  }

  if (found != *slot)
  {
    web->setMark (mark);
    return 1;
  }

  linecards [*slot - 1].type = M3_LINECARD_TABLE;
  linecards [*slot - 1].parent = this;
  linecards [*slot - 1].slot = *slot;  
  linecards [*slot - 1].uc = 0;
  
  portType = getPortType (web);
  
  if (!(s = get (web, "slotState")))
  {
    printFormat ("couldn't get slotState for slot %d", *slot);
    return 0;
  }  

  if (!(s = get (web, "Uc")))
    return 0;

  uc = atoi (s);
  insist (uc > 0 && uc < M3_MAX_UC);

  if (portType <= 0)
  {  
    printFormat ("couldn't get port type");
    return 0;
  }
  
  if (!(s = get (web, "Type:")))
  {
    printFormat ("couldn't get type");
    return 0;
  }

  linecards [*slot - 1].fake4 = (int) strstr (s, "LC2E2X4FX");

  get (web, "reboot");
  
  if (!(s = get (web, "Linecard")))
  {
    printFormat ("couldn't get line card");
    return 0;
  }
  
  linecards [*slot - 1].index = atoi (s);
  insist (linecards [*slot - 1].index > 0);
  
  if ((s = getBefore (web, "Slot", "Xbar16")))
  {
    linecards [*slot - 1].xbar.type = M3_XBAR16_TABLE;
    linecards [*slot - 1].xbar.parent = &linecards [*slot - 1];
    linecards [*slot - 1].xbar.index = atoi (s);
    insist (linecards [*slot - 1].xbar.index > 0); 
    linecards [*slot - 1].xbar.uc = uc;
    
    for (int i = 0; i < 16; i++)
    {
      if (!(s = get (web, "XbarPort")))
      {
	printFormat ("couldn't get XbarPort %d, slot %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].xbar.ports [i].type = M3_XBAR_PORT_TABLE;
      linecards [*slot - 1].xbar.ports [i].parent = &linecards [*slot - 1].xbar;
      linecards [*slot - 1].xbar.ports [i].index = atoi (s);
      linecards [*slot - 1].xbar.ports [i].portNumber = i + 1;
      linecards [*slot - 1].xbar.ports [i].port = 0;

      if (!(s = get (web, "goodCrcs")) || !get64 (s, &linecards [*slot - 1].xbar.ports [i].goodCrcs [1]))
      {
	printFormat ("couldn't get goodCrcs for XbarPort %d, slot %d", i, *slot);
	return 0;
      }
      if (!(s = get (web, "badCrcs")) || !get64 (s, &linecards [*slot - 1].xbar.ports [i].badCrcs [1]))
      {
	printFormat ("couldn't get badCrcs for XbarPort %d, slot %d", i, *slot);
	return 0;
      }
      if (linecards [*slot - 1].fake4)
	linecards [*slot - 1].xbar.ports [i].goodCrcs [1] = linecards [*slot - 1].xbar.ports [i].badCrcs [1];

      if (!(s = get (web, "portDown")))
      {
	printFormat ("couldn't get portDown for XbarPort %d, slot %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].xbar.ports [i].portDown = 0;
      
      if (!(s = get (web, "controlForXbarPort")))
      {
	printFormat ("couldn't get control for XbarPort %d, slot %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].xbar.ports [i].control = atoi (s);
    }
  }
  
  insist (portType > 0 && portType <= M3_NUM_TABLES);
  
  int numPorts;
  numPorts = 0;
  
  for (int i = 0; i < M3_MAX_LINE_CARD_PORT; i++)
    linecards [*slot - 1].ports [i].type = 0;
  
  for (int i = 0; i < (linecards [*slot - 1].fake4 ? 4 : M3_MAX_LINE_CARD_PORT) && (s = get (web, m3_table_names [portType - 1])); i++)
  {
    linecards [*slot - 1].ports [i].type = portType;
    linecards [*slot - 1].ports [i].parent = &linecards [*slot - 1];
    linecards [*slot - 1].ports [i].index = atoi (s);
    insist (linecards [*slot - 1].ports [i].index > 0);
    linecards [*slot - 1].ports [i].portNumber = i + 1;
    linecards [*slot - 1].ports [i].port = 0;
    linecards [*slot - 1].ports [i].serdesFaultCount [1] = 0;
    linecards [*slot - 1].ports [i].goodCrcs [1] = 0;
    linecards [*slot - 1].ports [i].badCrcs [1] = 0;

    if (portType == M3_HSSDC_PORT_TABLE || portType == M3_SHORTWAVE_FIBER_PORT_TABLE)
    {
      if (!(s = get (web, portType == M3_HSSDC_PORT_TABLE ? "serdesFaultForHssdcPort" : "serdesFaultForShortwaveFiberPort")))
      {
	printFormat ("couldn't get serdesFault for port %d, line card %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].ports [i].portDown |= atoi (s);

      if (!(s = get (web, portType == M3_HSSDC_PORT_TABLE ? "serdesFaultCountForHssdcPort" : "serdesFaultCountForShortwaveFiberPort")))
      {
	printFormat ("couldn't get serdesFaultCount for port %d, line card %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].ports [i].serdesFaultCount [1] = atoi (s);

      if (!(s = get (web, portType == M3_HSSDC_PORT_TABLE ? "notInSyncForHssdcPort" : "notInSyncForShortwaveFiberPort")))
      {
	printFormat ("couldn't get notInSync for port %d, line card %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].ports [i].portDown |= atoi (s);

      if (!(s = get (web, portType == M3_HSSDC_PORT_TABLE ? "controlForHssdcPort" : "controlForShortwaveFiberPort")))
      {
	printFormat ("couldn't get control for port %d, line card %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].ports [i].control = atoi (s);
    }
    else if (portType == M3_LAN_PORT_TABLE)
    {
      if (!(s = get (web, "portOKForLanPort")))
      {
	printFormat ("couldn't get portOKForLanPort for port %d, line card %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].ports [i].portDown = !atoi (s);
    
      if (!(s = get (web, "controlForLanPort")))
      {
	printFormat ("couldn't get control for port %d, line card %d", i, *slot);
	return 0;
      }
      linecards [*slot - 1].ports [i].control = atoi (s);
    }
  }
  
  for (int i = 0; i < M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES; i++)
  {
    if (!(s = getBefore (web, "Slot", "temperature")))
    {
      printFormat ("couldn't get temperature %d line card %d", i, *slot);
      return 0;
    }
    linecards [*slot - 1].temperatures [i] = atoi (s);
  }
  
  return 1;
  exception: return 0;
}


int m3_Description::examine (m3_WebClient*web, int server)
{
  insist (this && web && server);
  this->server = server;
  return examine (web);
  exception: return 0;
}

int m3_Description::examineFile (m3_WebClient*web, char*filename)
{
  insist (this && web && filename && *filename);
  strncpy (this->filename, filename, M3_DISPLAY_STRING_LENGTH); /*still a kludge*/
  return examine (web);
  exception: return 0;
}

int m3_Description::examine (m3_WebClient*web, char*server)
{
  insist (this && web && server && *server);
  strncpy (this->serverString, server, 255);
  return examine (web);
  exception: return 0;
}

int m3_Description::getPortType (m3_WebClient*web)
{
  int type = -1;
  insist (this && web);

  char*mark;
  mark = web->getMark ();

  if (getBefore (web, "Slot", "SanPort"))
    type = M3_SAN_PORT_TABLE;
  if (getBefore (web, "Slot",  "LanPort"))
    type = M3_LAN_PORT_TABLE;
  if (getBefore (web, "Slot",  "HssdcPort"))
    type = M3_HSSDC_PORT_TABLE;
  if (getBefore (web, "Slot",  "ShortwaveFiberPort"))
    type = M3_SHORTWAVE_FIBER_PORT_TABLE;
  if (getBefore (web, "Slot",  "GEPort"))
    type = M3_GE_PORT_TABLE;
  
  web->setMark (mark);
  return type;
  exception: return 0;
}

int m3_Description::connect ()
{
  insist (this && backplaneType);
  
  if (backplaneType == 2)
  {
    insist (numBackplaneXbars == 0);
    
    for (int i = 0; i < 2; i++)
    {
      if (linecards [i].type && linecards [i].xbar.type)
      {
	for (int j = 0; j < 8; j++)
	{
	  linecards [i].ports [j].port = &linecards [i].xbar.ports [j + 8];
	  linecards [i].xbar.ports [j + 8].port = &linecards [i].ports [j];

	  if (linecards [(i + 1) % 2].type)
	  {
	    if (!linecards [(i + 1) % 2].xbar.type)
	    {
	      linecards [(i + 1) % 2].ports [j].port = &linecards [i].xbar.ports [j];
	      linecards [i].xbar.ports [j].port = &linecards [(i + 1) % 2].ports [j];
	    }
	    else
	    {
	      linecards [(i + 1) % 2].xbar.ports [j].port = &linecards [i].xbar.ports [j];	      
	    }
	  }
	}
      }
    }
    return 1;
  }
  insist (numBackplaneXbars == backplaneType / 2);
  for (int i = 0; i < backplaneType; i++)
  {
    if (linecards [i].type)
    {
      for (int j = 0; j < 8; j++)
      {
	int k;
	if (backplaneType == 4)
	  k =  16 * (j / 4) + i + ((j & 3) << 2);
	else if (backplaneType == 8)
	  k = 16 * (j / 2) + i + ((j & 1) * 8);
	else if (backplaneType == 16)
	  k = 16 * j + i;
	else 
	{
	  insist (0);
	}
      
	if (linecards [i].xbar.type)
	{
	  linecards [i].xbar.ports [j].port = &backplaneXbars [k / 16].ports [k % 16];
	  backplaneXbars [k / 16].ports [k % 16].port = &linecards [i].xbar.ports [j];
	  
	  linecards [i].ports [j].port = &linecards [i].xbar.ports [j + 8];
	  linecards [i].xbar.ports [j + 8].port = &linecards [i].ports [j];
	}
	else
	{
	  linecards [i].ports [j].port = &backplaneXbars [k / 16].ports [k % 16];
	  backplaneXbars [k / 16].ports [k % 16].port = &linecards [i].ports [j];
	}
      }
    }
  }
  return 1;
  exception: return 0;
}

int m3_Description::print (FILE*fp)
{
  insist (this && fp);
  
  if (server)
    fprintf (fp, "%d.%d.%d.%d\n",
	     (unsigned) (((char*)(&server)) [0] & 0xff),
	     (unsigned) (((char*)(&server)) [1] & 0xff),
	     (unsigned) (((char*)(&server)) [2] & 0xff),
	     (unsigned) (((char*)(&server)) [3] & 0xff));
  else
    fprintf (fp, "%s\n", serverString);
 
  fprintf (fp, "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n",
	   (unsigned) ethernetAddress [0] & 0xff,
	   (unsigned) ethernetAddress [1] & 0xff,
	   (unsigned) ethernetAddress [2] & 0xff,
	   (unsigned) ethernetAddress [3] & 0xff,
	   (unsigned) ethernetAddress [4] & 0xff,
	   (unsigned) ethernetAddress [5] & 0xff);


  for (int i = 0; i < M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES; i++)
    fprintf (fp, "Big Uc Temperature %d\n", bigUcTemperatures [i]);

  fprintf (fp, "numBackplaneTemperatures %d\n", numBackplaneTemperatures);
  
  for (int i = 0; i < numBackplaneTemperatures; i++)
    fprintf (fp, "Backplane Temperature %d\n", backplaneTemperatures [i]);

  if (numFans)
    fprintf (fp, "Fan Card Temperature %d\n", fanCardTemperature);

  fprintf (fp, "numFans %d\n", numFans);

  for (int i = 0; i < numFans; i++)
    fans [i].print (fp);
  
  for (int i = 0; i < M3_MAX_LINE_CARD; i++)
    linecards [i].print (fp);


  fprintf (fp, "numBackplaneXbars %d\n", numBackplaneXbars);

  for (int i = 0; i < numBackplaneXbars; i++)
    backplaneXbars [i].print (fp);

  return 1;
  exception: return 0;
}

int m3_Description::printErrors (FILE*fp, int silent)
{
  insist (this && fp);
  char t [80];
  
  if (server)
    sprintf (t, "%d.%d.%d.%d",
	     (unsigned) (((char*)(&server)) [0] & 0xff),
	     (unsigned) (((char*)(&server)) [1] & 0xff),
	     (unsigned) (((char*)(&server)) [2] & 0xff),
	     (unsigned) (((char*)(&server)) [3] & 0xff));
  else
    sprintf (t, "%s", serverString);
  
  for (int i = 0; i < M3_MAX_LINE_CARD; i++)
  {
    for (int j = 0; j < 16; j++)
    {
      if (!silent)
	linecards [i].xbar.ports [j].printErrors (t, fp);
      linecards [i].xbar.ports [j].shiftCounts ();
    }
    
    for (int j = 0; j < M3_MAX_LINE_CARD_PORT; j++)
    {
      if (!silent)
	linecards [i].ports [j].printErrors (t, fp);
      linecards [i].ports [j].shiftCounts ();
    }
  }

  for (int i = 0; i < numBackplaneXbars; i++)
  {  
    for (int j = 0; j < 16; j++)
    {
      if (!silent)
	backplaneXbars [i].ports [j].printErrors (t, fp);
      backplaneXbars [i].ports [j].shiftCounts ();
    }
  }

  return 1;
  exception: return 0;
}
 

int m3_Description::get64 (char*s, unsigned long long*c64)
{
  insist (this && s && c64);
  unsigned high, low;
  char*t;

  if ((t = strstr (s, "0x")) && t == s && strlen (s) >= 10)
  {
    char e [30];
    strncpy (e, s, 29);
    e [29] = 0;
    
    t = e + strlen (e) - 8;
    sscanf (t, "%x", &low);
    *t = 0;
    sscanf (e, "%x", &high);
    //*c64 =  (((unsigned long long) high) << 32) + low;
    *c64 = low;
    return 1;
  }
  else if (sscanf (s, "%u", &low) == 1)
  {
    *c64 = low;
    return 1;
  }
  exception: return 0;
}

int m3_DescriptionPort::shiftCounts ()
{
  insist (this);

  goodCrcs [0] = goodCrcs [1];
  badCrcs [0] = badCrcs [1];
  serdesFaultCount [0] = serdesFaultCount [1];
  
  return 1;
  exception: return 0;
}

int m3_Description::getNumXbars ()
{
  int n = 0;
  insist (this);
  
  for (int i = 0; i < M3_MAX_LINE_CARD; i++)
  {
    if (linecards [i].type && linecards [i].xbar.index)
      n++;
  }
  
  return numBackplaneXbars + n;
  exception: return 0;
}

  
int m3_Description::getLinecard (int xbar)
{
  int n = 0;
  insist (this);

  for (int i = 0; i < M3_MAX_LINE_CARD; i++)
  {
    if (linecards [i].type && linecards [i].xbar.index)
    {
      n++;
      if (n == xbar)
	return i;
    }
  }
  exception: return -1;
}

m3_DescriptionXbar*m3_Description::getXbar (int xbar)
{
  int n = 0;
  insist (this && xbar);

  for (int i = 0; i < M3_MAX_LINE_CARD; i++)
  {
    if (linecards [i].type && linecards [i].xbar.index)
    {
      n++;
      if (n == xbar)
	return &linecards [i].xbar;
    }
  }

  xbar -= n;
  insist (xbar >= 0 && xbar <= numBackplaneXbars);
  return &backplaneXbars [xbar - 1];
  
  exception: return 0;
}

m3_DescriptionLinecard*m3_Description::getLinecardFromUc (int uc)
{
  insist (this);
  for (int i = 0; i < backplaneType; i++)
    if (linecards [i].index && linecards [i].uc == uc)
      return &linecards [i];
  exception: return 0;
}

m3_DescriptionXbar*m3_Description::getXbarFromSlot (int slot)
{
  insist (this);
  for (int i = 0; i < numBackplaneXbars; i++)
    if (backplaneXbars [i].index && backplaneXbars [i].slot == slot)
      return &backplaneXbars [i];
  exception: return 0;
}

m3_DescriptionLinecard*m3_Description::getLinecardFromSlot (int slot)
{
  insist (this);
  for (int i = 0; i < backplaneType; i++)
    if (linecards [i].index && linecards [i].slot == slot)
      return &linecards [i];
  exception: return 0;
}

m3_DescriptionXbar*m3_Description::getXbarFromUc (int uc)
{
  insist (this);
  for (int i = 0; i < numBackplaneXbars; i++)
    if (backplaneXbars [i].index && backplaneXbars [i].uc == uc)
      return &backplaneXbars [i];
  exception: return 0;
}

m3_DescriptionLinecard*m3_Description::getLinecardFromPart (int partType, int index, int*portIndex)
{
  insist (this && portIndex);

  for (int i = 0; i < backplaneType; i++)
  {
    if (linecards [i].index)
    {
      for (*portIndex = 0; *portIndex < 8; (*portIndex)++)
	if (linecards [i].ports [*portIndex].type == partType && linecards [i].ports [*portIndex].index == index)
	  return &linecards [i];
	else
	  printf ("no");
    }
  }
  exception: return 0;
}

m3_Description*m3_Description::find (char ethernetAddress [6], m3_Description**descriptions, int numDescriptions)
{
  insist (ethernetAddress && descriptions && numDescriptions > 0);

  for (int i = 0; i < numDescriptions; i++)
    if (descriptions [i] && !memcmp (ethernetAddress, descriptions [i]->ethernetAddress, 6))
      return descriptions [i];
  
  exception: return 0;
}

int m3_Description::connect (mt_Graph*graph, m3_XbarTable*table, m3_Description**descriptions, int numDescriptions)
{
  insist (this && graph && table && descriptions && numDescriptions > 0);

  for (int i = 0; i < graph->getNumSwitches (); i++)
  {
    m3_XbarTableEntry*e1,*e2;
    m3_Description*s1,*s2;
    mt_Node*n1, *n2;
    m3_DescriptionXbar*x1,*x2;

    n1 = graph->getSwitch (i);
    insist (n1);
    e1 = table->lookup (n1);
    insist (e1);

    s1 = find (e1->ethernetAddress, descriptions, numDescriptions);
    insist (s1);    
    insist (e1->xbar > 0 && e1->xbar <= s1->getNumXbars ());

    x1 = s1->getXbar (e1->xbar);
    insist (x1);    
    insist (x1->index > 0 && x1->index <= M3_MAX_LINE_CARD + M3_MAX_BACKPLANE_XBAR);

    for (int p = 0; p < 16; p++)
    {
      int pp = p + e1->correction;

      if ((n2 = n1->getNode (p)))
      {
	int o, oo;
	insist (p >= 0 && p < 16);
	o = n1->getOpposite (p);
	insist (o >= 0 && o < 16);

	insist (x1->ports [pp].port);
	insist (x1->ports [pp].port->parent);
	
	m3_DescriptionLinecard*c1,*c2;
	c1 = (m3_DescriptionLinecard*) x1->ports [pp].port->parent;

	if (c1->type != M3_LINECARD_TABLE)
	  continue;
	

	int p1 = x1->ports [pp].port->portNumber - 1;
	insist (p1 >= 0 && p1 < 16);

	if (n2->isHost ())
	{  
	  insist (!c1->linecardPorts [p1].node);
	  c1->linecardPorts [p1].node = n2;
	  c1->linecardPorts [p1].portNumber = p1 + 1;
	  continue;
	}
	
	e2 = table->lookup (n2);
	insist (e2);
	s2 = find (e2->ethernetAddress, descriptions, numDescriptions);
	insist (s2);

	insist (e2->xbar > 0 && e2->xbar <= s2->getNumXbars ());

	x2 = s2->getXbar (e2->xbar);
	insist (x2);

	insist (x2->index > 0 && x2->index <= M3_MAX_LINE_CARD + M3_MAX_BACKPLANE_XBAR);

	oo = o + e2->correction;
	insist (pp >= 0 && pp < 16);
	insist (oo >= 0 && oo < 16);

	insist (x2->ports [oo].port);
	insist (x2->ports [oo].port->parent);
	
	c2 = (m3_DescriptionLinecard*) x2->ports [oo].port->parent;
	if (c2->type != M3_LINECARD_TABLE)
	  continue;

	int p2 = x2->ports [oo].port->portNumber - 1;
	insist (p2 >= 0 && p2 < 16);
	
	if (c1->linecardPorts [p1].port || c2->linecardPorts [p2].port)
	{
	  insist (c2->linecardPorts [p2].port == &c1->linecardPorts [p1]);
	  insist (c1->linecardPorts [p1].port == &c2->linecardPorts [p2]);
	  insist (c1->linecardPorts [p1].portNumber == p1 + 1);
	  insist (c2->linecardPorts [p2].portNumber == p2 + 1);
	  insist (c1->linecardPorts [p1].parent == c1->ports [p1].parent);
	  insist (c2->linecardPorts [p2].parent == c2->ports [p2].parent);

	  continue;
	}

	c1->linecardPorts [p1].port = &c2->linecardPorts [p2];
	c2->linecardPorts [p2].port = &c1->linecardPorts [p1];

	c1->linecardPorts [p1].parent = c1->ports [p1].parent;
	c2->linecardPorts [p2].parent = c2->ports [p2].parent;

	c1->linecardPorts [p1].portNumber = p1 + 1;
	c2->linecardPorts [p2].portNumber = p2 + 1;

	insist (!c1->linecardPorts [p1].node);
	insist (!c2->linecardPorts [p2].node);
	
      }
    }
  }

  return 1;
  exception: return 0;
}
