/*
  m3_Description.h
  finucane@myri.com
*/

#ifndef m3_Description_h
#define m3_Description_h

#include "m3_WebClient.h"
#include "m3_defines.h"
#include "mt_Graph.h"
#include "m3_XbarTable.h"

#define M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES 2
#define M3_DESCRIPTION_NUM_BACKPLANE_TEMPERATURES 2

class m3_DescriptionPart : public m3_Component
{
  public:
  int type;
  int index;
  m3_DescriptionPart*parent;
  m3_DescriptionPart ();
  virtual int print (FILE*fp);
};


class m3_DescriptionPort : public m3_DescriptionPart
{
  public:
  int portNumber;
  m3_DescriptionPort*port;

  unsigned long long goodCrcs [2];
  unsigned long long badCrcs [2];
  unsigned serdesFaultCount [2];
  int portDown;
  int control;
  
  m3_DescriptionPort ();
  virtual int print (FILE*fp);
  int printPort (FILE*fp);
  int shiftCounts ();
  char*string64 (char*s, unsigned long long c64);
  int printErrors (char*server, FILE*fp);
};

class m3_DescriptionLinecardPort : public m3_DescriptionPort
{
  public:
  mt_Node*node;
  m3_DescriptionLinecardPort ();
  int print (FILE*fp);
};

class m3_DescriptionXbar : public m3_DescriptionPart
{
  public:
  int slot;
  int uc;
  m3_DescriptionPort ports [16];
  m3_DescriptionXbar ();
  virtual int print (FILE*fp);
};


class m3_DescriptionLinecard : public m3_DescriptionPart
{
  public:
  m3_DescriptionXbar xbar;
  m3_DescriptionPort ports [M3_MAX_LINE_CARD_PORT];
  m3_DescriptionLinecardPort linecardPorts [M3_MAX_LINE_CARD_PORT];
  int slot;
  int uc;
  int temperatures [M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES];
  int fake4;
  virtual int print (FILE*fp);
  m3_DescriptionLinecard ();
};

class m3_DescriptionFan : public m3_DescriptionPart
{
  public:
  
  m3_DescriptionFan ();
  virtual int print (FILE*fp);
  
  int fanSpeed;
  int minimumFanSpeed;
  int fault;
};

class m3_Description : public m3_DescriptionPart
{
  private:
  int server;
  char filename [M3_DISPLAY_STRING_LENGTH + 1];
  char*getBefore (m3_WebClient*web, char*token, const char*fmt, ...);
  char*get (m3_WebClient*web, const char*fmt, ...);
  char*getInt (m3_WebClient*web, char*name, int value);
  int numTemperatures;
  int getBackplaneXbars (m3_WebClient*web);
  int getLinecard (m3_WebClient*web, int*slot);
  int getFans (m3_WebClient*web);
  int connect ();
  int getPortType (m3_WebClient*web);
  int examine (m3_WebClient*web);
  int clear ();
  int get64 (char*s, unsigned long long*c64);
  m3_Description*find (char ethernetAddress [6], m3_Description**descriptions, int numDescriptions);
  void fake4 ();
  
  public:

  int backplaneTemperatures [M3_DESCRIPTION_NUM_BACKPLANE_TEMPERATURES];
  int bigUcTemperatures [M3_DESCRIPTION_NUM_LINECARD_TEMPERATURES];
  int fanCardTemperature;
  
  int ipAddress;
  char ethernetAddress [6];
  int backplaneType;
  int numBackplaneXbars;
  int numFans;
  int numBackplaneTemperatures;
  
  m3_DescriptionLinecard linecards [M3_MAX_LINE_CARD];
  m3_DescriptionXbar backplaneXbars [M3_MAX_BACKPLANE_XBAR];
  m3_DescriptionFan fans [M3_MAX_FAN];

  char serverString [256];
  char buffer [1000];

  m3_Description ();
  ~m3_Description ();
  int examine (m3_WebClient*web, int server);
  int examine (m3_WebClient*web, char*server);
  int examineFile (m3_WebClient*web, char*filename);
  int print (FILE*fp);
  int printErrors (FILE*fp, int silent);
  int getNumXbars ();
  int getLinecard (int xbar);
  m3_DescriptionXbar*getXbar (int xbar);
  m3_DescriptionLinecard*getLinecardFromUc (int uc);
  m3_DescriptionLinecard*getLinecardFromSlot (int slot);
  m3_DescriptionLinecard*getLinecardFromPart(int partType, int index, int*portIndex);
  m3_DescriptionXbar*getXbarFromUc (int uc);
  m3_DescriptionXbar*getXbarFromSlot (int slot);
  int connect (mt_Graph*graph, m3_XbarTable*table, m3_Description**descriptions, int numDescriptions);
};


#endif /*m3_Description_h*/
