/*
   hex2flash.c
   class to spew a hex file into a vxWorks
   board over the ethernet.

   (c), 2001, Myricom, Inc.
   finucane@myri.com (David Finucane)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <signal.h>

#include "m3_defines.h"
#include "m3_Hex2Flash.h"
#include "insist.h"

#define MAX_LINE 100

/* message format:

   variable version

   M3_FLASH_STRING
   4 byte flag = 0 = M3_FLASH_VARIABLE
   4 byte base
   4 byte size
   image

   or

   permanent version

   M3_FLASH_STRING      
   4 byte flag = 1 = M3_FLASH_PERMANENT
   6 byte ethernet address
   4 byte ip address
   4 byte netmask
   4 byte base
   4 byte size
   image

   or

   exended version

   M3_FLASH_STRING
   4 byte flag = 2 = M3_FLASH_EXTENDED
   4 byte count

   repeat count times:   
   [
     4 byte value
   ]

   4 byte base
   4 byte size
   image

 */

m3_Hex2Flash::m3_Hex2Flash (int verbose)
{
  this->verbose = verbose;
}

int m3_Hex2Flash::hex_size (FILE*fp)
{
  int c = 0;
  int nBytes;

  char *s;
  char buffer[MAX_LINE + 1];
  char temp[MAX_LINE + 1];

  rewind(fp);
  while (s = fgets(buffer, MAX_LINE, fp))
  {
    temp[2] = 0;
    strncpy(temp, s, 2);
    s += 2;

    if (!strcmp(temp, "S9"))
      continue;

    if (strcmp(temp, "S3"))
      break;

    temp[2] = 0;
    strncpy(temp, s, 2);
    s += 2;

    if (sscanf(temp, "%x", &nBytes) != 1)
      break;

    /*subtract off address */
    nBytes -= 4;

    /*subtract off checksum */
    nBytes -= 1;

    c += nBytes;
  }
  rewind(fp);
  return c;
}



char*m3_Hex2Flash::hex_image(FILE*fp, int size, unsigned*start)
{
  char *image;
  char buffer[MAX_LINE + 1];
  char temp[MAX_LINE];
  int nBytes, bytes, line, i, tb;
  unsigned base, lastbase;
  char *s;

  lastbase = bytes = 0;

  if (!(image = (char *) malloc(size)))
  {
    printFormat ("couldn't malloc %d bytes", size);
    return 0;
  }

  for (line = 1; s = fgets(buffer, MAX_LINE, fp); line++) 
  {
    temp[2] = 0;
    strncpy(temp, s, 2);
    s += 2;

    if (!strcmp(temp, "S9"))
      continue;

    if (strcmp(temp, "S3"))
      break;

    temp[2] = 0;
    strncpy(temp, s, 2);
    s += 2;

    if (sscanf(temp, "%x", &nBytes) != 1)
      break;

    /*subtract off address */
    nBytes -= 4;

    /*subtract off checksum */
    nBytes -= 1;

    temp[8] = 0;
    strncpy(temp, s, 8);
    s += 8;

    if (sscanf(temp, "%x", &base) != 1)
      break;

    if (!lastbase)
      *start = base;
    else if (base != lastbase) 
    {
      printFormat ("hex file not contiguous.");
      return 0;
    }

    temp[2] = 0;
    for (i = 0; i < nBytes; i++, base++) 
    {
      strncpy(temp, s, 2);
      sscanf(temp, "%x", &tb);
      image[bytes++] = tb;
      s += 2;
    }
    lastbase = base;
  }
  if (s || bytes != size) 
  {
    printFormat ("error on line %d", line);
    return 0;
  }
  return image;
}


int m3_Hex2Flash::hex_open(char *hostname, int port)
{
  struct hostent *he;
  struct sockaddr_in sin;
  int fd;

  memset(&sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_port = htons(port);

  if (he = gethostbyname(hostname))
    memcpy((char *) &sin.sin_addr, he->h_addr, he->h_length);
  else if ((sin.sin_addr.s_addr = inet_addr(hostname)) == -1) 
  {
    printFormat ("can't get IP address of %s", hostname);
    return 0;
  }

  if ((fd = socket(PF_INET, SOCK_STREAM, 0)) < 0) 
  {
    perror("socket");
    return 0;
  }

  if (connect(fd, (struct sockaddr *) &sin, sizeof(sin)) < 0) 
  {
    perror("connect");
    return 0;
  }
  return fd;
}

int m3_Hex2Flash::epilogue (int fd)
{
  int c;
  char*s;
  
  for (;;) 
  {
    memset (terminal_buffer, 0, MAX_TERMINAL_MESSAGE_LENGTH);
    c = read (fd, terminal_buffer, MAX_TERMINAL_MESSAGE_LENGTH);
    if (c <= 0)
      return 0;

    for (s = terminal_buffer; s < terminal_buffer + c; s += strlen(s) + 1)
      printFormat ("%s", s);
    fflush (stdout);
  }
  return 0;
}

int m3_Hex2Flash::program (char*hostname, char*filename, int variable, char ethernet [6], char ip [4], char netmask [4], int extended_count, int*extended_values)
{
  int size = 0;
  char *image = 0;
  unsigned start;

  insist (filename && hostname && extended_count >= 0 && extended_count <= M3_FLASH_MAX_EXTENDED);

  if ((size = getImage (filename, &image, &start)))
  {
    int r;
    insist (image);
    r = program (image, size, start, hostname, variable, ethernet, ip, netmask, extended_count, extended_values);
    free (image);
    return r;
  }
  
  exception: return 0;
}

int m3_Hex2Flash::program (char*image, int size, unsigned start, char*hostname, int variable, char ethernet [6], char ip [4], char netmask [4], int extended_count, int*extended_values)
{
  int port, fd, c;
  FILE *fp;
  char *s;
  int bytes, flag;
  int tb, i;

  insist (image && size > 0 && hostname && extended_count >= 0 && extended_count <= M3_FLASH_MAX_EXTENDED);

  flag = M3_FLASH_VARIABLE;

  if (!variable)
  {
    flag = extended_count ? M3_FLASH_EXTENDED : M3_FLASH_PERMANENT;
    insist (ethernet && ip && netmask);

    int i;    
    for (i = 0; i < 6 && !ethernet [i]; i++);
    if (i == 6)
    {
      printFormat ("empty ethernet address");
      return 0;
    }
  }
  
  if (!(fd = hex_open (hostname, M3_FLASH_PORT))) 
  {
    printFormat ("couldn't connect to %s:%d", hostname, M3_FLASH_PORT);
    return 0;
  }

  printFormat ("writing M3_FLASH_STRING.");

  if (write (fd, (void*)M3_FLASH_STRING, strlen (M3_FLASH_STRING)) != strlen (M3_FLASH_STRING)) 
  {
    printFormat ("couldn't write M3_FLASH_STRING.");
    epilogue (fd);
    return 0;
  }

  printFormat ("writing flag %d", flag);
  tb = htonl (flag);
  if (write (fd, &tb, sizeof(int)) != sizeof (int)) 
  {
    printFormat ("couldn't write flag.");
    epilogue (fd);
    return 0;
  }

  if (flag != M3_FLASH_VARIABLE) 
  {
    insist (ethernet && ip && netmask);
    
    printFormat ("writing ethernet address = %02x:%02x:%02x:%02x:%02x:%02x",
		 ethernet[0] & 0xff, ethernet[1] & 0xff, ethernet[2] & 0xff,
		 ethernet[3] & 0xff, ethernet[4] & 0xff, ethernet[5] & 0xff);

    if (write(fd, ethernet, 6) != 6) 
    {
      printFormat ("couldn't write ethernet address.");
      epilogue (fd);
      return 0;
    }


    printFormat ("writing ip address %d.%d.%d.%d", ip [0] & 0xff, ip [1] & 0xff, ip [2] & 0xff, ip [3] & 0xff);
    if (write (fd, ip, 4) != 4)
    {
      printFormat ("couldn't write ip address");
      epilogue (fd);
      return 0;
    }

    printFormat ("writing netmask %d.%d.%d.%d", netmask [0] & 0xff , netmask [1] & 0xff, netmask [2] & 0xff, netmask [3] & 0xff);
  
    if (write (fd, netmask, 4) != 4)
    {
      printFormat ("couldn't write netmask");
      epilogue (fd);
      return 0;
    }
  }
  if (flag == M3_FLASH_EXTENDED)
  {
    printFormat ("writing extended count %d", extended_count);
    tb = htonl (extended_count);
    if (write(fd, &tb, sizeof (int)) != sizeof (int)) 
    {
      printFormat ("couldn't write extended count.");
      epilogue (fd);
      return 0;
    }

    for (i = 0; i < extended_count; i++)
    {
      printFormat ("writing %#x", extended_values [i]);
      tb = htonl (extended_values [i]);
      if (write(fd, &tb, sizeof (int)) != sizeof (int)) 
      {
	printFormat ("couldn't write it.");
	epilogue (fd);
	return 0;
      }
    }
  }
  
  printFormat ("writing base %x", start);
  tb = htonl (start);
  if (write (fd, &tb, sizeof(int)) != sizeof(int)) 
  {
    printFormat ("couldn't write base.");
    epilogue (fd);
    return 0;
  }

  printFormat ("writing size");
  tb = htonl (size);
  if (write (fd, &tb, sizeof(int)) != sizeof(int)) 
  {
    printFormat ("couldn't write size.");
    epilogue (fd);
    return 0;
  }

  printFormat ("writing image...");
  fflush (stdout);
  if (write (fd, image, size) != size) 
  {
    printFormat ("couldn't write image.");
    epilogue (fd);
    return 0;
  }

  printFormat ("Done.");
  epilogue (fd);
  return 1;
  exception: return 0;
}


int m3_Hex2Flash::getVersionString (char*image, int size, char*versionString)
{
  FILE *fp;

  insist (this && image && size > 0 && versionString);

  image [size - 1] = image [size - 2] = 0;
  
  for (char*s = image; s < image + size; s += 1 + strlen (s))
  {
    char*t;
    if ((t = strstr (s, "release")))
    {
      strncpy (versionString, t, M3_TEST_VERSION_STRING_LENGTH);
      return 1;
    }
  }
  exception: return 0;
}

int m3_Hex2Flash::getImage (char*filename, char**image, unsigned*start)
{
  FILE *fp;
  int size = 0;

  insist (this && filename && image && start);

  if (!(fp = fopen (filename, "r"))) 
  {
    printFormat ("couldn't open %s", filename);
    return 0;
  }

  if (!(size = hex_size (fp)))
  {
    printFormat ("error in  %s", filename);
    fclose (fp);
    return 0;
  }
  printFormat ("size = %d", size);
  printFormat ("reading image file");

  if (!(*image = hex_image (fp, size, start))) 
  {
    printFormat ("error in  %s", filename);
    fclose (fp);
    return 0;
  }
  
  fclose (fp);
  printFormat ("base is %x", *start);
  return size;
  
  exception: return 0;
}
