/*
  m3_RollcallClient.c
  finucane@myri.com (David Finucane)
*/

#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#define m3_define_trap_names
#include "m3_defines.h"
#include "m3_RollcallClient.h"
#include "insist.h"

m3_RollcallClient::m3_RollcallClient ()
{
  fd = 0;
  srand ((int) time (0));
  addresses = (int*) malloc (sizeof (int) * (maxAddresses = 100));
  insist (addresses);
  exception:;
}

m3_RollcallClient::~m3_RollcallClient ()
{
  if (addresses) free (addresses);
}

int m3_RollcallClient::open ()
{
  struct sockaddr_in sa;
  int s, option;

  sa.sin_port = 0;
  sa.sin_family = AF_INET;
  sa.sin_addr.s_addr = htonl (INADDR_ANY);
  
  if ((fd = socket (PF_INET, SOCK_DGRAM, 0)) < 0)
  {
    printFormat ("socket () failed");
    return 0;
  }
  
  option = 1;
  setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, (char*) &option, sizeof (option));
  setsockopt (fd, SOL_SOCKET, SO_BROADCAST, (char*) &option, sizeof (option));

  option = 1024 * 1024;
  setsockopt (fd, SOL_SOCKET, SO_RCVBUF, (char*) &option, sizeof (option));

  if (bind (fd, (struct sockaddr*)&sa, sizeof (sa)) < 0)
  {
    printFormat ("bind() failed");
    ::close (fd);
    return 0;
  }
  return 1;
}

int m3_RollcallClient::close ()
{
  if (fd)
    ::close (fd);
  return 1;
}

#define NUM_TRIES 1000

int m3_RollcallClient::findAddresses (int**addresses)
{
  m3_rollcall_message_t m, r;
  struct sockaddr_in sa;
  fd_set mask;
  struct timeval timeout;
  socklen_t length;

  insist (this && this->addresses && addresses && fd);

  memset ((char*) &sa, 0, sizeof (struct sockaddr_in));
  m.instance = rand ();
  int numAddresses;
  numAddresses = 0;
  insist (maxAddresses > 0);
  
  for (int tries = NUM_TRIES; tries; tries--)
  {
    m.type = M3_ROLLCALL_QUERY;

    sa.sin_family = AF_INET;
    sa.sin_port = htons (M3_ROLLCALL_PORT);
    sa.sin_addr.s_addr = htonl (INADDR_BROADCAST);

    m.type = htonl (M3_ROLLCALL_QUERY);
    
    if (sendto (fd, (char*) &m, sizeof (m), 0, (struct sockaddr*) &sa, sizeof (sa)) < 0)
    {
      printFormat ("sendto() failed");
      return 0;
    }

    for (;;)
    {
      timeout.tv_sec = 0;
      timeout.tv_usec = 1000 * 10;
      
      FD_ZERO (&mask);
      FD_SET (fd, &mask);

      if (select (1 + fd, &mask, 0, 0, &timeout) != 1)
	break;
      
      insist (FD_ISSET (fd, &mask));
      
      length = sizeof (sa);

      if (recvfrom (fd, (char*) &r, sizeof (r), 0, (struct sockaddr*) &sa, &length) == sizeof (r))
      {
	if (r.instance != m.instance || htonl (r.type) != M3_ROLLCALL_REPLY || htons (sa.sin_port) != M3_ROLLCALL_PORT)
	  continue;

	//printf ("got a reply from %x\n", sa.sin_addr.s_addr);
	
	int i;
	for (i = 0; i < numAddresses && this->addresses [i] != sa.sin_addr.s_addr; i++);

	if (i == numAddresses)
	{
	  if (numAddresses == maxAddresses)
	  {
	    int*a = (int*) malloc (sizeof (int) * (maxAddresses *= 2));
	    insist (a);
	    memmove (a, this->addresses, sizeof (int) * numAddresses);
	    free (this->addresses);
	    this->addresses = a;
	  }
	  this->addresses [numAddresses++] = sa.sin_addr.s_addr;
	  tries = NUM_TRIES;
	}
	
	m.type = htonl (M3_ROLLCALL_OK);
	if (sendto (fd, (char*) &m, sizeof (m), 0, (struct sockaddr*) &sa, sizeof (sa)) < 0)
	{
	  printFormat ("sendto() failed");
	  return 0;
	}
      }
    }
  }
  
  *addresses = this->addresses;
  return numAddresses;
  exception: return 0;
};
