/*
  m3_SwitchTest.c
  test class for m3 switches
  finucane@myri.com
*/

#include "m3_SwitchTest.h"
#include "insist.h"

#include "snmp_pp.h"
#include "eventlist.h"
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>

#include "oid.h"
#include "target.h"
#include "collect.h"


class m3_SwitchTestSnmp : m3_Component
{
  public:
  int numTargets;
  Snmp snmp;
  CTarget**targets;
  int status;
  
  m3_SwitchTestSnmp (int numTargets, char*community, char*names[]);
  ~m3_SwitchTestSnmp ();
};

m3_SwitchTestSnmp::m3_SwitchTestSnmp (int numTargets, char*community, char*names[]) : snmp (status)
{
  this->numTargets = numTargets;
  targets = (CTarget**) calloc (sizeof (CTarget*), numTargets);
  insist (targets);
  
  for (int i = 0; i < numTargets; i++)
  {
    IpAddress address (names [i]); 
    targets [i] = new CTarget (address, community, community);
    insist (targets [i]);
  }
  exception:;
}

m3_SwitchTestSnmp::~m3_SwitchTestSnmp ()
{
  if (targets)
  {
    for (int i = 0; i < numTargets; i++)
      if (targets [i]) delete targets [i];
    free (targets);
  }
}

m3_SwitchTest::m3_SwitchTest (int numSwitches, char*names[])
{
  insist (this && numSwitches > 0 && names);
  namesspace = 0;
  this->names = 0;
  this->numSwitches = numSwitches;
  
  snmp = (void*) new m3_SwitchTestSnmp (numSwitches, "public", names);
  insist (snmp);  
  
  exception:;
}

m3_SwitchTest::m3_SwitchTest (int numSwitches, ...)
{
  va_list ap;

  namesspace = 0;
  names = 0;
  insist (numSwitches > 0);
  namesspace = (char*) malloc (sizeof (char) * MAX_SWITCH_NAME * numSwitches);
  names = (char**) malloc (sizeof (char*) * numSwitches);
  this->numSwitches = numSwitches;
  insist (names && namesspace);
 
  va_start (ap, numSwitches);
  
  for (int i = 0; i < numSwitches; i++)
  {
    char*name = va_arg (ap, char*);
    names [i] = namesspace + MAX_SWITCH_NAME * sizeof (char) * i;
    strncpy (names [i], name, MAX_SWITCH_NAME);
  }
  
  va_end (ap);

  snmp = (void*) new m3_SwitchTestSnmp (numSwitches, "public", names);
  insist (snmp);

  exception:;
}

m3_SwitchTest::~m3_SwitchTest ()
{
  if (namesspace) free (namesspace);
  if (names) free (names);
  if (snmp) delete (m3_SwitchTestSnmp*) snmp;
}

int m3_SwitchTest::getBigUc (int target, m3_SwitchTestBigUc*bigUc)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_BIG_UC_VALUES];
  int status;
  int id = 1;
  
  insist (this && bigUc);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_BIG_UC_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_BIG_UC_TABLE, i + 1, id);
    //printf ("oid is %s\n", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getBigUc snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_BIG_UC_VALUES);

  int t, j;
  long unsigned len;
  j = 1;  //sic

  vb [j++].get_value (t); bigUc->backplaneType = t;

  vb [j++].get_value ((unsigned char*) bigUc->versionForBigUc, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  bigUc->versionForBigUc [len] = 0;
  
  vb [j++].get_value ((unsigned char*) bigUc->ethernetAddress, len);
  vb [j++].get_value (t); bigUc->primaryEthernetPortState = t;
  vb [j++].get_value (t); bigUc->secondaryEthernetPortState = t;
  vb [j++].get_value (t); bigUc->primaryEthernetPortIsActive = t;
  vb [j++].get_value ((unsigned char*) bigUc->ipAddress, len);
  vb [j++].get_value (t); bigUc->upTimeForBigUc = t;
  vb [j++].get_value (t); bigUc->switchTrapCount = t;
  vb [j++].get_value (t); bigUc->hostTrapCount = t;
  vb [j++].get_value (t); bigUc->trapTimeout = t;
  vb [j++].get_value (t); bigUc->trapsDropped = t;
  vb [j++].get_value (t); bigUc->trapsSent = t;
  vb [j++].get_value ((unsigned char*) bigUc->trapIpAddress, len);

  vb [j++].get_value ((unsigned char*) bigUc->community, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  bigUc->community [len] = 0;

  vb [j++].get_value (t); bigUc->commandForBigUc = t;
  vb [j++].get_value (t); bigUc->slotChangeCount = t;
  vb [j++].get_value (t); bigUc->refreshes = t;

  vb [j++].get_value ((unsigned char*) bigUc->trapSmtpServer, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  bigUc->trapSmtpServer [len] = 0;
  
  vb [j++].get_value ((unsigned char*) bigUc->trapMailAddress, len);
  vb [j++].get_value (t); bigUc->trapMailPeriod = t;
  
  insist (j == NUM_BIG_UC_VALUES);
  
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getVoltage (int target, int id, m3_SwitchTestVoltage*voltage)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_VOLTAGE_VALUES];
  int status;

  insist (this && voltage && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_VOLTAGE_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_VOLTAGE_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getVoltage snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_VOLTAGE_VALUES);

  int t, j;
  j = 1;  //sic

  vb [j++].get_value (t); voltage->type = t;
  vb [j++].get_value (t); voltage->voltage = t;
  vb [j++].get_value (t); voltage->nominal = t;

  insist (j == NUM_VOLTAGE_VALUES);
  
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getFan (int target, int id, m3_SwitchTestFan*fan)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_FAN_VALUES];
  int status;

  insist (this && fan && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_FAN_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_FAN_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getFan snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_FAN_VALUES);

  int t, j;
  j = 1;  //sic semper tyrranis

  vb [j++].get_value (t); fan->speed = t;
  vb [j++].get_value (t); fan->minimumSpeed = t;
  vb [j++].get_value (t); fan->bad = t;
  vb [j++].get_value (t); fan->badCount = t;
  vb [j++].get_value (t); fan->trayMissing = t;
  vb [j++].get_value (t); fan->trayMissingCount = t;

  insist (j == NUM_FAN_VALUES);
  
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getTemperature (int target, int id, m3_SwitchTestTemperature*temperature)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_TEMPERATURE_VALUES];
  int status;

  insist (this && temperature && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_TEMPERATURE_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_TEMPERATURE_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getTemperature snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_TEMPERATURE_VALUES);

  int t, j;
  j = 1;  //sic

  vb [j++].get_value (t); temperature->selfTestResult = t;
  vb [j++].get_value (t); temperature->temperature = t;

  insist (j == NUM_TEMPERATURE_VALUES);
  
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getXbarPort (int target, int id, m3_SwitchTestPort*port)
{
  char s[255];
  Pdu pdu;
  Counter64 c;
  Vb vb [NUM_XBAR_PORT_VALUES];
  int status;

  insist (this && port && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_XBAR_PORT_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_XBAR_PORT_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getXbarPort snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_XBAR_PORT_VALUES);

  port->type = XBAR;

  int t, j;
  j = 0;  

  vb [j++].get_value (t); port->port = t;
  vb [j++].get_value (t); port->selfTestResult = t;
  vb [j++].get_value (t); port->invalidRoutes = t;
  vb [j++].get_value (t); port->illegalSymbols = t;
  vb [j++].get_value (t); port->missedBeats = t;
  vb [j++].get_value (t); port->deadPorts = t;
  
  vb [j++].get_value (c); port->goodCrcs = ((long long) c.high () << 32) + c.low ();
  vb [j++].get_value (c); port->badCrcs = ((long long) c.high () << 32) + c.low ();

  vb [j++].get_value (t); port->dataIn = t;
  vb [j++].get_value (t); port->dataOut = t;
  vb [j++].get_value (t); port->portDownOnIllegalSymbol = t;
  vb [j++].get_value (t); port->portDownOnIllegalSymbolCount = t;
  vb [j++].get_value (t); port->portDownOnMissedBeat = t;
  vb [j++].get_value (t); port->portDownOnMissedBeatCount = t;
  vb [j++].get_value (t); port->portDown = t;
  vb [j++].get_value (t); port->portUpCount = t;
  vb [j++].get_value (t); port->receiveTimeout = t;
  vb [j++].get_value (t); port->receiveTimeoutCount = t;
  vb [j++].get_value (t); port->transmitTimeout = t;
  vb [j++].get_value (t); port->transmitTimeoutCount = t;
  vb [j++].get_value (t); port->control = t;

  insist (j == NUM_XBAR_PORT_VALUES);
  
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getHssdcPort (int target, int id, m3_SwitchTestPort*port)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_HSSDC_PORT_VALUES];
  int status;

  insist (this && port && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_HSSDC_PORT_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_HSSDC_PORT_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getHssdcPort snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_HSSDC_PORT_VALUES);

  port->type = HSSDC;

  int t, j;
  j = 0;  

  vb [j++].get_value (t); port->port = t;
  vb [j++].get_value (t); port->overruns = t;
  vb [j++].get_value (t); port->sanFault = t;
  vb [j++].get_value (t); port->sanFaultCount = t;
  vb [j++].get_value (t); port->serdesFault = t;
  vb [j++].get_value (t); port->serdesFaultCount = t;
  vb [j++].get_value (t); port->notInSync = t;
  vb [j++].get_value (t); port->notInSyncCount = t;
  vb [j++].get_value (t); port->selfTestResult = t;
  vb [j++].get_value (t); port->control = t;

  insist (j == NUM_HSSDC_PORT_VALUES);
  
  return 1;
  exception: return 0;
}
int m3_SwitchTest::getShortwaveFiberPort (int target, int id, m3_SwitchTestPort*port)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_SHORTWAVE_FIBER_PORT_VALUES];
  int status;

  insist (this && port && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_SHORTWAVE_FIBER_PORT_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_SHORTWAVE_FIBER_PORT_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getShortwaveFiberPort snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_SHORTWAVE_FIBER_PORT_VALUES);

  port->type = SHORTWAVE_FIBER;

  int t, j;
  j = 0;

  vb [j++].get_value (t); port->port = t;
  vb [j++].get_value (t); port->overruns = t;
  vb [j++].get_value (t); port->sanFault = t;
  vb [j++].get_value (t); port->sanFaultCount = t;
  vb [j++].get_value (t); port->serdesFault = t;
  vb [j++].get_value (t); port->serdesFaultCount = t;
  vb [j++].get_value (t); port->notInSync = t;
  vb [j++].get_value (t); port->notInSyncCount = t;
  vb [j++].get_value (t); port->selfTestResult = t;
  vb [j++].get_value (t); port->control = t;

  vb [j++].get_value (t); port->signalLost = t;
  vb [j++].get_value (t); port->signalLostCount = t;

  insist (j == NUM_SHORTWAVE_FIBER_PORT_VALUES);
  
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getLanPort (int target, int id, m3_SwitchTestPort*port)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_LAN_PORT_VALUES];
  int status;

  insist (this && port && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_LAN_PORT_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_LAN_PORT_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getLanPort snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_LAN_PORT_VALUES);

  port->type = LAN;

  int t, j;
  j = 0;

  vb [j++].get_value (t); port->port = t;
  vb [j++].get_value (t); port->missedBeats = t;
  vb [j++].get_value (t); port->illegalSymbolOnLanCount = t;
  vb [j++].get_value (t); port->illegalSymbols = t;
  vb [j++].get_value (t); port->portUpCount = t;
  vb [j++].get_value (t); port->portDownCount = t;
  vb [j++].get_value (t); port->portFast = t;
  vb [j++].get_value (t); port->noCable = t;
  vb [j++].get_value (t); port->missedBeatForLanPort = t;
  vb [j++].get_value (t); port->illegalOnLan = t;
  vb [j++].get_value (t); port->illegalOnSan = t;
  vb [j++].get_value (t); port->portOK = t;  

  insist (j == NUM_LAN_PORT_VALUES);
  
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getLineCardVoltages (int target, int slotNumber, unsigned*voltageA, unsigned*voltageB)
{
  char s[255];
  Pdu pdu, pdu2, pdu3;
  Vb vb [2];
  int status;
  int numParts;

  insist (this && slotNumber);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  insist (voltageB && voltageA);
  
  if (!(numParts = getParts (target, slotNumber, MAX_PARTS, parts)))
    return 0;

  for (int i = 0; i < numParts; i++)
  {
    if (parts [i].type == M3_LINECARD_TABLE)
    {
      for (int j = 0; j < 2; j++)
      {
	sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_LINECARD_TABLE, j + 2, parts [i].typeID);
	//printf ("oid is %s", s);
	vb [j].set_oid (s);
	pdu += vb [j];
      }
      if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
      {
	printFormat ("m3_SwitchTest::getSlot snmp->get failed.");
	printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
	return 0;
      }
      long t;
      pdu.get_vblist (vb, 2);
      vb [0].get_value (t); *voltageA = t;
      vb [1].get_value (t); *voltageB = t;
      return 1;
    }    
  }
  exception: return 0;
}

int m3_SwitchTest::getSlot (int target, int slotNumber, m3_SwitchTestSlot*slot)
{
  char s[255];
  Pdu pdu, pdu2;
  Vb vb [NUM_SLOT_VALUES + NUM_UC_VALUES];
  int status;

  insist (this && slot && slotNumber);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_SLOT_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_SLOT_TABLE, i + 1, slotNumber);
    //printf ("oid is %s\n", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getSlot snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_SLOT_VALUES);

  long unsigned len;
  long t;
  int j;
  j = 0;

  vb [j++].get_value (t); slot->id = t;
  vb [j++].get_value (t); slot->ucID = t;
  vb [j++].get_value ((unsigned char*) slot->slotSerialNumber, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  slot->slotSerialNumber [len] = 0;
  vb [j++].get_value (t); slot->lostCommunicationCount = t;
  vb [j++].get_value (t); slot->reestablishCommunicationCount = t;
  vb [j++].get_value (t); slot->changeCount = t;
  vb [j++].get_value (t); slot->slotState = t;

  insist (j == NUM_SLOT_VALUES);

  if (!slot->ucID)
    return 0;
  
  for (int i = 0; i < NUM_UC_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_UC_TABLE, i + 1, slot->ucID);
    //printf ("oid is %s\n", s);
    vb [i].set_oid (s);
    pdu2 += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu2, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getSlot snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }

  pdu2.get_vblist (vb, NUM_UC_VALUES);

  j = 0;
  vb [j++].get_value (t); slot->ucIDII = t;
  vb [j++].get_value (t); slot->firstPartID = t;
  vb [j++].get_value (t); slot->lastPartID = t;
  vb [j++].get_value (t); slot->state = t;
  vb [j++].get_value (t); slot->overTemperatureCount = t;
  vb [j++].get_value (t); slot->resetCount = t;
  vb [j++].get_value (t); slot->firmwareFaultCount = t;
  vb [j++].get_value ((unsigned char*) slot->serialNumber, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  slot->serialNumber [len] = 0;  
  vb [j++].get_value ((unsigned char*) slot->engineeringDateCode, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  slot->engineeringDateCode [len] = 0;
  vb [j++].get_value ((unsigned char*) slot->firmwareVersion, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  slot->firmwareVersion [len] = 0;
  vb [j++].get_value ((unsigned char*) slot->monitorFirmwareVersion, len, M3_SWITCH_TEST_DISPLAY_STRING_LENGTH - 1);
  slot->monitorFirmwareVersion [len] = 0;
  vb [j++].get_value (t); slot->scratchPad = t;
  vb [j++].get_value (t); slot->reboot = t;
  vb [j++].get_value (t); slot->monitorCause = t;
  vb [j++].get_value (t); slot->firstCoreDumpID = t;
  vb [j++].get_value (t); slot->lastCoreDumpID = t;

  insist (j == NUM_UC_VALUES);  
  insist (slot->ucIDII == slot->ucID);
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getPartsOld (int target, int id, int maxParts, m3_SwitchTestPart*parts)
{
  char s[255];
  Pdu pdu;
  int status;

  insist (this && id && maxParts > 0 && parts);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);

  if (!getSlot (target, id, &slot))
    return 0;
  
  insist (slot.lastPartID - slot.firstPartID + 1 <= maxParts);
  
  Vb*vb;
  vb = new Vb [2 * (slot.lastPartID - slot.firstPartID + 1)];
  insist (vb);
  
  int ns;
  ns = 0;

  /* had to increase MAX_VBS in snmp++/include/pdu.h for this to work*/
  
  for (int i = 0; i <= slot.lastPartID - slot.firstPartID; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_PART_TABLE, 2, i + slot.firstPartID);
    //printf ("oid is %s\n", s);
    vb [i * 2].set_oid (s);
    pdu += vb [i * 2];

    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_PART_TABLE, 3, i + slot.firstPartID);
    //printf ("oid is %s\n", s);
    vb [i * 2 + 1].set_oid (s);
    pdu += vb [i * 2 + 1];

  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getSlot snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    delete [] vb;
    return 0;
  }

  insist (pdu.get_vb_count () == 2 * (slot.lastPartID - slot.firstPartID + 1));
  
  pdu.get_vblist (vb, 2 * (slot.lastPartID - slot.firstPartID + 1));

  for (int i = 0; i <= slot.lastPartID - slot.firstPartID; i++)
  {
    long t;
    vb [i * 2].get_value (t); parts [i].type = t;
    vb [i * 2 + 1].get_value (t); parts [i].typeID = t;
    insist (parts [i].type > 0 && parts [i].type <= M3_NUM_TABLES);
    insist (parts [i].typeID > 0);
  }
  
  delete [] vb;
  
  return slot.lastPartID - slot.firstPartID + 1;
  exception: return 0;
}

int m3_SwitchTest::getParts (int target, int id, int maxParts, m3_SwitchTestPart*parts)
{
  char s[255];
  Pdu pdu;
  int status;

  insist (this && id && maxParts > 0 && parts);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);

  if (!getSlot (target, id, &slot))
    return 0;
  
  insist (slot.lastPartID - slot.firstPartID + 1 <= maxParts);

  int num_parts;
  num_parts = slot.lastPartID - slot.firstPartID + 1;
  int chunk_size;
  chunk_size = 10;

  for (int i = 0; i < num_parts; i+= chunk_size)
  {
    int num = chunk_size < num_parts - i ? chunk_size : num_parts - i;
    
    if (getParts (target, id, parts + i, slot.firstPartID + i, slot.firstPartID + i + num - 1) != num)
      return 0;
  }
  return num_parts;
  exception: return 0;
}

int m3_SwitchTest::getParts (int target, int id, m3_SwitchTestPart*parts, int firstPartID, int lastPartID)
{
  char s[255];
  Pdu pdu;
  int status;

  insist (this && id > 0 && parts);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);

  Vb*vb;
  vb = new Vb [2 * (lastPartID - firstPartID + 1)];
  insist (vb);
  
  int ns;
  ns = 0;

  /* had to increase MAX_VBS in snmp++/include/pdu.h for this to work*/
  
  for (int i = 0; i <= lastPartID - firstPartID; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_PART_TABLE, 2, i + firstPartID);
    //printf ("oid is %s\n", s);
    vb [i * 2].set_oid (s);
    pdu += vb [i * 2];

    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_PART_TABLE, 3, i + firstPartID);
    //printf ("oid is %s\n", s);
    vb [i * 2 + 1].set_oid (s);
    pdu += vb [i * 2 + 1];

  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getSlot snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    delete [] vb;
    return 0;
  }

  insist (pdu.get_vb_count () == 2 * (lastPartID - firstPartID + 1));
  
  pdu.get_vblist (vb, 2 * (lastPartID - firstPartID + 1));

  for (int i = 0; i <= lastPartID - firstPartID; i++)
  {
    long t;
    vb [i * 2].get_value (t); parts [i].type = t;
    vb [i * 2 + 1].get_value (t); parts [i].typeID = t;
    insist (parts [i].type > 0 && parts [i].type <= M3_NUM_TABLES);
    insist (parts [i].typeID > 0);
  }
  
  delete [] vb;
  
  return lastPartID - firstPartID + 1;
  exception: return 0;
}

int m3_SwitchTest::getPortCounts (int target, int slot, int firstPort, int numPorts, m3_SwitchTestPort*ports)
{
  insist (this);
  insist (target >= 0 && target < numSwitches);
  insist (numPorts <= MAX_PORTS);

  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  int portsSeen, skippedPorts;
  portsSeen = skippedPorts = 0;
  
  for (int i = 0; i < numParts && portsSeen < numPorts; i++)
  {
    switch (parts [i].type)
    {
      case M3_XBAR_PORT_TABLE:
	if (++skippedPorts > firstPort && !getXbarPort (target, parts [i].typeID, &ports [portsSeen++]))
	  return 0;
	break;
      case M3_HSSDC_PORT_TABLE:
	if (++skippedPorts > firstPort && !getHssdcPort (target, parts [i].typeID, &ports [portsSeen++]))
	  return 0;
	break;
      case M3_SHORTWAVE_FIBER_PORT_TABLE:
	if (++skippedPorts > firstPort && !getShortwaveFiberPort (target, parts [i].typeID, &ports [portsSeen++]))
	  return 0;
	break;
      case M3_LAN_PORT_TABLE:
	if (++skippedPorts > firstPort && !getLanPort (target, parts [i].typeID, &ports [portsSeen++]))
	  return 0;
	break;
      default:;
    }
  }
  insist (portsSeen == numPorts);
  return 1;
  exception: return 0;
}

/*c = a - b. returns 0 if a != b*/
int m3_SwitchTest::getPortDifference (m3_SwitchTestPort*a, m3_SwitchTestPort*b, m3_SwitchTestPort*c)
{
  int d = 1;
  insist (this && a && b && c);

  c->type = a->type - b->type; d = d && c->type == 0;
  c->port = a->port - b->port; d = d && c->port == 0;
  c->control = a->control - b->control; d = d && c->control == 0;
  c->selfTestResult = a->selfTestResult - b->selfTestResult; d = d && c->selfTestResult == 0;

  if (a->type == XBAR)
  {
    c->invalidRoutes = a->invalidRoutes - b->invalidRoutes; d = d && c->invalidRoutes == 0;
    c->illegalSymbols = a->illegalSymbols - b->illegalSymbols; d = d && c->illegalSymbols == 0;
    c->missedBeats = a->missedBeats - b->missedBeats; d = d && c->missedBeats == 0;
    c->deadPorts = a->deadPorts - b->deadPorts; d = d && c->deadPorts == 0;
    c->goodCrcs = a->goodCrcs - b->goodCrcs; d = d && c->goodCrcs == 0;
    c->badCrcs = a->badCrcs - b->badCrcs; d = d && c->badCrcs == 0;
    c->dataIn = a->dataIn - b->dataIn; d = d && c->dataIn == 0;
    c->dataOut = a->dataOut - b->dataOut; d = d && c->dataOut == 0;
    c->portDownOnIllegalSymbol = a->portDownOnIllegalSymbol - b->portDownOnIllegalSymbol; d = d && c->portDownOnIllegalSymbol == 0;
    c->portDownOnIllegalSymbolCount = a->portDownOnIllegalSymbolCount - b->portDownOnIllegalSymbolCount; d = d && c->portDownOnIllegalSymbolCount == 0;
    c->portDownOnMissedBeat = a->portDownOnMissedBeat - b->portDownOnMissedBeat; d = d && c->portDownOnMissedBeat == 0;
    c->portDownOnMissedBeatCount = a->portDownOnMissedBeatCount - b->portDownOnMissedBeatCount; d = d && c->portDownOnMissedBeatCount == 0;
    c->portDown = a->portDown - b->portDown; d = d && c->portDown == 0;
    c->portUpCount = a->portUpCount - b->portUpCount; d = d && c->portUpCount == 0;
    c->receiveTimeout = a->receiveTimeout - b->receiveTimeout; d = d && c->receiveTimeout == 0;
    c->receiveTimeoutCount = a->receiveTimeoutCount - b->receiveTimeoutCount; d = d && c->receiveTimeoutCount == 0;
    c->transmitTimeout = a->transmitTimeout - b->transmitTimeout; d = d && c->transmitTimeout == 0;
    c->transmitTimeoutCount = a->transmitTimeoutCount - b->transmitTimeoutCount; d = d && c->transmitTimeoutCount == 0;
  }
  else
  {
    c->overruns = a->overruns - b->overruns; d = d && c->overruns == 0;
    c->sanFault = a->sanFault - b->sanFault; d = d && c->sanFault == 0;
    c->sanFaultCount = a->sanFaultCount - b->sanFaultCount; d = d && c->sanFaultCount == 0;
    c->serdesFault = a->serdesFault - b->serdesFault; d = d && c->serdesFault == 0;
    c->serdesFaultCount = a->serdesFaultCount - b->serdesFaultCount; d = d && c->serdesFaultCount == 0;
    c->notInSync = a->notInSync - b->notInSync; d = d && c->notInSync == 0;
    c->notInSyncCount = a->notInSyncCount - b->notInSyncCount; d = d && c->notInSyncCount == 0;
  }
  if (a->type == SHORTWAVE_FIBER)
  {
    c->signalLost = a->signalLost - b->signalLost; d = d && c->signalLost == 0;
    c->signalLostCount = a->signalLostCount - b->signalLostCount; d = d && c->signalLostCount == 0;
  }
  
  return d;
  exception: return 0;
}

int m3_SwitchTest::getIntegerOid (int target, char*oid)
{
  Pdu pdu;
  Vb vb;
  int status;
  long value;
  
  insist (this);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);

  //printf ("oid is %s", oid);
  vb.set_oid (oid);
  pdu += vb;

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getIntegerOid failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }

  pdu.get_vblist (&vb, 1);
  vb.get_value (value);
  return value;
  exception: return 0;
}

int m3_SwitchTest::getRefreshes (int target)
{
  char s[255];
  
  insist (this);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);

  sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_BIG_UC_TABLE, 19, 1);

  return getIntegerOid (target, s);
  exception: return 0;
}

int m3_SwitchTest::setIntegerOid (int target, char*oid, unsigned int value)
{
  Pdu pdu;
  Vb vb;
  int status;  

  //printf ("setOid %s (%s)\n", oid, p);
  
  vb.set_oid (oid);
  vb.set_value ((long) value);
  pdu += vb;

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.set (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::setIntegerOid snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }

  //printf ("set %s to: %s\n", oid, vb.get_printable_value ());
}


int m3_SwitchTest::setPortControl (int target, int slot, int port, int value)
{
  char s [255];
  insist (this);
  insist (target >= 0 && target < numSwitches);
 
  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  int portSeen;
  portSeen = 0;
  
  for (int i = 0; i < numParts; i++)
  {
    if (parts [i].type != M3_XBAR_PORT_TABLE && parts [i].type != M3_HSSDC_PORT_TABLE && parts [i].type != M3_SHORTWAVE_FIBER_PORT_TABLE)
      continue;


    if (portSeen == port)
    {
      sprintf (s, parts [i].type == M3_XBAR_PORT_TABLE ? M3_controlForXbarPort_format :
	       (parts [i].type == M3_HSSDC_PORT_TABLE ? M3_controlForHssdcPort_format : 
		M3_controlForShortwaveFiberPort_format), parts [i].typeID);

      return setIntegerOid (target, s, value);
    }
    portSeen++;
  }
  insist (0);
  exception: return 0;
}

int m3_SwitchTest::setXbarTimeout (int target, int slot, int value)
{
  char s [255];
  insist (this);
  insist (target >= 0 && target < numSwitches);
 
  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  for (int i = 0; i < numParts; i++)
  {
    if (parts [i].type != M3_XBAR16_TABLE)
      continue;

    sprintf (s, M3_xbar16Timeout_format, parts [i].typeID);
    return setIntegerOid (target, s, value);
  }
  insist (0);
  exception: return 0;
}

int m3_SwitchTest::getXbarSelfTestResult (int target, int slot)
{
  char s [255];
  insist (this);
  insist (target >= 0 && target < numSwitches);
 
  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  for (int i = 0; i < numParts; i++)
  {
    if (parts [i].type != M3_XBAR16_TABLE)
      continue;

    sprintf (s, M3_xbar16SelfTestResult_format, parts [i].typeID);
    return getIntegerOid (target, s);
  }
  insist (0);
  exception: return 0;
}

int m3_SwitchTest::getVoltages (int target, int slot, int numVoltages, m3_SwitchTestVoltage*voltages)
{
  insist (this);
  insist (target >= 0 && target < numSwitches);
  insist (numVoltages <= MAX_VOLTAGES);

  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  int voltagesSeen;
  voltagesSeen = 0;
  
  for (int i = 0; i < numParts && voltagesSeen < numVoltages; i++)
  {
    if (parts [i].type == M3_VOLTAGE_TABLE && !getVoltage (target, parts [i].typeID, &voltages [voltagesSeen++]))
      return 0;
  }
  insist (voltagesSeen == numVoltages);
  return 1;
  exception: return 0;
}

int m3_SwitchTest::getFans (int target, int slot, int numFans, m3_SwitchTestFan*fans)
{
  insist (this);
  insist (target >= 0 && target < numSwitches);
  insist (numFans <= MAX_FANS);
  insist (fans);
  
  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  int fansSeen;
  fansSeen = 0;
  
  for (int i = 0; i < numParts && fansSeen < numFans; i++)
  {
    if (parts [i].type == M3_FAN_TABLE && !getFan (target, parts [i].typeID, &fans [fansSeen++]))
      return 0;
  }
  insist (fansSeen == numFans);
  return 1;
  exception: return 0;
}


int m3_SwitchTest::getTemperatures (int target, int slot, int numTemperatures, m3_SwitchTestTemperature*temperatures)
{
  insist (this);
  insist (target >= 0 && target < numSwitches);
  insist (numTemperatures <= MAX_TEMPERATURES);

  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  int temperaturesSeen;
  temperaturesSeen = 0;
  
  for (int i = 0; i < numParts && temperaturesSeen < numTemperatures; i++)
  {
    if (parts [i].type == M3_TEMPERATURE_TABLE && !getTemperature (target, parts [i].typeID, &temperatures [temperaturesSeen++]))
      return 0;
  }
  insist (temperaturesSeen == numTemperatures);
  return 1;
  exception: return 0;
}



int m3_SwitchTest::getCurrent (int target, int id, m3_SwitchTestCurrent*current)
{
  char s[255];
  Pdu pdu;
  Vb vb [NUM_CURRENT_VALUES];
  int status;

  insist (this && current && id);
  insist (target >= 0 && target < numSwitches);
  insist (snmp && ((m3_SwitchTestSnmp*) snmp)->targets && ((m3_SwitchTestSnmp*) snmp)->targets [target]);
  
  for (int i = 0; i < NUM_CURRENT_VALUES; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, M3_CURRENT_TABLE, i + 1, id);
    //printf ("oid is %s", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = ((m3_SwitchTestSnmp*) snmp)->snmp.get (pdu, *((m3_SwitchTestSnmp*) snmp)->targets [target])) != SNMP_CLASS_SUCCESS)
  {
    printFormat ("m3_SwitchTest::getCurrent snmp->get failed.");
    printFormat (((m3_SwitchTestSnmp*) snmp)->snmp.error_msg (status));
    return 0;
  }
  
  pdu.get_vblist (vb, NUM_CURRENT_VALUES);

  int t, j;
  j = 1;  //sic semper tyrranis

  vb [j++].get_value (t); current->overcurrentFault = t;
  vb [j++].get_value (t); current->current = t;

  insist (j == NUM_CURRENT_VALUES);
  
  return 1;
  exception: return 0;
}



int m3_SwitchTest::getCurrents (int target, int slot, int numCurrents, m3_SwitchTestCurrent*currents)
{
  insist (this);
  insist (target >= 0 && target < numSwitches);
  insist (numCurrents <= MAX_CURRENTS);

  int numParts;
  
  if (!(numParts = getParts (target, slot, MAX_PARTS, parts)))
    return 0;

  int currentsSeen;
  currentsSeen = 0;
  
  for (int i = 0; i < numParts && currentsSeen < numCurrents; i++)
  {
    if (parts [i].type == M3_CURRENT_TABLE && !getCurrent (target, parts [i].typeID, &currents [currentsSeen++]))
      return 0;
  }
  insist (currentsSeen == numCurrents);
  return 1;
  exception: return 0;
}


