/*
  m3_SwitchTest.h
  test class for m3 switches
  finucane@myri.com
*/

#ifndef m3_SwitchTest_h
#define m3_SwitchTest_h


#include "m3_Component.h"
#include "m3_defines.h"

#define M3_SWITCH_TEST_DISPLAY_STRING_LENGTH 255


typedef struct m3_SwitchTestBigUc
{  
  unsigned id;
  unsigned backplaneType;
  char versionForBigUc [M3_DISPLAY_STRING_LENGTH];
  char ethernetAddress [6];
  unsigned primaryEthernetPortState;
  unsigned secondaryEthernetPortState;
  unsigned primaryEthernetPortIsActive;
  char ipAddress [4];
  unsigned upTimeForBigUc;
  unsigned switchTrapCount;
  unsigned hostTrapCount;
  unsigned trapTimeout;
  unsigned trapsDropped;
  unsigned trapsSent;
  char trapIpAddress [4];
  char community [M3_DISPLAY_STRING_LENGTH];
  unsigned commandForBigUc;
  unsigned slotChangeCount;
  unsigned refreshes;
  char trapSmtpServer [4];
  char trapMailAddress [M3_DISPLAY_STRING_LENGTH];
  unsigned trapMailPeriod;
} m3_SwitchTestBigUc;

typedef struct m3_SwitchTestSlot
{
  unsigned id;
  unsigned ucID;
  char slotSerialNumber [M3_SWITCH_TEST_DISPLAY_STRING_LENGTH];
  unsigned lostCommunicationCount;
  unsigned reestablishCommunicationCount;
  unsigned changeCount;
  unsigned slotState;

  unsigned ucIDII;
  unsigned firstPartID;
  unsigned lastPartID;
  unsigned state;
  unsigned overTemperatureCount;
  unsigned resetCount;
  unsigned firmwareFaultCount;
  char serialNumber [M3_SWITCH_TEST_DISPLAY_STRING_LENGTH];
  char engineeringDateCode [M3_SWITCH_TEST_DISPLAY_STRING_LENGTH];
  char firmwareVersion [M3_SWITCH_TEST_DISPLAY_STRING_LENGTH];
  char monitorFirmwareVersion [M3_SWITCH_TEST_DISPLAY_STRING_LENGTH];
  unsigned scratchPad;
  unsigned reboot;
  unsigned monitorCause;
  unsigned firstCoreDumpID;
  unsigned lastCoreDumpID;  
} m3_SwitchTestSlot;

typedef struct m3_SwitchTestPort
{
  int type;
  unsigned port;
  unsigned selfTestResult;
  unsigned invalidRoutes;
  unsigned illegalSymbols;
  unsigned missedBeats;
  unsigned deadPorts;
  unsigned long long goodCrcs;
  unsigned long long badCrcs;
  unsigned dataIn;
  unsigned dataOut;
  unsigned portDownOnIllegalSymbol;
  unsigned portDownOnIllegalSymbolCount;
  unsigned portDownOnMissedBeat;
  unsigned portDownOnMissedBeatCount;
  unsigned portDown;
  unsigned portUpCount;
  unsigned receiveTimeout;
  unsigned receiveTimeoutCount;
  unsigned transmitTimeout;
  unsigned transmitTimeoutCount;
  unsigned control;

  unsigned overruns;
  unsigned sanFault;
  unsigned sanFaultCount;
  unsigned serdesFault;
  unsigned serdesFaultCount;
  unsigned notInSync;
  unsigned notInSyncCount;

  unsigned signalLost;
  unsigned signalLostCount;

  unsigned illegalSymbolOnLanCount;
  unsigned portDownForLanPortCount;
  unsigned portDownCount;
  unsigned portFast;
  unsigned noCable;
  unsigned missedBeatForLanPort;
  unsigned illegalOnLan;
  unsigned illegalOnSan;
  unsigned portOK;
  
} m3_SwitchTestPort;

typedef struct m3_SwitchTestTemperature
{
  int selfTestResult;
  int temperature;
};

typedef struct m3_SwitchTestVoltage
{
  int type;
  int voltage;
  int nominal;
} m3_SwitchTestVoltage;

class m3_SwitchTestPart
{
  public:
  int type;
  int typeID;
};

class m3_SwitchTestFan
{
  public:
  int speed;
  int minimumSpeed;
  int bad;
  int badCount;
  int trayMissing;
  int trayMissingCount;
  int loicPrylli;
};


class m3_SwitchTestCurrent
{
  public:
  int overcurrentFault;
  int current;
};

struct m3_SwitchTestSlot;
class m3_SwitchTest: public m3_Component
{
  public:
  enum
  {
    MAX_SWITCH_NAME = 255,
    MAX_VOLTAGES = 8,
    MAX_PORTS = 16,
    MAX_PARTS = 100,
    MAX_TEMPERATURES = 8,
    MAX_FANS = M3_MAX_FAN,
    MAX_CURRENTS = M3_MAX_CURRENT,
    XBAR = M3_XBAR_PORT_TABLE,
    HSSDC = M3_HSSDC_PORT_TABLE,
    SHORTWAVE_FIBER = M3_SHORTWAVE_FIBER_PORT_TABLE,
    LAN = M3_LAN_PORT_TABLE,
    VOLTAGE_33 = M3_VOLTAGE_33,
    VOLTAGE_66 = M3_VOLTAGE_66,
    VOLTAGE_132 = M3_VOLTAGE_132,
    NUM_XBAR_PORT_VALUES = 21,
    NUM_HSSDC_PORT_VALUES = 10,
    NUM_SHORTWAVE_FIBER_PORT_VALUES = 12,
    NUM_SLOT_VALUES = 7,
    NUM_UC_VALUES = 16,
    NUM_VOLTAGE_VALUES = 4,
    NUM_TEMPERATURE_VALUES = 3,
    NUM_BIG_UC_VALUES = 22,
    NUM_FAN_VALUES = 7,
    NUM_LAN_PORT_VALUES = 13,
    NUM_CURRENT_VALUES = 3
  };

  private:
  m3_SwitchTestSlot slot;
  m3_SwitchTestPart parts [MAX_PARTS];
  void*snmp;
  int numSwitches;
  char*namesspace;
  char**names;
  m3_SwitchTestPort ports [MAX_PORTS];
  int getXbarPort (int target, int id, m3_SwitchTestPort*port);
  int getHssdcPort (int target, int id, m3_SwitchTestPort*port);
  int getShortwaveFiberPort (int target, int id, m3_SwitchTestPort*port);
  int setIntegerOid (int target, char*oid, unsigned int value);
  int getTemperature (int target, int id, m3_SwitchTestTemperature*temperature);
  int getVoltage (int target, int id, m3_SwitchTestVoltage*voltage); 
  int getIntegerOid (int target, char*oid);
  int getFan (int target, int id, m3_SwitchTestFan*fan);
  int getLanPort (int target, int id, m3_SwitchTestPort*port);
  int getCurrent (int target, int id, m3_SwitchTestCurrent*current);
  
  public:  
  m3_SwitchTest (int numSwitches, ...);
  m3_SwitchTest (int numSwitches, char*names[]);
  ~m3_SwitchTest ();
  int getPortCounts (int target, int slot, int firstPort, int numPorts, m3_SwitchTestPort*ports);
  /*c = a - b. returns 0 if a != b*/
  int getPortDifference (m3_SwitchTestPort*a, m3_SwitchTestPort*b, m3_SwitchTestPort*c);
  int getSlot (int target, int slotNumber, m3_SwitchTestSlot*slot);
  int getRefreshes (int target);
  int setPortControl (int target, int slot, int port, int value);
  int setXbarTimeout (int target, int slot, int value);
  int getVoltages (int target, int slot, int numVoltages, m3_SwitchTestVoltage*voltages);
  int getTemperatures (int target, int slot, int numTemperatures, m3_SwitchTestTemperature*temperatures); 
  int getXbarSelfTestResult (int target, int slot);
  int getLineCardVoltages (int target, int slotNumber, unsigned*voltageA, unsigned*voltageB);
  int getBigUc (int target, m3_SwitchTestBigUc*bigUc);
  int getParts (int target, int id, int maxParts, m3_SwitchTestPart*parts);
  int getPartsOld (int target, int id, int maxParts, m3_SwitchTestPart*parts);
  int getParts (int target, int id, m3_SwitchTestPart*parts, int firstPartID, int lastPartID);
  int getFans (int target, int slot, int numFans, m3_SwitchTestFan*fans);
  int getCurrents (int target, int slot, int numCurrents, m3_SwitchTestCurrent*currents);
};

#endif /*m3_SwitchTest_h*/
