/*
  m3_TestClient.h
  class to access switch test port
  finucane@myri.com (David Finucane)
*/

#ifndef m3_TestClient_h
#define m3_TestClient_h

#include <stdarg.h>
#include "m3_defines.h"
#include "m3_Component.h"


#define M2_GET_PORT_STATE 36

typedef struct m2_port_state_message_t
{
  int type;
  int xbar;
  int port;
  int uptime;
  int good_low;
  int good_high;
  int bad_low;
  int bad_high;
  int timeouts;
  int illegal_symbols;
  int missed_beats;
  int bad_routes;
  int dead_routes;
  int port_state;
  int control;
} m2_port_state_message_t;

class m3_TestClient : public m3_Component
{
  private:
  char buffer [M3_TEST_MTU];

  protected:
  int fd;
  int ipAddress;

  int send (char*m, int size);
  int receive (char*m, int size);
  int read (int fd, char*buffer, int length);
  int send (char*request, int requestLength, char*response, int responseLength);
  int simpleCommand (int xbar, int type);
  int simpleCommand (int xbar, int type, int integer);
  int integerCommand (int xbar, int type);
  int integerCommand (int xbar, int type, int integer);
  int m2_getPortStates (m2_port_state_message_t*psm);

  public:
  int close ();
  int read (char*buffer, int length);
  int signaled;
  int open (int address);
  int open (char*switchname);
  m3_TestClient (char*switchname=0);
  int getVersionString (char*version);
  int reboot ();
  int debug ();
  int startUpdate ();
  int stopUpdate ();
  int getUptime ();
  int getExtendedCount ();
  int getExtendedValue (int index);
  int getEthernetAddress (char*address);  
  int getActiveEthernetPort ();  
  int setActiveEthernetPort (int port);
  int setNotify (char ip_address [4], int port, int types1_32, int types33_64);
  int testLink (int mtu);
  int serial (int*result, int port, int command, int txlen, char*txbuf, int rxlen, char*rxbuf);
  int stream (int*result, int port, int command, int txlen, char*txbuf, int rxlen, char*rxbuf);
  void printBuffer (char*p, int len);
  int getInvalidRouteXbar ();
  int setWebPassword (char*username, char*password);
  int disableTask (char*taskname);
  int getPacketCounters (m3_int64**good, m3_int64**bad);

  int m2_getPacketCounters (m3_int64*good, m3_int64*bad);
  int m2_getInvalidRoutes (int*invalidRoutes, int*port);
  
  static char*firmwareGetDate (char*firmwareVersion, char*date);
  static char*firmwareGetType (char*firmwareVersion, char*type);
  static char*firmwareGetFilename (char*firmwareVersion, char*separator, char*dir, char*name);
  static int firmwareDateParse (char*date, int*month, int*day, int*year, int*hour, int*minute, int*second);
  static int firmwareDateCompare (char*date1, char*date2);

};


#endif /*m3_TestClient_h*/
