/*
  m3_WebClient.c
  finucane@myri.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "m3_WebClient.h"
#include "insist.h"

m3_WebClient::m3_WebClient ()
{
  fd = 0;
  tp = 0;
  
  if (!(buffer = (char*) malloc (size = 1000 * 1000)))
    size = 0;
}

m3_WebClient::~m3_WebClient ()
{
  if (buffer) free (buffer);
}

int m3_WebClient::open (int server)
{
  insist (this && server);

  struct sockaddr_in sa;
  memset (&sa, 0, sizeof (sa));
  sa.sin_family = AF_INET;
  sa.sin_port = htons (80);
  sa.sin_addr.s_addr = server;
  
  if ((fd = socket (PF_INET, SOCK_STREAM, 0)) < 0)
  {
    printFormat ("socket failed");
    return fd = 0;
  }

  if (connect (fd, (struct sockaddr*) &sa, sizeof (sa)) < 0)
  {
    printFormat ("connect failed");
    return 0;
  }

  return fd;
  exception: return 0;
}

int m3_WebClient::close ()
{
  insist (this);
  if (fd)
  {
    ::close (fd);
    fd = 0;
  }
  return 1;
  exception: return 0;
}


int m3_WebClient::get (int fd)
{
  insist (this && fd);
  
  char*s;
  s = buffer;
  bytesRead = 0;
  
  for (;;)
  {
    int left = size - bytesRead;
    
    if (left / 2 < 2)
    {
      char*t;
      t = (char*) malloc (size *= 2);
      insistp (t, ("malloc failed"));
      memcpy (t, buffer, bytesRead);
      s = t + (s - buffer);
      free (buffer);
      buffer = t;
      left = size - bytesRead;
    }
    
    int n = read (fd, s, left / 2);
    if (n <= 0) break;
    bytesRead += n;
    s += n;
  }
  insist (size - bytesRead >= 2);
  buffer [bytesRead] = buffer [bytesRead + 1]  = 0;
  return 1;
  exception: return 0;
}


char*m3_WebClient::get (int server, char*url)
{
  insist (this && buffer && size > 1000);
  insist (server && url);

  tp = 0;

  if (!open (server))
    return 0;

  int numBytes;
  numBytes = sprintf (buffer, "GET %s HTTP/1.0\r\n", url);

  if (write (fd, buffer, numBytes) != numBytes)
  {
    printFormat ("write failed");
    return 0;
  }
  
  if (!get (fd))
  {
    close ();
    return 0;
  }
  
  close ();
  return tokenize ();
  exception: return 0;
}

char*m3_WebClient::post (int server, char*url, int numVariables, ...)
{
  va_list ap;
  char*s;
  
  char b [100];
  
  insist (this && buffer && size > 1000);
  insist (server && url);
  insist (numVariables > 0);
  
  tp = 0;

  if (!open (server))
    return 0;

  int numBytes;
  numBytes = sprintf (buffer, "POST %s HTTP/1.0\r\n", url);

  if (write (fd, buffer, numBytes) != numBytes)
  {
    printFormat ("write failed");
    return 0;
  }

  *buffer = 0;
  s = buffer;
  
  va_start (ap, numVariables);

  for (int i = 0; i < numVariables; i++)
  {
    char*s1 = va_arg (ap, char*);
    char*s2 = va_arg (ap, char*);
    
    s += sprintf (s, "%s%s=%s", i ? "&" : "", s1, s2);
  }
  
  va_end (ap);
  numBytes = sprintf (b, "content-length: %d\r\n", (int) (s - buffer));

  if (write (fd, b, numBytes) != numBytes)
  {
    printFormat ("write of content-length failed");
    return 0;
  }

  sprintf (b, "\r\n");
  if (write (fd, b, 2) != 2)
  {
    printFormat ("write of CRLF failed");
    return 0;
  }

  numBytes = (int) (s - buffer);

  if (write (fd, buffer, numBytes) != numBytes)
  {
    printFormat ("write of post data failed");
    return 0;
  }
  
  if (!get (fd))
  {
    close ();
    return 0;
  }
  
  close ();
  return tokenize ();
  exception: return 0;
}

char*m3_WebClient::get (char*filename)
{
  insist (this && buffer && size > 1000);

  tp = 0;
  int fd;
  fd = ::open (filename, O_RDONLY, 0);
  if (fd <= 0)
  {
    m3_Component::printFormat ("couldn't open \"%s\"", filename);
    return 0;
  }
 
  if (!get (fd))
  {
    ::close (fd);
    return 0;
  }
  
  ::close (fd);
  return tokenize ();
  exception: return 0;
}

int m3_WebClient::getServer (char*server)
{
  struct hostent *he;
  int sin_addr;
  
  insistf (server);
  
  if ((he = gethostbyname (server)))
  {
    insist (he->h_length == sizeof (sin_addr));
    memcpy ((char*)&sin_addr, he->h_addr, he->h_length);
  }
  else if ((sin_addr = inet_addr (server)) == (unsigned) -1)
  {
    m3_Component::printFormat ("can't get IP address of %s", server);
    return 0;
  }

  return sin_addr;
  exception: return 0;
}

char*m3_WebClient::get (char*server, char*url)
{
  return get (getServer (server), url);
  exception: return 0;
}

char*m3_WebClient::token ()
{
  insist (this);
  char*t;
  
  while (tp < buffer + bytesRead && !*tp)
    tp++;
  
  if  (tp >= buffer + bytesRead)
    return 0;
  
  t = tp;
  tp += strlen (tp) + 1;
  return t;
  exception: return 0;
}


char*m3_WebClient::getMark ()
{
  return tp;
}

char*m3_WebClient::setMark (char*s)
{
  insist (this && s == 0 || (s >= buffer && s <= buffer + bytesRead));
  
  return tp = s ? s : buffer;
  exception: return 0;
}

    
char*m3_WebClient::tokenize ()
{
  insist (this);
  
  for (char*t = buffer; t < buffer + bytesRead;)
  {
    char*tt;
    
    if (isspace (*t))
      *t++ = 0;
    else if (*t == '<' && (tt = strchr (t, '>')))
    {
      while (t <= tt)
	*t++ = 0;
    }
    else t++;
  }
  
  return tp = buffer;
  exception: return 0;
}
