/*
  m3_XbarFinder.c
  class to talk to the switch
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

#include "mt_Network.h"
#include "m3_XbarFinder.h"
#include "m3_defines.h"
#include "insist.h"

m3_XbarFinder::m3_XbarFinder ()
{
  port = 0;
  fd = 0;
 
  open ();
}

int m3_XbarFinder::open ()
{
  char hostname [256];
  struct hostent*hp;
  socklen_t length;
  struct sockaddr_in a;

  
  fd = socket (AF_INET, SOCK_DGRAM, 0);
  if (fd < 0)
  {
    printFormat ("failed opening datagram socket");
    return 0;
  }
  a.sin_family = AF_INET;
  a.sin_addr.s_addr = INADDR_ANY;
  a.sin_port = 0;
      
  if (bind (fd, (struct sockaddr*) &a, sizeof (a)) < 0)
  {
    printFormat ("binding datagram socket failed");
    ::close (fd);
    return fd = 0;
  }
      
  length = sizeof (a);
  if (getsockname (fd, (struct sockaddr*) &a, &length) < 0)
  {
    printFormat ("getsockname failed");
    ::close (fd);
    return fd = 0;
  }
      
  if (gethostname (hostname, 255) < 0 || !(hp = gethostbyname (hostname)))
  {
    printf ("couldn't get our ip address\n");
    ::close (fd);
    return fd = 0;
  }

  if (hp->h_addr [0] == 127 && hp->h_addr [1] == 0 && hp->h_addr [2] == 0 && hp->h_addr [3] == 1)
    memset (address, 0, 4);
  else
    memcpy (address, hp->h_addr, 4);
  return port = ntohs (a.sin_port);

  exception: return 0;
}

m3_XbarFinder::~m3_XbarFinder ()
{
  close ();
}

int m3_XbarFinder::close ()
{
  insist (this);
  
  if (fd)
    ::close (fd);
  return 1;
  exception: return 0;
}

int m3_XbarFinder::setNotify ()
{
  int*addresses = 0;
  int r, numAddresses;
  
  insist (this);  

  if (!rollcall.open ())
    return 0;

  r = (numAddresses = rollcall.findAddresses (&addresses)) && setNotify (addresses, numAddresses);

  rollcall.close ();
  
  return r;
  
  exception: return 0;
}

int m3_XbarFinder::receive (int usecs, char ethernetAddress [6], int*xbar)
{
  fd_set mask;
  struct timeval timeout;
  struct sockaddr_in sa;
  socklen_t length;
  m3_notify_message_t m;

  insist (this && xbar && ethernetAddress);
  
  for (;;)
  {
    timeout.tv_sec = 0;
    timeout.tv_usec = usecs;
  
    FD_ZERO (&mask);
    FD_SET (fd, &mask);

    if (select (1 + fd, &mask, 0, 0, &timeout) != 1)
      break;

    length = sizeof (sa);
    if (recvfrom (fd, (char*) &m, sizeof (m), 0, (struct sockaddr*) &sa, &length) == sizeof (m) && usecs && htonl (m.type) == M3_invalidRouteTrap)
    {
      *xbar = htonl (m.index);
      memcpy (ethernetAddress, m.ethernet_address, 6);
      return sa.sin_addr.s_addr;  
    }
  }
  exception: return 0;
}

int m3_XbarFinder::setNotify (int*addresses, int numAddresses)
{
  insist (this && fd && addresses && numAddresses);

  for (int i = 0; i < numAddresses; i++)
  {
    if (!test.open (addresses [i]) || !test.setNotify (address, port, 1 << (M3_invalidRouteTrap - 1), 0))
      return 0;
  }
  return 1;
  exception: return 0;
}

int m3_XbarFinder::setNotify (char**addresses, int numAddresses)
{
  insist (this && fd && addresses && numAddresses);

  for (int i = 0; i < numAddresses; i++)
  {
    if (!test.open (addresses [i]) || !test.setNotify (address, port, 1 << (M3_invalidRouteTrap - 1), 0))
      return 0;
  }
  return 1;
  exception: return 0;
}

int m3_XbarFinder::send (mt_Job*job, mt_Route*route, int usecs)
{
  mt_Network*network;
  int e, length;
  char buffer [32];
  char*p;
  
  memset (buffer, 0, 32);
  
  insist (this && job && route && usecs);
  
  network = job->getNetwork ();
  insist (network);

  if (!network->send (job, route, buffer, 32))
  {
    printFormat ("send failed");
    return 0;
  }

  network->setTimer (job, usecs);

  while ((e = network->wait (&p, &length)) != mt_Network::TIMEOUT);
  /*
  {
    if (e == mt_Network::SEND_DONE)
    {
      network->clearTimer (job);
      return 1;
    }
  }
  */
  return 1;
  printFormat ("send done failed");
  exception: return 0;
}

int m3_XbarFinder::find (mt_Job*job, mt_Route*route, char ipAddress [4], char ethernetAddress [6], int*xbar, int sendUsecs, int receiveUsecs)
{
  int a, length;
  insist (this && job && route && xbar && sendUsecs && receiveUsecs);

  receive (0, ethernetAddress, xbar);

  if (!send (job, route, sendUsecs) || !(a = receive (receiveUsecs, ethernetAddress, xbar)))
    return 0;

  memcpy (ipAddress, &a, 4);

  return 1;
  
  exception: return 0;    
}
