#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "mt_Network.h"
#include "m3_XbarTable.h"
#include "insist.h"

#define M3_XBAR_TABLE_MAXLINE 100
#define M3_XBAR_TABLE_PRINTF_STRING "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%d:%d"
#define M3_XBAR_TABLE_SCANF_STRING "%x:%x:%x:%x:%x:%x:%d:%d"

m3_XbarTableEntry::m3_XbarTableEntry ()
{
  node = 0;
  memset (ipAddress, 0, 4);
  memset (ethernetAddress, 0, 6);
  xbar = -1;
  correction = 0;
}

m3_XbarTable::m3_XbarTable (int numNodes)
{
  insist (numNodes > 0);
  numEntries = numNodes;
  entries = new m3_XbarTableEntry [numNodes];
  insist (entries);
  exception:;
}

m3_XbarTable::~m3_XbarTable ()
{
  insist (this);
  if (entries)
    delete entries;
  exception:;
}

int m3_XbarTable::fromFile (mt_Node*root, mt_Graph*g, char*filename)
{
  insist (this && root && g && filename);
  insist (entries && numEntries == g->getNumSwitches ());

  FILE*fp;
  char name [mt_Network::HOSTNAME_LENGTH];
  
  int fat [6];
  int numFound, lineno;
  char line [M3_XBAR_TABLE_MAXLINE + 1];
  char*s;
  
  fp = fopen (filename, "r");
  if (fp <= 0)
  {
    mt_Component::printFormat ("couldn't open %s", filename);
    return 0;
  }
  
  lineno = numFound = 0;
  int fluck;
  
  while (s = fgets (line, M3_XBAR_TABLE_MAXLINE, fp))
  {
    lineno++;
    if (numFound > numEntries)
    {
      mt_Component::printFormat ("too many entries");
      fclose (fp);
      return 0;
    }

    if (8 == sscanf (s, M3_XBAR_TABLE_SCANF_STRING,
		     &fat [0],
		     &fat [1],
		     &fat [2],
		     &fat [3],
		     &fat [4],
		     &fat [5],
		     &entries [numFound].xbar,
		     &entries [numFound].correction))
    {
      mt_Node*o, *n;
      mt_Route r;

      if (!(s = fgets (line, M3_XBAR_TABLE_MAXLINE, fp)))
      {
	mt_Component::printFormat ("unexpected end of file");
	fclose (fp);
	return 0;	
      }

      lineno++;
      
      while (*s && isspace (*s))
	s++;
      
      r.fromString (s);
      n = root->getNode (&r);

      if (!n || !n->isSwitch ())
      {
	mt_Component::printFormat ("line %d: %s does not lead from %s to a switch",
				   lineno, s, root->getName ());
	fclose (fp);
	return 0;
      }
      
      entries [numFound].node = n;
      if (entries [numFound].xbar <= 0 || entries [numFound].xbar > 64)
      {
	mt_Component::printFormat ("line %d: bad xbar %d\n", lineno, entries [numFound].xbar);
	fclose (fp);
	return 0;
      }
      
      for (int i = 0; i < 6; i++)
	entries [numFound].ethernetAddress [i] = fat [i];

      if (lookup (entries [numFound].ethernetAddress, entries [numFound].xbar) != &entries [numFound] ||
	  lookup (entries [numFound].node) != &entries [numFound])
      {
	mt_Component::printFormat ("line %d: duplicate entry", lineno);
	fclose (fp);
	return 0;	
      }
      
      numFound++;
    }
    else
    {
      mt_Component::printFormat ("line %d: syntax error", lineno);
      fclose (fp);
      return 0;	
    }
  }
 
  if (numFound != numEntries)
  {
    mt_Component::printFormat ("not enough entries");
    return 0;
  }  
  
  fclose (fp);
  return 1;
  
  exception: return 0;
}

int m3_XbarTable::toFile (char*filename)
{
  FILE*fp;
  insist (this && entries && numEntries > 0 && filename);

  fp = fopen (filename, "w+");
  insist (fp);

  for (int i = 0; i < numEntries; i++)
  {
    insist (entries [i].node);
    fprintf (fp, M3_XBAR_TABLE_PRINTF_STRING "\n",
	     (unsigned char) entries [i].ethernetAddress [0] & 0xff,
	     (unsigned char) entries [i].ethernetAddress [1] & 0xff,
	     (unsigned char) entries [i].ethernetAddress [2] & 0xff,
	     (unsigned char) entries [i].ethernetAddress [3] & 0xff,
	     (unsigned char) entries [i].ethernetAddress [4] & 0xff,
	     (unsigned char) entries [i].ethernetAddress [5] & 0xff,
	     entries [i].xbar,
	     entries [i].correction);

    fprintf (fp, "%s\n", entries[i].node->getRoute (0)->toString ());
  }

  fclose (fp);

  return 1;
  exception: return 0;
}

int m3_XbarTable::fromNetwork (mt_Job*job, mt_Graph*g, int*addresses, int numAddresses)
{
  insist (this && job && g);
  insist (entries && numEntries == g->getNumSwitches ());

  finder.setNotify (addresses, numAddresses);  
  
  for (int i = 0; i < g->getNumSwitches (); i++)
  {
    mt_Route*r;
    mt_Node*n = g->getSwitch (i);
    insist (n);
    
    r = n->getRoute (0);
    insist (r);

    mt_Component::printFormat ("%s", n->getName ());
    
    for (int j = 1; j < 1000; j++)
    {
      int xbar_port;
      
      if (finder.find (job, r, entries [i].ipAddress, entries[i].ethernetAddress, &xbar_port, 1000, 1000 * (j / 100 + 1)))
      {
	int inPort = (xbar_port - 1) % 16;	

	entries [i].xbar = 1 + (xbar_port - 1) / 16;
	entries [i].node = n;
	
	if (lookup (entries [i].ethernetAddress, entries [i].xbar) != &entries [i] ||
	    lookup (entries [i].node) != &entries [i])
	{
	  mt_Component::printFormat ("duplicate or late message. Retrying.");
	  entries [i].node = 0;
	  continue;
	 
	}
	int in = g->getRoot()->getIn (r);
	insist (in >= 0 && in < 16);
	entries [i].correction = inPort - in;
	if (entries [i].correction < 0 || entries [i].correction >= 16)
	{
	  mt_Component::printFormat ("chaotic response, %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x xbarPort %d",
				     (unsigned) (entries [i].ethernetAddress [0] & 0xff),
				     (unsigned) (entries [i].ethernetAddress [1] & 0xff),
				     (unsigned) (entries [i].ethernetAddress [2] & 0xff),
				     (unsigned) (entries [i].ethernetAddress [3] & 0xff), 
				     (unsigned) (entries [i].ethernetAddress [4] & 0xff), 
				     (unsigned) (entries [i].ethernetAddress [5] & 0xff), xbar_port);
	  
	  mt_Component::printFormat ("retrying.");
	  entries [i].node = 0;
	  continue;
	}
	break;
      }
    }
    if (!entries [i].node)
    {
      mt_Component::printFormat ("didn't find %s", n->getName ());
      finder.close ();
      return 0;
    }
  }
  finder.close ();
  return 1;
  exception: return 0;
}

m3_XbarTableEntry*m3_XbarTable::lookup (char ethernetAddress [6], int xbar)
{
  insist (this && ethernetAddress && entries && numEntries > 0);
 
  for (int i = 0; i < numEntries; i++)
    if (entries [i].xbar == xbar && !memcmp (entries [i].ethernetAddress, ethernetAddress, 6))
      return &entries [i];
  
  exception: return 0;
}

m3_XbarTableEntry*m3_XbarTable::lookup (mt_Node*node)
{
  insist (this && node && entries && numEntries > 0);
  for (int i = 0; i < numEntries; i++)
    if (entries [i].node == node)
      return &entries [i];
  
  exception: return 0;
}

int m3_XbarTable::equals (m3_XbarTable*table)
{
  insist (this && table);
  m3_XbarTableEntry*e;
  
  if (numEntries != table->numEntries)
    return 0;
  
  for (int i = 0; i < numEntries; i++)
  {
    if (!(e = table->lookup (entries [i].node)) ||
	e->node != entries [i].node ||
	memcmp (e->ethernetAddress, entries [i].ethernetAddress, 6) ||
	memcmp (e->ipAddress, entries [i].ipAddress, 4) ||
	e->xbar != entries [i].xbar ||
	e->correction != entries [i].correction)
      return 0;
  }
  return 1;
  exception: return 0;
}
