/*
  ming.c
  m3 + ping
  finucane@myri.com
 */


#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>


#include "m3_XbarFinder.h"
#include "m3_Description.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"

#include "mt_Graph.h"
#include "mt_MapFile.h"
#include "mt_Network.h"
#include "pi_Ping.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static void usage ()
{
  printf ("usage: ming <unit> <mapfile> <hostname> [switchnames]\n");
  exit (1);
}

#define MAX_FIND_TRIES 3

int main (int argc, char**argv)
{
  mt_Component::initialize (printLine);
  m3_Component::initialize (printLine);
  m3_RollcallClient rollcall;
  mt_Network*network;
  mt_Graph graph;
  mt_Node*myself;
  m3_XbarFinder finder;
  
  pi_Options options;
  char*mapfile,*hostname;
  int unit;

  if (argc != 4)
    usage ();

  options.unit = atoi (argv [1]);
  mapfile = argv [2];
  hostname = argv [3];  

  if (options.unit < 0)
  {
    printf ("%s is a bad unit\n", argv [1]);
    usage ();
  }

  mt_MapFile mf (mapfile, mt_File::_READ);
  
  if (!mf.read (&graph))
  {
    printf ("couldn't read %s\n", mapfile);
    return 1;
  }
  
  if (!(myself = graph.getNode (hostname)))
  {
    printf ("node \"%s\" is not in %s", hostname, mapfile);
    return 1;
  }

  if (!graph.computeRoutes (myself))
  {  
    printf ("couldn't compute routes");
    return 1;
  }
  
  int ipAddresses [graph.getNumSwitches ()];
  int xbars [graph.getNumSwitches ()];
  
  network = mt_getNetwork ();
  if (!network)
  {
    printf  ("getNetwork failed\n");
    return 1;
  }
  
  strncpy (options.myself, hostname, mt_Network::HOSTNAME_LENGTH);
  strncpy (options.mapFile, mapfile, mt_Network::HOSTNAME_LENGTH);
  options.testXbars = 1;
  options.testHosts = 0;
  options.count = 3;
  options.size = 100;
  options.careful = 1;
  options.verbose = 1;
  options.printXbars = 1;
  
  pi_Ping*ping = new pi_Ping (myself, network, mt_getCalculator (), &options);
  if (!ping)
  {
    printf ("new ping failed\n");
    return 1;
  }
  ping->start ();
  ping->print ();

  if (!finder.setNotify ())
    return 0;

  printf ("finding xbars\n");
  for (int i = 0; i < graph.getNumSwitches (); i++)
  {
    printf ("%s", graph.getSwitch (i)->getName ());
    char e [6];
    
    int j;
    for (j = 1; j < MAX_FIND_TRIES; j++)
    {
      if (finder.find (ping, graph.getSwitch (i)->getRoute (0), (char*) &ipAddresses [i], e, &xbars [i], 1000 * 100, j * 1000 * 1000))
      {
	xbars [i] = 1 + (xbars [i] - 1) / 16;
	
	printf (" %d.%d.%d.%d:%d",
		(unsigned) (((char*)&ipAddresses [i]) [0] & 0xff),
		(unsigned) (((char*)&ipAddresses [i]) [1] & 0xff),
		(unsigned) (((char*)&ipAddresses [i]) [2] & 0xff),
		(unsigned) (((char*)&ipAddresses [i]) [3] & 0xff),
		xbars [i]);
	break;
      }
    }
    if (j == MAX_FIND_TRIES)
    {
      printf (" couldn't find xbar");
      //return 1;
    }
    
    printf ("\n");
  }

  for (int i = 0; i < graph.getNumSwitches (); i++)
  {
    for (int j = 0; j < 16; j++)
    {
      for (int k = 0; k < 16; k++)
      {
	int numSent, numReceived;
	if (!ping->getCount (i, j, k, &numSent, &numReceived))
	  return 1;
	if (numSent != numReceived)
	{
	  printf ("%d/%d %d.%d.%d.%d:%d:%d.%d-%d\n", numReceived, numSent, 
		  (unsigned) (((char*)&ipAddresses [i]) [0] & 0xff),
		  (unsigned) (((char*)&ipAddresses [i]) [1] & 0xff),
		  (unsigned) (((char*)&ipAddresses [i]) [2] & 0xff),
		  (unsigned) (((char*)&ipAddresses [i]) [3] & 0xff),
		  xbars [i],
		  j, k);
	}
      }
    }
  }
  return 0;
}
