/*
  old_counts.c
  thing to print out useful port numberings with crc counts for m2 closses and spines.
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#include <sys/time.h>
#include <sys/types.h>


#include "m3_TestClient.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static m3_TestClient*client = 0;
static void signal_handler (int)
{
  if (client)
    client->signaled = 1;
}

static void usage (char*s)
{
  printf ("usage: %s <good | bad> <m2 switch names>\n", s);
  exit (1);
}

void printNumbering ()
{
  printf ("\n\n       M2 Clos / Spine Port Number Chart\n");
  printf ("\n");
  printf ("       Clos front (lan ports):\n");
  printf ("\n");
  printf (" xbar  xbar  xbar  xbar  xbar  xbar  xbar  xbar\n");
  printf ("   7     6     5     4     3     2     1     0\n");
  printf (" ----  ----  ----  ----  ----  ----  ----  ----\n");
  printf ("12  8 12  8 12  8 12  8 12  8 12  8 12  8 12  8\n");
  printf ("13  9 13  9 13  9 13  9 13  9 13  9 13  9 13  9\n");
  printf ("14 10 14 10 14 10 14 10 14 10 14 10 14 10 14 10\n");
  printf ("15 11 15 11 15 11 15 11 15 11 15 11 15 11 15 11\n");
  printf ("\n");
  printf ("       Clos back (san ports):\n");
  printf ("\n");
  printf (" xbar  xbar  xbar  xbar  xbar  xbar  xbar  xbar\n");
  printf ("   8     9    10    11    12    13    14    15\n");
  printf (" ----  ----  ----  ----  ----  ----  ----  ----\n");
  printf ("13  9 13  9 13  9 13  9 13  9 13  9 13  9 13  9 \n");
  printf ("12  8 12  8 12  8 12  8 12  8 12  8 12  8 12  8 \n");
  printf ("\n");
  printf ("15 11 15 11 15 11 15 11 15 11 15 11 15 11 15 11 \n");
  printf ("14 10 14 10 14 10 14 10 14 10 14 10 14 10 14 10 \n");
  printf ("\n");
  printf ("       Spine back (san \"A\" link on even xbars):\n");
  printf ("\n");
  printf ("   xbar          xbar          xbar          xbar\n");
  printf ("   0+1           2+3           4+5           5+7\n");
  printf ("   ----          ----          ----          ----\n");
  printf ("14 12 10  8   14 12 10  8   14 12 10  8   14 12 10  8\n");
  printf ("15 13 11  9   15 13 11  9   15 13 11  9   15 13 11  9\n");
  printf (" 7  5  3  1    7  5  3  1    7  5  3  1    7  5  3  1   \n");
  printf (" 6  4  2  0    6  4  2  0    6  4  2  0    6  4  2  0\n");
  printf ("\n");
}


typedef struct row_t
{
  int _switch;
  int xbar;
  int port;
  unsigned long long good;
  unsigned long long bad;
}row_t;


static int compare (const void*a, const void*b)
{
  return ((row_t*)a)->bad > ((row_t*)b)->bad ? 1 : (((row_t*)a)->bad < ((row_t*)b)->bad ? -1 : 0);
}

int main (int argc, char**argv)
{
  m3_Component::initialize (printLine);
  m3_TestClient client;
  int numSwitches = 0;
  char**switchNames = argv + 1;
  int numRows = 0;
  
  if (argc < 2)
    usage (argv [0]);
  
  numSwitches = argc - 1;

  printf ("good packet / bad crc counts for %d m3 switch%s sorted by bad crcs\n", numSwitches, numSwitches != 1 ? "es" : "");

  int numPorts [numSwitches];
  m3_int64 good [16 * 16];
  m3_int64 bad [16 * 16];;
  row_t rows [numSwitches * 16 * 16];
  
  for (int i = 0; i < numSwitches; i++)
  {
    printf ("get-port-states for switch %s\n", switchNames [i]);
    fflush (stdout);
    
    if (!client.open (switchNames [i]))
    {
      printf ("open failed for switch %s", switchNames [i]);
      return 0;  
    }
    if (!(numPorts [i] = client.m2_getPacketCounters (good, bad)))
    {
      printf ("getPacketCounters failed for switch %s", switchNames [i]);
      return 0;  
    }
    for (int j = 0; j < numPorts [i]; j++, numRows++)
    {
      rows [numRows]._switch = i;
      rows [numRows].xbar = j / 16;
      rows [numRows].port = j % 16;
      rows [numRows].good = (((unsigned long long) good [j].high) << 32) + ((unsigned) good[j].low);
      rows [numRows].bad = (((unsigned long long) bad[j].high) << 32) + ((unsigned) bad[j].low);
    }
  }
  
  printNumbering ();
  qsort (rows, numRows, sizeof (row_t), &compare);
  
  for (int i = numRows - 1; i >= 0; i--)
  {
    printf ("%s xbar %d port %d %llu / %llu \n", switchNames [rows [i]._switch], rows [i].xbar, rows [i].port, rows [i].good, rows [i].bad);
  }  
}
