/*

  Copyright (c) 1999
  Hewlett-Packard Company

  ATTENTION: USE OF THIS SOFTWARE IS SUBJECT TO THE FOLLOWING TERMS.
  Permission to use, copy, modify, distribute and/or sell this software
  and/or its documentation is hereby granted without fee. User agrees
  to display the above copyright notice and this license notice in all
  copies of the software and any documentation of the software. User
  agrees to assume all liability for the use of the software; Hewlett-Packard
  makes no representations about the suitability of this software for any
  purpose. It is provided "AS-IS" without warranty of any kind,either express
  or implied. User hereby grants a royalty-free license to any and all
  derivatives based upon this software code base.

*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <X11/Shell.h>
#include <Xm/MenuShell.h>

#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/ScrolledW.h>
#include <Xm/MessageB.h>
#include <Xm/TextF.h>
#include <Xm/Label.h>
#include <Xm/Form.h>

#include <snmp_pp.h>

#include "xtrapform.h"

extern Widget		UxTopLevel;
extern Snmp		*the_Session;
extern Widget		FormHandle;

static Widget           dialogWgt = NULL;

void ErrorDialog(char *errorString){
  Widget tmpWgt;
  Arg args[5];
  int i=0;
  XmString message;
  XmString title;

  message = XmStringCreateSimple(errorString);
  title = XmStringCreateSimple("Error");

    /*---------------------------------------------------------------*/
    /* Create a standard Motif message box, but hide the Cancel and  */
    /* Help buttons, because the user just gets to look at the error */
    /* no other action is available...                               */
    /*---------------------------------------------------------------*/
  if (dialogWgt == NULL){
    XtSetArg(args[i], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); i++;
    XtSetArg(args[i], XmNdialogTitle, title); i++;

    dialogWgt = XmCreateErrorDialog(FormHandle, "errorDialog", args, i);
    tmpWgt = XmMessageBoxGetChild(dialogWgt, XmDIALOG_CANCEL_BUTTON);
    XtUnmanageChild(tmpWgt);
    tmpWgt = XmMessageBoxGetChild(dialogWgt, XmDIALOG_HELP_BUTTON);
    XtUnmanageChild(tmpWgt);
  }

  XtVaSetValues(dialogWgt, 
		XmNmessageString, message, 
		NULL);

    /*-----------------------------------------*/
    /* Toss the strings... Can we do this yet? */
    /*-----------------------------------------*/
  XmStringFree(message);
  XmStringFree(title);

    /*------------------------*/
    /* Now display the window */
    /*------------------------*/
  XtManageChild(dialogWgt);
}


/*******************************************************************************
       The following are callback functions.
*******************************************************************************/

void  CQueryForm :: activateCB_pbQuery( Widget /*wgt*/, XtPointer, XtPointer)
{
   XmTextClearSelection(scrolledText,NULL);
}

void  CQueryForm :: Wrap_activateCB_pbQuery( Widget wgt, 
				        XtPointer cd, 
					XtPointer cb)
{
	CQueryForm               *UxContext;
	UxContext = (CQueryForm *) cd;

	UxContext->activateCB_pbQuery(wgt, cd, cb);
}

void  CQueryForm :: activateCB_pbQuit( Widget, XtPointer, XtPointer)
{
   exit(0);
}

void  CQueryForm :: Wrap_activateCB_pbQuit(
					Widget wgt, 
					XtPointer cd, 
					XtPointer cb)
{
	CQueryForm               *UxContext;
	UxContext = (CQueryForm *) cd;

	UxContext->activateCB_pbQuit(wgt, cd, cb);
}

void trap_callback( int reason,
					Snmp *session,
					Pdu &pdu,
					SnmpTarget &target,
					void *cbd) {
  Widget *scrolledText = (Widget*) cbd;
  char outbuff[1024];
  char scratch[256];
  GenAddress address;
  Oid trapid;
  Vb vb;
  TimeTicks timestamp;

  if ( reason == SNMP_CLASS_NOTIFICATION) {
     target.get_address( address);
	 pdu.get_notify_id( trapid);
	 pdu.get_notify_timestamp( timestamp);
	 strcpy( outbuff,"Trap From:");
	 strcat( outbuff, address.get_printable());
	 strcat( outbuff, "\n");
	 strcat( outbuff, "Trap ID:");
	 strcat( outbuff, trapid.get_printable());
	 strcat( outbuff, "\n");
 	 strcat( outbuff, "Trap Time:");
	 strcat( outbuff, timestamp.get_printable());
	 strcat( outbuff, "\n");
	 for( int z=0;z<pdu.get_vb_count();z++) { 
	    strcat( outbuff,"Vb# ");
		sprintf( scratch,"%d",z+1);
		strcat( outbuff, scratch);
		strcat( outbuff, "\n");
		pdu.get_vb(vb,z);
		sprintf( scratch,"Oid = %s\nValue= %s\n", vb.get_printable_oid(), vb.get_printable_value());
		strcat( outbuff, scratch);
     }
	 XmTextInsert( *scrolledText, XmTextGetLastPosition(*scrolledText), outbuff);
	 XmTextShowPosition( *scrolledText, XmTextGetLastPosition(*scrolledText));
  }

};

/*******************************************************************************
       The 'buildWidgets' function creates all the widgets for the
       main screen.
*******************************************************************************/

Widget CQueryForm::buildWidgets()
{
	Widget		theShell;


	// Creation of form1
	theShell = XtVaCreatePopupShell( "form1_shell",
		applicationShellWidgetClass, UxTopLevel,
		XmNx, 140,
		XmNy, 370,
		XmNwidth, 600,
		XmNheight, 380,
		XmNshellUnitType, XmPIXELS,
		XmNtitle, "Received SNMP Traps",
		NULL );


	form1 = XtVaCreateManagedWidget( "form1",
			xmFormWidgetClass,
			theShell,
			XmNwidth, 440,
			XmNheight, 380,
			XmNresizePolicy, XmRESIZE_NONE,
			XmNunitType, XmPIXELS,
			NULL );



	// Creation of scrolledWindowText
	scrolledWindowText = XtVaCreateManagedWidget( "scrolledWindowText",
			xmScrolledWindowWidgetClass,
			form1,
			XmNscrollingPolicy, XmAPPLICATION_DEFINED,
			XmNvisualPolicy, XmVARIABLE,
			XmNscrollBarDisplayPolicy, XmSTATIC,
			XmNx, 30,
			XmNy, 160,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNbottomOffset, 60,
			XmNrightAttachment, XmATTACH_FORM,
			XmNrightOffset, 10,
			XmNleftAttachment, XmATTACH_FORM,
			XmNleftOffset, 20,
			XmNtopAttachment, XmATTACH_WIDGET,
			NULL );


	// Creation of scrolledText
	scrolledText = XtVaCreateManagedWidget( "scrolledText",
			xmTextWidgetClass,
			scrolledWindowText,
			XmNwidth, 500,
			XmNheight, 160,
			XmNeditMode, XmMULTI_LINE_EDIT ,
			XmNeditable, FALSE,
			NULL );

	// start up trap reception
    XmTextInsert( scrolledText, XmTextGetLastPosition(scrolledText), "Starting Trap Reception..");
	XmTextShowPosition( scrolledText, XmTextGetLastPosition(scrolledText));
	TargetCollection targets;
	OidCollection oids;
    int status = the_Session->notify_register( oids, targets, &trap_callback,(void*)&scrolledText);    
    XmTextInsert( scrolledText, XmTextGetLastPosition(scrolledText), the_Session->error_msg(status));
	XmTextShowPosition( scrolledText, XmTextGetLastPosition(scrolledText));
	XmTextInsert( scrolledText, XmTextGetLastPosition(scrolledText), "\n");
	XmTextShowPosition( scrolledText, XmTextGetLastPosition(scrolledText));


	// Creation of pbQuery
	pbQuery = XtVaCreateManagedWidget( "pbQuery",
			xmPushButtonWidgetClass,
			form1,
			XmNx, 20,
			XmNy, 330,
			XmNwidth, 150,
			XmNheight, 40,
			RES_CONVERT( XmNlabelString, "Clear" ),
			XmNsensitive, TRUE,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftOffset, 10,
			XmNtopAttachment, XmATTACH_NONE,
			XmNtopOffset, 0,
			XmNbottomOffset, 10,
			NULL );
	XtAddCallback( pbQuery, XmNactivateCallback,
		(XtCallbackProc) &CQueryForm::Wrap_activateCB_pbQuery,
		(XtPointer) this );


	// Creation of pbQuit
	pbQuit = XtVaCreateManagedWidget( "pbQuit",
			xmPushButtonWidgetClass,
			form1,
			XmNx, 390,
			XmNy, 330,
			XmNwidth, 150,
			XmNheight, 40,
			RES_CONVERT( XmNlabelString, "Quit" ),
			XmNbottomAttachment, XmATTACH_FORM,
			XmNbottomOffset, 10,
			XmNleftAttachment, XmATTACH_NONE,
			XmNleftOffset, 0,
			XmNrightAttachment, XmATTACH_FORM,
			XmNrightOffset, 10,
			XmNtopAttachment, XmATTACH_NONE,
			XmNtopOffset, 0,
			NULL );
	XtAddCallback( pbQuit, XmNactivateCallback,
		(XtCallbackProc) &CQueryForm::Wrap_activateCB_pbQuit,
		(XtPointer) this );


	return ( theShell );
}

/*******************************************************************************
       The following is the constructor function.
*******************************************************************************/

CQueryForm::CQueryForm( Widget UxParent )
{
	this->UxParent = UxParent;
}

/*******************************************************************************
       The following is the procedural creation function.  This is used
       instead of the constructor.  No real good reason for this...
*******************************************************************************/

Widget create_form1( Widget UxParent )
{
	CQueryForm *theInterface = 
			new CQueryForm( UxParent );
	return (theInterface->buildWidgets());
}
