/*
  snmp_port.c
  program to talk to switc hthrough snmp port
  finucane@myri.com (David Finucane)
*/

#include "snmp_pp.h"
#include "eventlist.h"
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <oid.h>
#include <target.h>
#include <collect.h>

#define m3_define_trap_names
#include "m3_defines.h"

#define die(s){printf s; printf ("\n"); exit (1);}
typedef void (*dump_function) (Snmp*, CTarget*, int, int);

int make_number (char*p)
{
  int n;
  sscanf (p, "%i", &n);
  return n;
}

static char scratch_oid [255];
char*make_oid (char*format, int id)
{
  sprintf (scratch_oid, format, id);
  return scratch_oid;
}
static char scratch_value [255];
char*make_value (int d)
{
  sprintf (scratch_value, "%d", d);
  return scratch_value;
}
int last_number (char*oid)
{
  char*s;
  return (s = strrchr (oid, '.')) ? make_number (s + 1) : 0;
}

int indent_level = 0;
char*indents [] =
{
  "",
  "  ",
  "    ",
  "      ",
  "        ",
  "          ",
  "            ",
};

typedef struct object_thing
{
  char*name;
  char**names;
  int num_names;
  dump_function dump;
}object_thing;

typedef struct mask_thing
{
  char*name;
  unsigned int mask;
}mask_thing;

mask_thing masks [] =
{
  {
    "portSlow", 0x1
  },
  {
    "portOff", 0x2
  },
  {
    "portHaOff", 0x4
  },
  {
    "internalLoopback", 0x1
  },
  {
    "externalLoopback", 0x2
  },
  {
    "resetSerdesSan", 0x4
  },
  {
    "laserOff", 0x4
  },
  {
    "ledAmber", 0x1
  },
  {
    "ledGreen", 0x2
  },
  {
    "ledOff", 0x3
  },
  {
    "ledBlinkGreen", 0x4
  },
  {
    "ledBlinkAmber", 0x5
  },
  {
    "ledBlinkAmberGreen", 0x6
  },
  {
    "resetLanPort", 0x1
  }
};

#define num_masks (sizeof (masks) / sizeof (masks [0]))

int maskable (char*type)
{
  return !strcmp (type, "integer") || !strcmp (type, "integer32");
}

typedef struct control_thing
{
  char*name;
  char*type;
  char*oid;
}control_thing;

control_thing controls [] =
{
  {
    "reboot", "integer", "1.3.6.1.4.1.1771.3.1.4.1.13.%d"
  },
  {
    "shutdownForLinecard", "integer", "1.3.6.1.4.1.1771.3.1.5.1.4.%d"
  },
  {
    "shutdownForBackplane", "integer", "1.3.6.1.4.1.1771.3.1.6.1.2.%d"
  },
  {
    "shutdownForBackplaneMaster", "integer", "1.3.6.1.4.1.1771.3.1.7.1.4.%d"
  },
  {
    "timeoutForXbar16", "integer32", M3_xbar16Timeout_format
  },
  {
    "controlForXbarPort", "integer32", M3_controlForXbarPort_format
  },
  {
    "controlForHssdcPort", "integer32", M3_controlForHssdcPort_format
  },
  {
    "controlForShortwaveFiberPort", "integer32", M3_controlForShortwaveFiberPort_format
  },
  {
    "primaryEthernetPortIsActive", "integer", "1.3.6.1.4.1.1771.3.1.16.1.7.%d"
  },
  {
    "hostTrapCount", "integer32", "1.3.6.1.4.1.1771.3.1.16.1.11.%d"
  },
  {
    "trapTimeout", "integer32", "1.3.6.1.4.1.1771.3.1.16.1.12.%d"
  },
  {
    "trapIpAddress", "ipaddress", "1.3.6.1.4.1.1771.3.1.16.1.15.%d"
  },
  {
    "community", "displaystring", "1.3.6.1.4.1.1771.3.1.16.1.16.%d"
  },
  {
    "commandForBigUc", "integer", "1.3.6.1.4.1.1771.3.1.16.1.17.%d"
  },
  {
    "trapSnmpServer", "ipaddress", "1.3.6.1.4.1.1771.3.1.16.1.20.%d"
  },
  {
    "trapMailAddress", "displaystring", "1.3.6.1.4.1.1771.3.1.16.1.21.%d"
  },
  {
    "trapMailPeriod", "integer", "1.3.6.1.4.1.1771.3.1.16.1.22.%d"
  },
  {
    "forceLed", "integer", "1.3.6.1.4.1.1771.3.1.19.1.3.%d"
  },
  {
    "controlForLanPort", "integer", "1.3.6.1.4.1.1771.3.1.13.1.13.%d"
  },
  {
    "controlForGePort", "integer", "1.3.6.1.4.1.1771.3.1.23.1.4.%d"
  }                         
};

#define num_types 7
char*possible_types [num_types] =
{
  "displaystring",
  "ipaddress",
  "macaddress",
  "counter32",
  "counter64",
  "integer32",
  "integer"
};

char*TableLength_names [] =
{
  "tableID",
  "tableLength" 
};
char*TrapMask_names [] =
{
  "trapMaskIDForTrapMask",
  "trapEnabledForTrapMask" 
};
char*Part_names [] =
{
  "partID",
  "partType",
  "typeID"
};
char*Uc_names [] =
{
  "ucID",
  "firstPartID",
  "lastPartID",
  "state",
  "overTemperatureCount",
  "resetCount",
  "firmwareFaultCount",
  "serialNumber",
  "engineeringDateCode",
  "firmwareVersion",
  "monitorFirmwareVersion",
  "scratchPad",
  "reboot",
  "monitorCause",
  "firstCoreDumpID",
  "lastCoreDumpID"
};
char*Linecard_names [] =
{
  "linecardID",
  "voltageAForLinecard",
  "voltageBForLinecard",
  "shutdownForLinecard"
};
char*Backplane_names [] =
{
  "backplaneID",
  "shutdownForBackplane"  
};
char*BackplaneMaster_names [] =
{
  "backplaneMasterID",
  "voltageAForBackplaneMaster",
  "voltageBForBackplaneMaster",
  "shutdownForBackplaneMaster",
  "backplaneStatus"
};
char*Fan_names [] =
{
  "fanID",
  "fanSpeed",
  "minimumFanSpeed",
  "fanBad",
  "fanBadCount",
  "fanTrayMissing",
  "fanTrayMissingCount"
};
char*PowerSupply_names [] =
{
  "powerSupplyID",
  "voltageForPowerSupply",
  "acBad",
  "acBadCount",
  "dcBad",
  "dcBadCount"
};
char*Xbar16_names [] =
{
  "xbar16ID",
  "firstXbarPort",
  "lastXbarPort",
  "timeoutForXbar16",
  "selfTestResultForXbar16"
};
char*XbarPort_names [] =
{
  "xbarPortID",
  "selfTestResultForXbarPort",
  "invalidRoutes",
  "illegalSymbols",
  "missedBeats",
  "deadPorts",
  "goodCrcs",
  "badCrcs",
  "dataIn",
  "dataOut",
  "portDownOnIllegalSymbol",
  "portDownOnIllegalSymbolCount",
  "portDownOnMissedBeat",
  "portDownOnMissedBeatCount",
  "portDown",
  "portUpCount",
  "receiveTimeout",
  "receiveTimeoutCount",
  "transmitTimeout",
  "transmitTimeoutCount",
  "controlForXbarPort"
};
char*SanPort_names [] =
{
  "sanPortID"
};
char*LanPort_names [] =
{
  "lanPortID",
  "missedBeatForLanPortCount",
  "illegalSymbolOnLanCount",
  "illegalSymbolOnSanCount",
  "portUpForLanPortCount",
  "portDownForLanPortCount",
  "portFast",
  "noCable",
  "missedBeatForLanPort",
  "illegalOnLan",
  "illegalOnSan",
  "portOKForLanPort",
  "controlForLanPort"
};
char*HssdcPort_names [] =
{
  "hssdcPortID",
  "overrunsForHssdcPort",
  "sanFaultForHsddcPort",
  "sanFaultCountForHsddcPort",
  "serdesFaultForHssdcPort",
  "serdesFaultCountForHssdcPort",
  "notInSyncForHssdcPort",
  "notInSyncCountForHssdcPort",
  "selfTestResultForHssdcPort",
  "controlForHssdcPort"
};
char*ShortwaveFiberPort_names [] =
{
  "shortwaveFiberPortID",
  "overrunsForShortwaveFiberPort",
  "sanFaultForShortwaveFiberPort",
  "sanFaultCountForShortwaveFiberPort",
  "serdesFaultForShortwaveFiberPort",
  "serdesFaultCountForShortwaveFiberPort",
  "notInSyncForShortwaveFiberPort",
  "notInSyncCountForShortwaveFiberPort",
  "signalLost",
  "signalLostCount",
  "selfTestResultForShortwaveFiberPort",
  "controlForShortwaveFiberPort"
};
char*BigUc_names [] =
{
  "bigUcID",
  "backplaneType",
  "versionForBigUc",
  "ethernetAddress",
  "primaryEthernetPortState",
  "secondaryEthernetPortState",
  "primaryEthernetPortIsActive",
  "ipAddress",
  "upTimeForBigUc",
  "switchTrapCount",
  "hostTrapCount",
  "trapTimeout",
  "trapsDropped",
  "trapsSent",
  "trapIpAddress",
  "community",
  "commandForBigUc",
  "slotChangeCount",
  "refreshes",
  "trapSmtpServer",
  "trapMailAddress",
  "trapMailPeriod"
};
char*Voltage_names [] =
{
  "voltageID",
  "voltageType",
  "voltage",
  "nominal"
};
char*Temperature_names [] =
{
  "temperatureID",
  "selfTestResultForTemperature",
  "temperature"
};
char*OperatingLed_names [] =
{
  "operatingLedID",
  "ledState",
  "forceLed"
};
char*CoreDump_names [] =
{
  "coreDumpID",
  "pageLength",
  "page"
};
char*Slot_names [] =
{
  "slotID",
  "ucIDForSlot",
  "serialNumberForSlot",
  "lostCommunicationCount",
  "reestablishCommunicationCount",
  "changeCount",
  "slotState"
};

char*Current_names [] =
{
  "currentID",
  "overcurrentFault",
  "currentValue"
};

char*GePort_names [] =
{
  "gePortID",
  "stateForGEPort",
  "rebootCountForGePort"
  "controlForGePort"
};

void dump_object (Snmp*snmp, CTarget*target, int object, int id);
void dump_Part (Snmp*snmp, CTarget*target, int object, int id);
void dump_Slot (Snmp*snmp, CTarget*target, int object, int id);
void dump_Uc (Snmp*snmp, CTarget*target, int object, int id);
void dump_Core (Snmp*snmp, CTarget*target, int object, int id);

object_thing objects [] =
{
  {
    "TableLength", TableLength_names,sizeof(TableLength_names)/sizeof(TableLength_names[0]), dump_object
  },
  {
    "TrapMask", TrapMask_names,sizeof(TrapMask_names)/sizeof(TrapMask_names[0]), dump_object
  },
  {
    "Part", Part_names,sizeof(Part_names)/sizeof(Part_names[0]), dump_Part
  },
  {
    "Uc", Uc_names,sizeof(Uc_names)/sizeof(Uc_names[0]), dump_Uc
  },
  {
    "Linecard", Linecard_names,sizeof(Linecard_names)/sizeof(Linecard_names[0]), dump_object
  },
  {
    "Backplane", Backplane_names,sizeof(Backplane_names)/sizeof(Backplane_names[0]), dump_object
  },
  {
    "BackplaneMaster", BackplaneMaster_names,sizeof(BackplaneMaster_names)/sizeof(BackplaneMaster_names[0]), dump_object
  },
  {
    "Fan", Fan_names,sizeof(Fan_names)/sizeof(Fan_names[0]), dump_object
  },
  {
    "PowerSupply", PowerSupply_names,sizeof(PowerSupply_names)/sizeof(PowerSupply_names[0]), dump_object
  },
  {
    "Xbar16", Xbar16_names,sizeof(Xbar16_names)/sizeof(Xbar16_names[0]), dump_object
  },
  {
    "XbarPort", XbarPort_names,sizeof(XbarPort_names)/sizeof(XbarPort_names[0]), dump_object
  },
  {
    "SanPort", SanPort_names,sizeof(SanPort_names)/sizeof(SanPort_names[0]), dump_object
  },
  {
    "LanPort", LanPort_names,sizeof(LanPort_names)/sizeof(LanPort_names[0]), dump_object
  },
  {
    "HssdcPort", HssdcPort_names,sizeof(HssdcPort_names)/sizeof(HssdcPort_names[0]), dump_object
  },
  {
    "ShortwaveFiberPort", ShortwaveFiberPort_names,sizeof(ShortwaveFiberPort_names)/sizeof(ShortwaveFiberPort_names[0]), dump_object
  },
  {
    "BigUc", BigUc_names,sizeof(BigUc_names)/sizeof(BigUc_names[0]), dump_object
  },
  {
    "Voltage", Voltage_names,sizeof(Voltage_names)/sizeof(Voltage_names[0]), dump_object
  },
  {
    "Temperature", Temperature_names,sizeof(Temperature_names)/sizeof(Temperature_names[0]), dump_object
  },
  {
    "OperatingLed", OperatingLed_names,sizeof(OperatingLed_names)/sizeof(OperatingLed_names[0]), dump_object
  },
  {
    "CoreDump", CoreDump_names,sizeof(CoreDump_names)/sizeof(CoreDump_names[0]), dump_Core
  },
  {
    "Slot", Slot_names, sizeof(Slot_names)/sizeof(Slot_names[0]), dump_Slot
  },
  {
    "Current", Current_names, sizeof(Current_names)/sizeof(Current_names[0]), dump_object
  },
  {
    "GePort", GePort_names, sizeof(GePort_names)/sizeof(GePort_names[0]), dump_object
  }
};

#define num_objects (sizeof (objects) / sizeof (objects [0]))
#define num_controls (sizeof (controls) / sizeof (controls [0]))
#define num_masks (sizeof (masks) / sizeof (masks [0]))
#define num_traps (sizeof (m3_trap_names) / sizeof (m3_trap_names [0]))

static void print_types ()
{
  for (int i = 0; i < num_types; i++)
    printf ("  %s\n", possible_types [i]);
}

static void print_controls ()
{
  for (int i = 0; i < num_controls; i++)
    printf ("%s\n", controls [i].name);
}
static void print_traps ()
{
  for (int i = 0; i < num_traps; i++)
    printf ("%s\n", m3_trap_names [i]);
}
static void print_masks ()
{
  for (int i = 0; i < num_masks; i++)
    printf ("%s %02.2x\n", masks [i].name, masks [i].mask);
}
static void print_objects ()
{
  for (int i = 0; i < num_objects; i++)
    printf ("%s\n", objects [i].name);
}

int make_mask (int argc, char*argv[], int*mask)
{
  int i;
  *mask = 0;
  
  for (i = 0; i < argc; i++)
  {
    if (*argv [i] == '-')
      break;
    
    int j;
    for (j = 0; j < num_masks; j++)
      if (!strcmp (masks [j].name, argv [i]))
	break;

    int m = j == num_masks ? make_number (argv [0]) : masks [j].mask;
    *mask |= m;
  }
  return i;
}

static void usage (char*s)
{
  if (s)
    printf ("unknown option %s\n", s);
  
  printf ("usage:\n");
  printf ("snmp_port <host> <community>\n");
  printf (" -trap [-dump]\n");
  printf (" -dump\n");

  printf (" -dump <object> <id>\n");
  printf (" -peek <object> <id>\n");
  printf ("  possible object:\n");
  print_objects ();
  
  printf (" -get <oid>\n");
  printf (" -get <control> <id>\n");
  printf (" -set <control> <id> <bitname bitname ... > \n");
  printf (" -set <oid> <type> <bitname bitname ... > \n");
  printf ("  possible controls:\n");
  print_controls ();
  printf ("  possible bitnames (also use positive numbers):\n");
  print_masks ();
  
  printf ("\npossible types:\n");
  print_types ();
  
  printf ("-count-traps [trap trap trap ...]\n");
  printf ("-enable-traps [trap trap ...]\n");
  printf ("-disable-traps [trap trap ...]\n");
  printf ("  possible traps:\n");
  print_traps ();
  
  exit (1);
}

char oid [255];

int getOid (Snmp*snmp, CTarget*target, char*oid)
{
  Pdu pdu;
  Vb vb;
  int status;  

  //printf ("getOid %s\n", oid);

  vb.set_oid (oid);
  pdu += vb;

  if ((status = snmp->get (pdu, *target)) != SNMP_CLASS_SUCCESS)
    die ((snmp->error_msg (status)));
      
  pdu.get_vb (vb, 0);
  
  printf ("%s\n", vb.get_printable_value ());
  return 1;
}

int getTrapCount (Snmp*snmp, CTarget*target, int trap)
{
  char s [255];

  sprintf (s, "%s.1.%d.1.3.%d", M3_OID, M3_TRAP_MASK_TABLE, trap + 1);
  printf ("%s ", m3_trap_names [trap]);
  return getOid (snmp, target, s);
}

int setOid (Snmp*snmp, CTarget*target, char*oid, char*type, char*p)
{
  Pdu pdu;
  Vb vb;
  int status;  

  //printf ("setOid %s (%s)\n", oid, p);
  
  vb.set_oid (oid);

  if (!strcmp (type, "displaystring"))
    vb.set_value (p);
  else if (!strcmp (type, "ipaddress"))
    vb.set_value (IpAddress (p));
  else if (!strcmp (type, "macaddress"))
    vb.set_value (MacAddress (p));
  else if (!strcmp (type, "counter32"))
    vb.set_value (Counter32 (make_number (p)));
  else if (!strcmp (type, "counter64"))
    vb.set_value (Counter64 (make_number (p)));
  else if (!strcmp (type, "integer32"))
    vb.set_value ((long) make_number (p));
  else if (!strcmp (type, "integer"))
    vb.set_value ((int) make_number (p));
  else die (("unknown type %s", type));
  
  pdu += vb;

  if ((status = snmp->set (pdu, *target)) != SNMP_CLASS_SUCCESS)
    die ((snmp->error_msg (status)));

  /*  if ((status = snmp->get (pdu, *target)) != SNMP_CLASS_SUCCESS)
      die ((snmp->error_msg (status)));

      pdu.get_vb (vb, 0);
  */
  //printf ("set %s to: %s\n", oid, vb.get_printable_value ());
  return 1;
}

int setTrapEnabled (Snmp*snmp, CTarget*target, int trap, int enabled)
{
  char s [255];
  sprintf (s, "%s.1.%d.1.2.%d", M3_OID, M3_TRAP_MASK_TABLE, trap + 1);
  return setOid (snmp, target, s, "integer", make_value (enabled));

}

int getTableLength (Snmp*snmp, CTarget*target, int table)
{
  Pdu pdu;
  Vb vb;
  int status;
  long int value;
  
  sprintf (oid, "%s.1.%d.1.2.%d", M3_OID, M3_TABLE_LENGTH_TABLE, table);

  vb.set_oid (oid);
  pdu += vb;

  if ((status = snmp->get (pdu, *target)) != SNMP_CLASS_SUCCESS)
    die ((snmp->error_msg (status)));
  
  pdu.get_vblist (&vb, 1);
  vb.get_value (value);

  //printf ("length of table %d is %d\n", table, value);
  
  return (int) value;
}

static int dumpTraps;
static OidCollection oids;
static TargetCollection targets;
static int insideCallback;

void trapCallback (int reason, Snmp*snmp, Pdu&pdu, SnmpTarget &target, void*cbd)
{
  GenAddress address;
  Oid trapid;
  Vb vb;
  TimeTicks timestamp;
  CTarget*ctarget = (CTarget*)cbd;
  long part;
  int status;
  
  if (insideCallback || reason != SNMP_CLASS_NOTIFICATION)
    return;
  
  insideCallback = 1;
  
  pdu.get_notify_id (trapid);
  pdu.get_notify_timestamp (timestamp);
  target.get_address( address);

  printf ("%s\n", m3_trap_names [last_number (trapid.get_printable ()) - 1]);
  printf ("from %s\ntime %s\n", address.get_printable (), timestamp.get_printable ());

  part = 0;
  
  for (int i = 0; i < pdu.get_vb_count (); i++)
  {
    int variable;
    
    pdu.get_vb (vb, i);
    printf ("%s is %s\n", m3_trap_variables [variable = last_number (vb.get_printable_oid ()) - 1], vb.get_printable_value ());

    if (!strcmp (m3_trap_variables [variable], "partID"))
      vb.get_value (part);
  }

  if (dumpTraps && part)
    dump_Part (snmp, ctarget, M3_PART_TABLE - 1, part);
  
  printf ("\n");

  for (int j = 0; j < num_controls; j++)
  {
    if (!strcmp (controls [j].name, "hostTrapCount"))
    {
      pdu.get_vb (vb, 0);
      long value;
      vb.get_value (value);

      setOid (snmp, ctarget, make_oid (controls [j].oid, 1), controls [j].type, make_value (value));
      break;
    }
  }
  insideCallback = 0;
}

void dump_object (Snmp*snmp, CTarget*target, int object, int id)
{
  Pdu pdu;
  Vb vb [objects [object].num_names];
  char s [255];
  int status;

  for (int i = 0; i < objects [object].num_names; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, object + 1, i + 1, id);
    //printf ("oid is %s\n", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = snmp->get (pdu, *target)) != SNMP_CLASS_SUCCESS)
    die ((snmp->error_msg (status)));
  
  pdu.get_vblist (vb, objects [object].num_names);

  printf ("%s%s\n", indents [indent_level], objects [object].name);
  indent_level++;

  for (int i = 0; i < objects [object].num_names; i++)
    printf ("%s%s: %s\n", indents [indent_level], objects [object].names [i], vb [i].get_printable_value ());
  indent_level--;
}


void dump_Part (Snmp*snmp, CTarget*target, int object, int id)
{
  Pdu pdu;
  Vb vb [objects [object].num_names];
  char s [255];
  int status;

  for (int i = 0; i < objects [object].num_names; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, object + 1, i + 1, id);
    //printf ("oid is %s\n", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = snmp->get (pdu, *target)) != SNMP_CLASS_SUCCESS)
    die ((snmp->error_msg (status)));
  
  pdu.get_vblist (vb, objects [object].num_names);

  long type;
  long typeID;
      
  vb[1].get_value (type);
  vb[2].get_value (typeID);  

  if (type < 1 || type > M3_NUM_TABLES)
  {
    printf ("error in snmp data (bad type)\n");
    return;
  }
  
  objects [type - 1].dump (snmp, target, type - 1, typeID);
}

void dump_Slot (Snmp*snmp, CTarget*target, int object, int id)
{
  Pdu pdu;
  Vb vb [objects [object].num_names];
  char s [255];
  int status;

  for (int i = 0; i < objects [object].num_names; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, object + 1, i + 1, id);
    //printf ("oid is %s\n", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = snmp->get (pdu, *target)) != SNMP_CLASS_SUCCESS)
    die ((snmp->error_msg (status)));
  
  pdu.get_vblist (vb, objects [object].num_names);

  long ucID;
  vb[1].get_value (ucID);  

  if (!ucID)
    return;
  
  printf ("%s%s\n", indents [indent_level], objects [object].name);
  indent_level++;

  for (int i = 0; i < objects [object].num_names; i++)
    printf ("%s%s: %s\n", indents [indent_level], objects [object].names [i], vb [i].get_printable_value ());
  

  objects [M3_UC_TABLE - 1].dump (snmp, target, M3_UC_TABLE - 1, ucID);
  
  indent_level--;
}


void dump_Core (Snmp*snmp, CTarget*target, int object, int id)
{
}

void dump_Uc (Snmp*snmp, CTarget*target, int object, int id)
{
  Pdu pdu;
  Vb vb [objects [object].num_names];
  char s [255];
  int status;

  for (int i = 0; i < objects [object].num_names; i++)
  {
    sprintf (s, "%s.1.%d.1.%d.%d", M3_OID, object + 1, i + 1, id);
    //printf ("oid is %s\n", s);
    vb [i].set_oid (s);
    pdu += vb [i];
  }

  if ((status = snmp->get (pdu, *target)) != SNMP_CLASS_SUCCESS)
    die ((snmp->error_msg (status)));
  
  pdu.get_vblist (vb, objects [object].num_names);

  long firstPartID, lastPartID, firstCoreDumpID, lastCoreDumpID;
      
  vb[1].get_value (firstPartID);
  vb[2].get_value (lastPartID);
  vb[14].get_value (firstCoreDumpID);
  vb[15].get_value (lastCoreDumpID);
  
  printf ("%s%s\n", indents [indent_level], objects [object].name);
  indent_level++;

  for (int i = 0; i < objects [object].num_names; i++)
    printf ("%s%s: %s\n", indents [indent_level], objects [object].names [i], vb [i].get_printable_value ());

  for (int i = firstPartID; i <= lastPartID; i++)
    objects [M3_PART_TABLE - 1].dump (snmp, target, M3_PART_TABLE - 1, i);
  
  for (int i = firstCoreDumpID; i <= lastCoreDumpID; i++)
    objects [M3_CORE_DUMP_TABLE - 1].dump (snmp, target, M3_CORE_DUMP_TABLE - 1, i);
  
  indent_level--;
}

void dump (Snmp*snmp, CTarget*target)
{
  int num_slots = getTableLength (snmp, target, M3_SLOT_TABLE);
  for (int i = 0; i < num_slots; i++)
    objects [M3_SLOT_TABLE - 1].dump (snmp, target, M3_SLOT_TABLE - 1, i + 1);
}

int main (int argc, char*argv [])
{
  int status;
    
  if (argc < 4)
    usage (0);

  char*host = argv [1];
  char*community = argv [2];
  
  IpAddress address (host);
  CTarget target (address, community, community);
  target.set_version (version2c);
  
  int maxfds;
  fd_set readfds, writefds, exceptfds;

  targets += target;
  
  target.set_timeout (1000*10);
  
  Snmp snmp (status);
  
  if (status != SNMP_CLASS_SUCCESS)
    die ((snmp.error_msg (status)));

  for (int i = 3; i < argc; i++)
  {
    if (!strcmp (argv [i], "-dump") || !strcmp (argv [i], "-peek"))
    {
      char*name = 0;
      int peek = !strcmp (argv [i], "-peek");
      
      if (++i == argc || *argv [i] == '-')
      {
	dump (&snmp, &target);
	i--;
	continue;
      }

      name = argv [i];
      int j;
	
      for (j = 0; j < num_objects; j++)
	if (!strcmp (objects [j].name, name))
	  break;
	
      if (j == num_objects)
	die (("%s is not a good object name", name));
	
      if (++i == argc)
	die (("expected id"));

      if (peek)
	dump_object (&snmp, &target, j, make_number (argv[i]));
      else
	objects[j].dump (&snmp, &target, j, make_number (argv[i]));
    }
    else if (!strcmp (argv [i], "-get"))
    {
      char*name;
      int id;
      
      if (++i == argc)
	die (("expected name"));
      
      name = argv [i];
      
      if (++i == argc || *argv [i] == '-')
      {
	getOid (&snmp, &target, name);
	continue;
      }
      
      id = make_number (argv [i]);
      
      int j;
      
      for (j = 0; j < num_controls; j++)
	if (!strcmp (controls [j].name, name))
	  break;
      
      if (j == num_controls)
	die (("%s is not an known controls name", name));
      
      getOid (&snmp, &target, make_oid (controls [j].oid, id));
    }
    else if (!strcmp (argv [i], "-set"))
    {
      char*name;
      char*type;
      int id = 0;
      
      if (++i == argc)
	die (("expected name"));
      name = argv [i];
      
      if (i + 1 == argc)
	die (("expected something more"));
      
      int j;
      for (j = 0; j < num_controls; j++)
	if (!strcmp (controls [j].name, name))
	  break;
      
      if (j != num_controls)
      {
	if (++i == argc)
	  die (("expected id"));      
	id = make_number (argv [i]);
	if (id == 0)
	  die (("zero id"));
	type = controls [j].type;
      }
      else
      {
	if (++i == argc)
	  die (("expected type"));      
	type = argv [i];
	
	int k;
	for (k = 0; k < num_controls; k++)
	  if (!strcmp (controls [k].type, type))
	    break;
	if (k == num_controls)
	  die (("%s is an unknown type", type));
      }
      
      char*value;

      if (maskable (type))
      {  
	int mask;
	i += make_mask (argc - i - 1, argv + i + 1, &mask);
	value = make_value (mask);
      }
      else value = argv [++i];

      setOid (&snmp, &target, id ? make_oid (controls [j].oid, id) : name, type, value);
    }
    else if (!strcmp (argv [i], "-trap"))
    {
      dumpTraps = i < argc - 1 && !strcmp (argv [++i], "-dump");
      
      status = snmp.notify_register (oids, targets, &trapCallback, &target);
      if (status != SNMP_CLASS_SUCCESS)
	die (("notify_register failed with %s", snmp.error_msg (status)));
      
      SNMPGetFdSets (maxfds, readfds, writefds, exceptfds);
      for (;;)
      {
	select (maxfds, &readfds, &writefds, &exceptfds, NULL);
	SNMPProcessPendingEvents ();
      }
    }
    else if (!strcmp (argv [i], "-enable-traps")|| !strcmp (argv [i], "-disable-traps") || !strcmp (argv [i], "-count-traps"))
    {
      int enable = !strcmp (argv [i], "-enable-traps");
      int count = !strcmp (argv [i], "-count-traps");
      int some = 0;

      for (i++; i < argc; i++)
      {
	int j;
	for (j = 0; j < num_traps; j++)
	  if (!strcmp (m3_trap_names [j], argv [i]))
	    break;
	if (j == num_traps)
	  break;
	
	some = 1;
	if (count)
	  getTrapCount (&snmp, &target, j);
	else
	  setTrapEnabled (&snmp, &target, j, enable);
      }
      if (!some)
      { 
	int k;
	for (k = 0; k < M3_NUM_TRAPS; k++)
	{
	  if (count)
	    getTrapCount (&snmp, &target, k);
	  else
	    setTrapEnabled (&snmp, &target, k, enable);
	}
      }
    }
    else usage (argv [i]);
  }
}
