/*
  find_proxy.c
  very evil thing to forward m3 swtich traps on crippled networks.
  finucane@myri.com (David Finucane)
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

#include <sys/time.h>
#include <sys/types.h>


#include "m3_TestClient.h"
#include "m3_RollcallClient.h"
#include "m3_defines.h"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static m3_TestClient*client = 0;
static void signal_handler (int)
{
  if (client)
    client->signaled = 1;
}

static void usage (char*s)
{
  printf ("usage: %s <host> <port> <low_type_bits> <high_type_bits> [-v] [switch names]\n", s);
  exit (1);
}

int main (int argc, char**argv)
{
  m3_Component::initialize (printLine);
  m3_RollcallClient rollcall;
  int*switches = 0;
  int verbose = 0;
  int numSwitches = 0;
  char**switchNames = 0;
  int types [2];
  int host;
  int port;
  
  if (argc < 5)
    usage (argv [0]);
 
  client = new m3_TestClient ();

  if (argc == 5)
  {
    rollcall.open ();
    numSwitches = rollcall.findAddresses (&switches);
    rollcall.close ();
  }
  else
  {
    if (!strcmp (argv [5], "-v"))
      verbose = 1;
    
    numSwitches = argc - 5 - verbose;
    switchNames = argv + 5 + verbose;
  }
  printf ("%d m3 switches\n", numSwitches);


  types [0] = types [1] = 0;
  sscanf (argv [3], "%i", &types [0]);
  sscanf (argv [4], "%i", &types [1]);  

  char hostname [256];
  struct hostent*hp;
  struct sockaddr_in sa;
  sa.sin_family = AF_INET;
  sa.sin_port = htons (atoi (argv[2]));

  if (!(hp = gethostbyname (argv [1])))
  {
    printf ("couldn't get ip address for %s\n", argv [1]);
    return 1;
  }
  memcpy ((char*)&sa.sin_addr.s_addr, (char*)hp->h_addr, 4);

  int notifyFd;
  int option;
  option = 1;

  notifyFd = socket (AF_INET, SOCK_DGRAM, 0);
  if (notifyFd <= 0)
  {
    perror ("socket failed for notifyFd");
    return 1;
  }
  setsockopt (notifyFd, SOL_SOCKET, SO_REUSEADDR, (char*) &option, sizeof (option));

  printf ("forwarding traps to %s:%s\n", argv [1], argv [2]);
  
  m3_notify_message_t m;
  int length;
  struct sockaddr_in a;
  int fd;
  fd = socket (AF_INET, SOCK_DGRAM, 0);
  if (fd < 0)
  {
    perror ("failed opening datagram socket");
    return 1;
  }
  memset ((char*)&a, 0, sizeof (a));

  a.sin_family = AF_INET;
  a.sin_addr.s_addr = INADDR_ANY;
  a.sin_port = 0;
      
  if (bind (fd, (struct sockaddr*) &a, sizeof (a)) < 0)
  {
    perror ("binding datagram socket failed");
    return 1;
  }
      
  length = sizeof (a);
  if (getsockname (fd, (struct sockaddr*) &a, (socklen_t*) &length) < 0)
  {
    perror ("getsockname failed");
    return 1;
  }
      
  if (gethostname (hostname, 255) < 0 || !(hp = gethostbyname (hostname)))
  {
    printf ("couldn't get our ip address\n");
    return 1;
  }
      
  printf ("using udp port %d\n", ntohs (a.sin_port));

  for (int i = 0; i < numSwitches; i++)
  {
    if (switches && !client->open (switches [i]) || switchNames && !client->open (switchNames [i]))
      return 1;
  
    if (!client->setNotify ((char*) hp->h_addr, ntohs (a.sin_port), types [0], types [1]))
      return 1;
  }

  if (!types [0] && !types [1])
  {
    printf ("just disabling notify.\n");
    exit (0);
  }
  else
  {
    printf ("listening for types 0x%x 0x%x\n", types [0], types [1]);
  }
  
  for (;;)
  {
    fd_set mask;
    struct timeval timeout;
    timeout.tv_sec = 0;
    timeout.tv_usec = 1000 * 10;
      
    FD_ZERO (&mask);
    FD_SET (fd, &mask);

    if (select (1 + fd, &mask, 0, 0, &timeout) == 1)
    {
      length = sizeof (a);      
      if (recvfrom (fd, (char*) &m, sizeof (m), 0, (struct sockaddr*) &a, (socklen_t*) &length) == sizeof (m))
      {
	sendto (notifyFd, (char*) &m, sizeof (m), MSG_DONTWAIT, (struct sockaddr*) &sa, sizeof (sa));

	if (verbose)
	{
	  printf ("%d.%d.%d.%d ",
		  (unsigned) (((char*)(&a.sin_addr.s_addr)) [0] & 0xff),
		  (unsigned) (((char*)(&a.sin_addr.s_addr)) [1] & 0xff),
		  (unsigned) (((char*)(&a.sin_addr.s_addr)) [2] & 0xff),
		  (unsigned) (((char*)(&a.sin_addr.s_addr)) [3] & 0xff));

	  printf ("%s (%d) low: %d high: %d %s %d slot %d\n",
		  htonl (m.type) > 0 && htonl (m.type) <= M3_NUM_TRAPS ? 
		  m3_trap_names [htonl (m.type) - 1] : "?", 
		  htonl (m.type),
		  htonl (m.low), htonl (m.high), 
		  htonl (m.partType) > 0 && htonl (m.partType) <= M3_NUM_TABLES ? 
		  m3_table_names [htonl (m.partType) - 1] : "?",
		  htonl (m.index),
		  htonl (m.slot));
	}
	
      }
    }
  }    
}
