#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "mt_Graph.h"
#include "mt_MapFile.h"

#define die() {printf ("died on %s:%d\n", __FILE__, __LINE__); exit (1);}
#define insist(e) do { if (!(e)) {printf ("assertion failed. %s:%d (%s)", __FILE__,__LINE__, #e); goto exception;}} while (0)
#define MU_XBAR_TABLE_PRINTF_STRING "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%d:%d"

static void printLine (char*s)
{
  printf ("%s\n", s);
}

static void usage ()
{
  printf ("usage: xbars4mute <mapfile> <host>\n");
  exit (1);
}


int main (int argc, char*argv [])
{
  mt_Component::initialize (printLine);

  char*mapfile = argv [1];
  char*hostname = argv [2];
  
  mt_Graph graph;
  mt_Node*root;
  
  if (argc != 3)
    usage ();

  mt_MapFile mf (mapfile, mt_File::_READ);

  if (!mf.read (&graph))
  {
    printf ("couldn't read \"%s\"\n", mapfile);
    return 0;
  }
  
  if (!(root = graph.getNode (hostname)))
  {
    printf ("node \"%s\" is not in \"%s\"\n", hostname, mapfile);
    return 0;
  }
  
  if (!graph.computeRoutes (root))
  {  
    printf ("couldn't compute routes\n");
    return 0;
  }
  
  int x, c;
  char s [mt_Node::NAME_LENGTH];
  int e [6];

  while (fscanf (stdin,  "%s %x:%x:%x:%x:%x:%x:%d:%d", s, e+0, e+1, e+2, e+3, e+4, e+5, &x, &c) == 9)
  {
    mt_Node*n;
    mt_Route*r;
    
    if (!(n = graph.getNode (s)))
    {
      printf ("%s is not in %s", s, mapfile);
      return 0;
    }
    r = n->getRoute (0);
    insist (r);

    printf (MU_XBAR_TABLE_PRINTF_STRING "\n",
	    (unsigned char) e [0] & 0xff,
	    (unsigned char) e [1] & 0xff,
	    (unsigned char) e [2] & 0xff,
	    (unsigned char) e [3] & 0xff,
	    (unsigned char) e [4] & 0xff,
	    (unsigned char) e [5] & 0xff,
	    x,
	    c);

    printf ("%s\n", r->toString ());

  }
  return 0;
  exception: return 1;
}
